import React, { Component } from 'react';
import { connect } from 'react-redux';
import { loadAccounts } from '../actions/app';
import { tu } from '../utils/i18n';
import { injectIntl } from 'react-intl';
import { filter } from 'lodash';
import { AddressLink } from './common/Links';
import { ONE_UNIT } from '../constants';
import { TRXPrice } from './common/Price';
import SmartTable from './common/SmartTable.js'
import { upperFirst } from 'lodash';
import { TronLoader } from './common/loaders';
import { QuestionMark } from './common/QuestionMark';
// import xhr from "axios/index";
import { Client } from '../services/api';
import { Tooltip } from 'antd'

class Accounts extends Component {

  constructor () {
    super();

    this.state = {
      loading: true,
      searchString: '',
      accounts: [],
      total: 0,
    }
  }

  componentDidMount () {
    this.loadAccounts();
  }

  loadAccounts = async (page = 1, pageSize = 20) => {

    this.setState({loading: true});

    const result = await Client.getAccounts({
      limit: pageSize,
      start: (page - 1) * pageSize
    })

    // let {txOverviewStats} = await Client.getTxOverviewStats();
    this.setState({
      loading: false,
      accounts: result.items,
      total: result.total,
      rangeTotal: result.range_total,
    });
  };

  onChange = (page, pageSize) => {
    this.loadAccounts(page, pageSize);
  };

  onSearchFieldChangeHandler = (e) => {
    this.setState({
      searchString: e.target.value,
    });
  };

  filteredAccounts () {
    let {accounts} = this.props;
    let {searchString} = this.state;

    searchString = searchString.toUpperCase();

    if (searchString.length > 0) {
      accounts = filter(accounts, a => a.address.toUpperCase().indexOf(searchString) !== -1);
    }

    return accounts;
  }

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: upperFirst(intl.formatMessage({id: 'address'})),
        dataIndex: 'address',
        key: 'address',
        align: 'left',
        className: 'ant_table',
        width: '40%',
        render: (text, record) => {
          return record.account_type === 2 ? (
              <span className="d-flex">
                <Tooltip placement="top" title={intl.formatMessage({id: 'contracts'})}>
                  <span><i className="far fa-file mr-1"/></span>
                </Tooltip>

                <AddressLink address={text} isContract={record.to_address_type === 2}/>
              </span>
            ) : <AddressLink address={text}/>
        }
      },
      // {
      //   // title: upperFirst(intl.formatMessage({id: 'supply'})),
      //   title: CHAIN_SYMBOL + ' Held',
      //   dataIndex: 'balance',
      //   key: 'supply',
      //   align: 'left',
      //   className: 'ant_table',
      //   // width: '12%',
      //   render: (text) => {
      //     return <div><FormattedNumber
      //       value={(((parseInt(text) / ONE_UNIT) / CIRCULATING_SUPPLY) * 100)}
      //       minimumFractionDigits={8}
      //       maximumFractionDigits={8}
      //     /> %</div>
      //   }
      // },
      // {
      //   title: upperFirst(intl.formatMessage({id: 'power'})),
      //   dataIndex: 'power',
      //   key: 'power',
      //   align: 'center',
      //   // width: '15%',
      //   render: (text) => {
      //     return <FormattedNumber value={parseInt(text) / ONE_UNIT}/>
      //   }
      // },
      {
        title: upperFirst(intl.formatMessage({id: 'balance'})),
        dataIndex: 'balance',
        key: 'balance',
        align: 'right',
        className: 'ant_table',
        // width: '15%',
        render: (text) => {
          return <TRXPrice amount={parseInt(text) / ONE_UNIT}/>
        }
      }
    ];
  }

  render () {
    let {intl} = this.props;
    let {total, loading, rangeTotal = 0, accounts} = this.state;
    let column = this.customizedColumn();
    // let tableInfo = intl.formatMessage({id: 'view_total'}) + ' ' + rangeTotal + ' ' + intl.formatMessage({id: 'account_unit'}) + '<br/>' +  '(' + intl.formatMessage({id: 'table_info_big'}) + ')';
    let tableInfoTip = intl.formatMessage({id: 'table_info_account_tip1'}) + ' ' + rangeTotal + ' ' + intl.formatMessage({id: 'table_info_account_tip2'});
    return (
      <main className="container header-overlap pb-3 token_black">
        {/*<div className="row">*/}
        {/*<div className="col-md-12">*/}
        {/*<div className="card h-100 text-center widget-icon accout_unit">*/}
        {/*/!* <WidgetIcon className="fa fa-users text-secondary"/> *!/*/}
        {/*<div className="card-body">*/}
        {/*<h3 className="text-primary">*/}
        {/*<FormattedNumber value={rangeTotal}/>*/}
        {/*</h3>*/}
        {/*{tu('total_accounts')}*/}
        {/*</div>*/}
        {/*</div>*/}
        {/*</div>*/}
        {/*</div>*/}
        {loading && <div className="loading-style"><TronLoader/></div>}
        <div className="row mt-2">
          <div className="col-md-12 table_pos">
            {total ? <div className="table_pos_info d-none d-md-block" style={{left: 'auto'}}>
              <div>{tu('view_total')} {rangeTotal} {tu('account_unit')} <QuestionMark placement="top"
                                                                                      info={tableInfoTip}/> <br/>
                <span>({tu('table_info_big')})</span></div>
            </div> : ''}
            <SmartTable bordered={true} loading={loading} column={column} data={accounts} total={total}
                        onPageChange={(page, pageSize) => {
                          this.loadAccounts(page, pageSize)
                        }}/>
          </div>
        </div>
      </main>
    )
  }
}

function mapStateToProps (state) {
  return {
    accounts: state.app.accounts,
  };
}

const mapDispatchToProps = {
  loadAccounts,
};


export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(Accounts))
