import React, { Component } from 'react';
import { tu } from '../../utils/i18n';
import { FormattedNumber, injectIntl } from 'react-intl';
import { loadBlocks } from '../../actions/blockchain';
import { connect } from 'react-redux';
import { TronLoader } from '../common/loaders';
import { AddressLink } from '../common/Links';
import TimeAgo from 'react-timeago';
import moment from 'moment';
import { Link } from 'react-router-dom';
import { TRXPrice } from '../common/Price';
import { withTimers } from '../../utils/timing';
import 'react-perfect-scrollbar/dist/css/styles.css';
import PerfectScrollbar from 'react-perfect-scrollbar'

import isMobile from '../../utils/isMobile';
// import { Truncate } from '../common/text';
import { ONE_UNIT } from '../../constants';

class RecentBlocks extends Component {
  _mounted = true;

  constructor () {
    super();
    this.state = {
      blocks: [],
      loading: true
    };
  }

  componentDidMount () {
    this._mounted = true
    this.loopLoadBlocks()
  }

  componentWillUnmount() {
    this._mounted = false
  }

  loopLoadBlocks = () => {
    if (this._mounted) {
      this.props.loadBlocks()
        .finally(() => {
          if (this.state.loading) {
            this.setState({
              loading: false
            })
          }
          setTimeout(() => {
            this.loopLoadBlocks()
          }, 10000)
        })
    }
  }

  render () {
    let {blocks} = this.props;
    if (this.state.loading) {
      return (
        <div className="text-center d-flex justify-content-center">
          <TronLoader/>
        </div>
      );
    }

    return (
      <div className="card shadow-base" style={styles.card}>
        <div className="card-header bg-tron-light d-flex">
          <i className="fa fa-cubes mr-3 fa_width_20 color-card-header"/>
          <h5 className="m-0 lh-175 color-card-header">{tu('blocks')}</h5>
          <Link to="/blockchain/blocks"
                className="ml-auto btn btn-sm btn-default"
                style={{borderRadius: '0.15rem'}}>
            {tu('view_all')}
          </Link>
        </div>
        <ul className="list-group list-group-flush" style={styles.list}>
          {
            isMobile ?
              <PerfectScrollbar>
                {
                  blocks.map(block => (
                    <li key={block.number} className="list-group-item overflow-h mobile-block">
                      <div key={block.number} className="d-flex flex-column">
                        <div className="media-body mb-0 d-flex" style={{paddingTop: 1}}>
                          <div className="text-left">
                            <div className="d-flex justify-content-between align-times-center">
                              <Link className="mr-1 d-flex justify-content-start color-link__primary pt-1 list-item-word"
                                    to={`/block/${block.number}`}>
                                <i className="fa fa-cube mr-2 mt-1 fa_width color-link__primary" style={{fontSize: '1rem'}}/>
                                #{block.number}
                              </Link>
                              <div className="text-muted color-grey-300 small pt-2">
                                <TimeAgo date={block.timestamp}
                                         title={moment(block.timestamp).format('MMM-DD-YYYY HH:mm:ss A')}/>
                              </div>
                            </div>
                            <div>
                              <i className="fas fa-exchange-alt mr-2 color-link__primary"/>
                              {
                                block.total_transactions ? (
                                  <Link
                                    className="color-link__primary"
                                    to={`/blockchain/transactions?block=${block.number}`}
                                  >
                                    <FormattedNumber value={block.total_transactions}/>{' '}
                                    {tu('transactions')}
                                  </Link>
                                ) : (
                                  <span>
                                    <FormattedNumber value={block.total_transactions}/>{' '}
                                    {tu('transactions')}
                                  </span>
                                )
                              }
                            </div>
                            <div className="text-gray-dark break-word d-flex list-item-word small">
                              <span className="mr-2 color-grey-300">{tu('produced_by')}:</span>
                              <AddressLink address={block.witness_address} truncate={false}>
                                <span className="color-link__primary">{block.witness_name || (block.witness_address ? block.witness_address.slice(0, 20) + '...' : '')}</span>
                              </AddressLink>
                            </div>
                            <div className="text-gray-dark break-word color-grey-200 list-item-word"
                                 style={styles.nowrap}>
                              <span className="small color-grey-300 d-inline-block">{tu('block_reward')}:</span>&nbsp;
                              <TRXPrice amount={block.reward_per_blocks / ONE_UNIT}/>
                            </div>

                          </div>
                          <div className="ml-auto text-right d-flex flex-column pt-2">
                          </div>
                        </div>
                      </div>
                    </li>
                  ))
                }
              </PerfectScrollbar>
              :
              <PerfectScrollbar>
                {
                  blocks.map(block => (
                    <li key={block.number} className="list-group-item overflow-h">
                      <div key={block.number} className="d-flex flex-column">
                        <div className="media-body mb-0 d-flex" style={{paddingTop: 1}}>
                          <div className="text-left">
                            <Link className="mr-1 d-flex justify-content-start color-link__primary pt-1 list-item-word"
                                  to={`/block/${block.number}`}>
                              <i className="fa fa-cube mr-2 mt-1 fa_width color-link__primary" style={{fontSize: '1rem'}}/>
                              #{block.number}
                            </Link>
                            <div>
                              <i className="fas fa-exchange-alt mr-2 color-link__primary"/>
                              {
                                block.total_transactions ? (
                                  <Link
                                    className="color-link__primary"
                                    to={`/blockchain/transactions?block=${block.number}`}
                                  >
                                    <FormattedNumber value={block.total_transactions}/>{' '}
                                    {tu('transactions')}
                                  </Link>
                                ) : (
                                  <span>
                                    <FormattedNumber value={block.total_transactions}/>{' '}
                                    {tu('transactions')}
                                  </span>
                                )
                              }
                            </div>
                            <div className="text-gray-dark break-word d-flex list-item-word font-sm">
                              <span className="mr-2 color-grey-300">{tu('produced_by')}:</span>
                              <AddressLink address={block.witness_address} truncate={false}>
                                <span className="color-link__primary">{block.witness_name || (block.witness_address ? block.witness_address.slice(0, 12) + '...' : '')}</span>
                              </AddressLink>
                            </div>

                          </div>
                          <div className="ml-auto text-right d-flex flex-column pt-2">
                            <div className="text-gray-dark break-word color-grey-200 list-item-word"
                                 style={styles.nowrap}>
                              <span className="d-inline-block">{tu('block_reward')}:</span>&nbsp;
                              <TRXPrice amount={block.reward_per_blocks / ONE_UNIT}/>
                            </div>
                            <div className="text-muted color-grey-300 small" style={styles.nowrap}>
                              <TimeAgo date={block.timestamp}
                                       title={moment(block.timestamp).format('MMM-DD-YYYY HH:mm:ss A')}/>

                            </div>
                          </div>
                        </div>
                      </div>
                    </li>
                  ))
                }
              </PerfectScrollbar>
          }

        </ul>
      </div>
    )
  }
}


function mapStateToProps (state) {
  return {
    blocks: state.blockchain.blocks,
    activeLanguage: state.app.activeLanguage,
    isRightText: state.app.isRightText
  };
}

const mapDispatchToProps = {
  loadBlocks,
};

export default connect(mapStateToProps, mapDispatchToProps)(withTimers(injectIntl(RecentBlocks)))

const styles = {
  list: {
    overflowX: 'none',
    height: 594,
  },
  card: {
    border: 'none',
    borderRadius: 0
  },
  nowrap: {
    flex: 1,
    whiteSpace: 'nowrap'
  }
};
