import React, { Component } from 'react';
import { tu } from '../../utils/i18n';
import { injectIntl } from 'react-intl';
import { loadTransactions } from '../../actions/blockchain';
import { connect } from 'react-redux';
import { TronLoader } from '../common/loaders';
import { Truncate } from '../common/text';
import { AddressLink, TransactionHashLink } from '../common/Links';
import TimeAgo from 'react-timeago';
import moment from 'moment';
import { Link } from 'react-router-dom';
import { withTimers } from '../../utils/timing';
import 'react-perfect-scrollbar/dist/css/styles.css';
import PerfectScrollbar from 'react-perfect-scrollbar'
import { NameWithId } from '../common/names';
import isMobile from '../../utils/isMobile';

class RecentTransfers extends Component {
  _mounted = true;

  constructor () {
    super();
    this.state = {
      transfers: [],
      loading: true
    };
  }

  componentDidMount () {
    this._mounted = true
    this.loopLoadTransactions()
  }

  componentWillUnmount() {
    this._mounted = false
  }

  loopLoadTransactions = () => {
    if (this._mounted) {
      this.props.loadTransactions()
        .finally(() => {
          if (this.state.loading) {
            this.setState({
              loading: false
            })
          }
          setTimeout(() => {
            this.loopLoadTransactions()
          }, 10000)
        });
    }
  }

  render () {
    let {transactions, isRightText} = this.props;
    if (this.state.loading) {
      return (
        <div className="text-center d-flex justify-content-center">
          <TronLoader/>
        </div>
      );
    }

    return (
      <div className="card shadow-base" style={styles.card}>
        <div className="card-header bg-tron-light d-flex">
          <i className="fa fa-server mr-3 fa_width_20 color-card-header" />
          <h5 className="m-0 lh-175 color-card-header">{tu('transfers')}</h5>
          <Link to="/blockchain/transfers"
                className="ml-auto btn btn-sm btn-default"
                style={{borderRadius: '0.15rem'}}>
            {tu('view_all')}
          </Link>
        </div>
        {
          isMobile ?
            <PerfectScrollbar>
              <ul className="list-group list-group-flush" style={styles.list}>

                {
                  transactions.map((transfer, i) => (
                    transfer &&
                    <li key={transfer.hash || i} className="list-group-item overflow-h mobile-block">
                      <div className="media">
                        <div className=" mb-0 w-100 d-flex ">

                          <div className="text-left pt-1 w-100">
                            <div className="d-flex justify-content-between">
                              <div className="pt-1 d-flex flex-1">
                                <i className="fa fa-bars mr-2 mt-1 fa_width color-link__primary" />
                                {/* <TransactionHashLink
                                    hash={transfer.hash}>{transfer.hash.substr(0, 13)}...</TransactionHashLink> */}
                                <div className="flex-1">
                                  <Truncate>
                                    <TransactionHashLink
                                      hash={transfer.hash}>{transfer.hash}...</TransactionHashLink>
                                  </Truncate>
                                </div>
                              </div>
                              <div className="text-muted color-grey-300 small pt-2 pl-3">
                                <TimeAgo date={transfer.timestamp}
                                         title={moment(transfer.timestamp).format('MMM-DD-YYYY HH:mm:ss A')}/>
                              </div>
                            </div>

                            <div
                              className={(isRightText ? 'flex-row-reverse justify-content-end' : '') + ' d-flex align-items-center'}>
                              <span className="color-grey-300 mr-2">{tu('from')}</span>
                              <AddressLink className="color-link__primary small" wrapClassName="d-inline-block w-50"
                                           address={transfer.owner_address}>
                                {transfer.owner_address}...
                              </AddressLink>
                            </div>
                            <div
                              className={(isRightText ? 'flex-row-reverse justify-content-end' : '') + ' d-flex align-items-center'}>
                              <span className="color-grey-300 mr-2">{tu('to')}</span>
                              <AddressLink className="color-link__primary small" wrapClassName="d-inline-block w-50"
                                           address={transfer.to_address}>
                                {transfer.to_address && transfer.to_address.substr(0, 20)}...
                              </AddressLink>
                            </div>
                            <div className="color-grey-200 pb-2">
                              <span className="color-grey-300 mr-2 d-inline-block">{tu('value')}</span>
                              <NameWithId value={transfer} type="symbol" totoken valueRange/>
                            </div>
                          </div>

                        </div>
                      </div>
                    </li>
                  ))
                }

              </ul>
            </PerfectScrollbar>
            :
            <PerfectScrollbar>
              <ul className="list-group list-group-flush" style={styles.list}>
                {
                  transactions.map((transfer, i) => (
                    transfer &&
                    <li key={transfer.hash || i} className="list-group-item overflow-h"
                        style={{minHeight: '100px'}}>
                      <div className="media">
                        <div className="media-body mb-0">
                          <div className="text-left pt-1 d-flex justify-content-between">
                            <div className="pt-1 d-flex pr-2 color-transfers-hash" style={{flex: 1, maxWidth: '400px'}}>
                              <i className="fa fa-bars mr-2 mt-1 fa_width color-link__primary" />
                              <Truncate>
                                <TransactionHashLink
                                  hash={transfer.hash}>{transfer.hash}</TransactionHashLink>
                              </Truncate>

                            </div>
                            <div className="color-grey-200 pt-1 " style={{fontSize: '1rem'}}>
                              {/* <TRXPrice amount={transfer.amount} name={transfer.tokenName} source='transfers'/> */}
                              <NameWithId value={transfer} type="symbol" totoken valueRange/>
                            </div>

                          </div>
                          <div className="d-flex pt-2 list-item-word">
                            <div
                              className={(isRightText ? 'flex-row-reverse justify-content-end' : '') + ' d-flex pt-2 text-left'}
                              style={{fontSize: '0.8rem'}}>
                              <span className="color-grey-300 mr-2">{tu('from')}</span>

                              <AddressLink wrapClassName="d-inline-block mr-2" className="color-link__primary"
                                           address={transfer.owner_address} truncate={false}>
                                {transfer.owner_address && transfer.owner_address.substr(0, 12)}...
                              </AddressLink>

                              <span className="color-grey-300 mr-2">{tu('to')}</span>
                              <AddressLink wrapClassName="d-inline-block mr-2" className="color-link__primary"
                                           address={transfer.to_address} truncate={false}>
                                {transfer.to_address && transfer.to_address.substr(0, 12)}...
                              </AddressLink>
                            </div>
                            <div className="text-muted text-right color-grey-300 small" style={styles.nowrap}>
                              <TimeAgo date={transfer.timestamp}
                                       title={moment(transfer.timestamp).format('MMM-DD-YYYY HH:mm:ss A')}/>
                            </div>
                          </div>
                        </div>
                      </div>
                    </li>
                  ))
                }
              </ul>
            </PerfectScrollbar>
        }
      </div>
    )
  }
}

function mapStateToProps (state) {
  return {
    transactions: state.blockchain.transactions,
    activeLanguage: state.app.activeLanguage,
    isRightText: state.app.isRightText
  };
}

const mapDispatchToProps = {
  loadTransactions
};

export default connect(mapStateToProps, mapDispatchToProps)(withTimers(injectIntl(RecentTransfers)))

const styles = {
  list: {
    overflowX: 'none',
    height: 594,
  },
  card: {
    border: 'none',
    borderRadius: 0
  },
  nowrap: {
    flex: 1,
    whiteSpace: 'nowrap'
  }
};
