import React, { Component } from 'react';
import { connect } from 'react-redux';
import xhr from 'axios/index';
import { FormattedNumber, injectIntl } from 'react-intl';
import { doSearch, getSearchType } from '../../services/search';
import CountUp from 'react-countup';
import { Client } from '../../services/api';
import { Link } from 'react-router-dom';
// import { TRXPrice } from '../common/Price';
import RecentBlocks from './RecentBlocks';
import { KEY_ENTER } from '../../utils/constants';
import { withTimers } from '../../utils/timing';
import isMobile from '../../utils/isMobile';
import RecentTransfers from './RecentTransfers';
import { tu } from '../../utils/i18n';
import { toastr } from 'react-redux-toastr';
import { HrefLink } from '../common/Links';
import { CHAIN_SYMBOL } from '../../constants';
// import { API_URL } from '../../constants';
// import { setWebsocket } from '../../actions/account';
// import SearchBar from '../common/SearchBar'

@connect(
  state => {
    return {
      blocks: state.blockchain.blocks,
      account: state.app.account,
      theme: state.app.theme,
      wsdata: state.account.wsdata,
      websocket: state.account.websocket,
    }
  },
  {
    // setWebsocket
  }
)
@withTimers
@injectIntl
export default class Home extends Component {

  constructor () {
    super();
    this.listener = null;
    this.state = {
      search: '',
      isShaking: false,
      hasFound: false,
      notice: [],
      systemStatuses: {},
      priceExchangeData: {}
    };
  }

  async loadSystemStatus () {
    // let {total} = await Client.getNodeLocations();
    // let {data} = await xhr.get(`${API_URL}/api/node`);
    // this.setState({
    //   onlineNodes: data.total
    // })
    let data = await Client.getSystemStatus()
    this.setState({
      systemStatuses: data
    })
  }

  doSearch = async () => {
    let {intl} = this.props;
    let {search} = this.state;
    let type = getSearchType(search);

    let result = await doSearch(search, type);
    if (result !== null) {
      this.setState({
        hasFound: true,
      });
      setTimeout(() => {
        window.location.hash = result;
      }, 600);
    } else {
      this.setState({
        search: '',
        isShaking: true,
      });

      setTimeout(() => {
        this.setState({
          isShaking: false,
        });
      }, 1000);

      toastr.warning(intl.formatMessage({id: 'warning'}), intl.formatMessage({id: 'search_not_found'}));
    }
  }

  onSearchKeyDown = (ev) => {
    if (ev.keyCode === KEY_ENTER) {
      this.doSearch();
    }
  };

  async loadPrice() {
    const fromCode = CHAIN_SYMBOL
    const toCode = 'USD'
    const {data} = await xhr.get(`https://api.midasprotocol.com/token/v1/prices?fromCodes=${fromCode}&toCodes=${toCode}`)
    const res = data ? data.data : {}
    const items = res ? res.items : []
    if (items.length > 0) {
      const priceData = items[0] && items[0].rates && items[0].rates[toCode]
      this.setState({
        priceExchangeData: priceData ? priceData : {}
      })
    }
  }

  async componentDidMount () {
    this.loadSystemStatus();
    this.loadPrice();
  }

  // getLogo = () => {
  //   let {theme} = this.props;
  //   switch (theme) {
  //     case 'tron':
  //       return require('../../images/tron-banner-tronblue.png');
  //     default:
  //       return require('../../images/tron-banner-1.png');
  //   }
  // };

  render () {
    // let {intl} = this.props;
    // let {search, isShaking, hasFound, onlineNodes, blockHeight, transactionPerDay, totalAccounts, txOverviewStats, addressesStats, maxTps, tps} = this.state;
    let { systemStatuses, priceExchangeData } = this.state;
    return (
      <main className="home pb-0">
        <div className="container-fluid position-relative d-flex pt-3 pt-md-4 mx-auto flex-column mb-3">
          <div className="container home-splash p-0 p-md-3">
            {/*<div className="row justify-content-center text-center">*/}
            {/*<div className="col-12 exchange">*/}
            {/*<div className="notice">*/}
            {/*<img src={require("../../images/announcement-logo.png")} alt="" />*/}
            {/*<div className="notice-wrap">*/}
            {/*{this.state.notice.map((v, i) => (*/}
            {/*<a*/}
            {/*className="item"*/}
            {/*key={v.id}*/}
            {/*href={v.html_url}*/}
            {/*target="_blank"*/}
            {/*>*/}
            {/*<span title={v.name} className="title">*/}
            {/*{v.name}*/}
            {/*</span>*/}
            {/*<span className="date">*/}
            {/*({v.created_at.substring(5, 10)})*/}
            {/*</span>*/}
            {/*</a>*/}
            {/*))}*/}
            {/*</div>*/}
            {/*{this.state.notice.length > 0 ? (*/}
            {/*<a*/}
            {/*href={*/}
            {/*intl.locale == "zh"*/}
            {/*? "https://support.tronscan.org/hc/zh-cn/categories/360001618172-%E5%85%AC%E5%91%8A%E4%B8%AD%E5%BF%83"*/}
            {/*: "https://support.tronscan.org/hc/en-us/categories/360001621692-Announcements"*/}
            {/*}*/}
            {/*target="_blank"*/}
            {/*>*/}
            {/*{tu("learn_more")}>*/}
            {/*</a>*/}
            {/*) : null}*/}
            {/*</div>*/}
            {/*/!*<p className="mt-5 mt-5-logo">*!/*/}
            {/*/!*<img src={this.getLogo()}*!/*/}
            {/*/!*className="animated ad-600ms zoomIn"/>*!/*/}
            {/*/!*</p>*!/*/}
            {/*/!*<h2 className="mb-5 text-muted animated fadeIn ad-1600ms" style={{fontSize: 32}}>*!/*/}
            {/*/!*{tu("tron_main_message")}*!/*/}
            {/*/!*</h2>*!/*/}
            {/*/!**/}
            {/*<div className={*/}
            {/*'input-group input-group-lg mb-4' +*/}
            {/*(isShaking ? ' animated shake ' : '') +*/}
            {/*(hasFound ? ' animated bounceOut' : '')*/}
            {/*}>*/}
            {/*<input type="text"*/}
            {/*className="form-control p-3 bg-tron-light  color-grey-100 border-0 box-shadow-none"*/}
            {/*style={{fontSize: 13, borderRadius: 0}}*/}
            {/*value={search}*/}
            {/*onKeyDown={this.onSearchKeyDown}*/}
            {/*onChange={ev => this.setState({search: ev.target.value})}*/}
            {/*placeholder={intl.formatMessage({id: 'search_description'})}/>*/}

            {/*<div className="input-group-append">*/}
            {/*<button className="btn btn-search box-shadow-none" onClick={this.doSearch}*/}
            {/*style={{borderRadius: 0}}>*/}
            {/*<i className="fa fa-search"/>*/}
            {/*</button>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/****************/}
            {/*<div className={'mb-3 mb-md-5'}>*/}
            {/*<div className="card shadow-base p-4" style={{border: 'none', borderRadius: 0}}>*/}
            {/*<SearchBar />*/}
            {/*</div>*/}
            {/*</div>*/}
            {
              isMobile ?
                <div className="row text-center mr-0 ml-0 mobile-home-state">
                  <div className="col-12  card  pt-1 mb-0 shadow-base" style={{border: 'none', borderRadius: 0}}>
                    <div className="row pt-3">
                      <div className="col-6 ">
                        <span className="hvr-underline-from-center hvr-underline-white text-muted">
                          <img src={require('../../images/home/node.png')}/>
                          <h2><CountUp start={0} end={systemStatuses.total_nodes || 0} duration={1}/></h2>
                          <p className="m-0">{tu('online_nodes')}</p>
                        </span>
                      </div>
                      <div className="col-6">
                        <Link to="/blockchain/blocks"
                              className="hvr-underline-from-center hvr-underline-white text-muted">
                          <img src={require('../../images/home/block.png')}/>
                          <h2><CountUp start={0} end={systemStatuses.block_height || 0} duration={1}/></h2>
                          <p className="m-0">{tu('block_height')}</p>
                        </Link>
                      </div>
                      <div className="col-6">
                        <Link to="/supernode/witnesses"
                              className="hvr-underline-from-center hvr-underline-white text-muted">
                          <img src={require('../../images/home/tps.png')}/>
                          {/*<h2>*/}
                          {/*<CountUp start={0} end={systemStatuses.currentTps || 0} duration={1}/>*/}
                          {/*{ '/' }*/}
                          {/*<CountUp start={0} end={systemStatuses.peakTps || 0} duration={1}/>*/}
                          {/*</h2>*/}
                          <h2><CountUp start={0} end={systemStatuses.total_witnesses || 0} duration={1}/></h2>
                          {/*<p className="m-0">{tu('current_MaxTPS')}</p>*/}
                          <p className="m-0">{ 'Total Supernodes' }</p>
                        </Link>
                      </div>

                      <div className="col-6">
                        <Link to="/blockchain/transactions"
                              className="hvr-underline-from-center hvr-underline-white text-muted">
                          <img src={require('../../images/home/transctions.png')}/>
                          <h2><CountUp start={0} end={systemStatuses.total_transactions || 0} duration={1}/></h2>
                          <p className="m-0">{ 'Transactions' }</p>
                        </Link>
                      </div>
                      <div className="col-6">
                        <Link to="/blockchain/accounts"
                              className="hvr-underline-from-center hvr-underline-white text-muted">
                          <img src={require('../../images/home/account.png')}/>
                          <h2><CountUp start={0} end={systemStatuses.total_accounts || 0} duration={1}/></h2>
                          <p className="m-0">{tu('total_accounts')}</p>
                        </Link>
                      </div>
                      <div className="col-6">
                        <HrefLink href="https://www.coingecko.com/en/coins/midas-cash" target="_blank"
                                  className="hvr-underline-from-center hvr-underline-white text-muted">
                          <img src={require('../../images/home/price.png')}/>
                          {/*<h2><TRXPrice amount={1} currency="USD" source="home"/></h2>*/}
                          <h2>
                            {
                              typeof priceExchangeData.rate !== 'undefined' && !isNaN(priceExchangeData.rate) ?
                                (
                                  <span>$
                                    <FormattedNumber
                                      value={priceExchangeData.rate}
                                      maximumFractionDigits={priceExchangeData.rate <= 0.01 ? 4 : 2}
                                    />
                                  </span>
                                ) : '-'
                            }
                            <img src={require('../../images/coingecko.png')} style={{width: 12, height: 12}}/>
                          </h2>
                          <p className="m-0">
                            {tu('pice_per_1trx')}&nbsp;
                            {
                              typeof priceExchangeData.percentChange24h !== 'undefined' && (
                                <span style={{ color: +priceExchangeData.percentChange24h >= 0 ? '#0fd586' : '#de4437' }}>
                                  {
                                    +priceExchangeData.percentChange24h >= 0 ? (
                                      <i className="fa fa-arrow-up" aria-hidden="true"/>
                                    ) : (
                                      <i className="fa fa-arrow-down" aria-hidden="true"/>
                                    )
                                  }
                                  <FormattedNumber
                                    value={Math.abs(priceExchangeData.percentChange24h || 0)}
                                    maximumFractionDigits={2}
                                  />%
                                </span>
                              )
                            }
                          </p>
                        </HrefLink>
                      </div>

                    </div>
                  </div>
                </div>
                :
                <div className="row text-center mr-0 ml-0">
                  <div className="col-12 card shadow-base pt-1 pl-0 pr-0" style={{border: 'none', borderRadius: 0}}>
                    <div className="card-body d-flex pt-4 pb-4 home-stats">
                      <div className="col-md-2 col-sm-12 col-xs-12 ">
                        <span className="hvr-underline-from-center hvr-underline-white text-muted">
                          <h2><CountUp start={0} end={systemStatuses.total_nodes || 0} duration={1}/></h2>
                          <p className="m-0">{tu('online_nodes')}</p>
                        </span>
                      </div>
                      <div className="col-md-2 col-sm-12 col-xs-12">
                        <Link to="/blockchain/blocks"
                              className="hvr-underline-from-center hvr-underline-white text-muted">
                          <h2><CountUp start={0} end={systemStatuses.block_height || 0} duration={1}/></h2>
                          <p className="m-0">{tu('block_height')}</p>
                        </Link>
                      </div>
                      <div className="col-md-2 col-sm-12 col-xs-12">
                        <Link to="/supernode/witnesses"
                              className="hvr-underline-from-center hvr-underline-white text-muted">
                          <h2>
                            {/*<CountUp start={0} end={systemStatuses.currentTps || 0} duration={1}/>*/}
                            {/*{ '/' }*/}
                            {/*<CountUp start={0} end={systemStatuses.peakTps || 0} duration={1}/>*/}
                            <CountUp start={0} end={systemStatuses.total_witnesses || 0} duration={1}/>
                          </h2>
                          <p className="m-0">{ 'Total Supernodes' }</p>
                        </Link>
                      </div>
                      <div className="col-md-2 col-sm-12 col-xs-12">
                        <Link to="/blockchain/transactions"
                              className="hvr-underline-from-center hvr-underline-white text-muted">
                          <h2><CountUp start={0} end={systemStatuses.total_transactions || 0} duration={1}/></h2>
                          <p className="m-0">{ 'Transactions' }</p>
                        </Link>
                      </div>
                      <div className="col-md-2 col-sm-12 col-xs-12">
                        <Link to="/blockchain/accounts"
                              className="hvr-underline-from-center hvr-underline-white text-muted">
                          <h2><CountUp start={0} end={systemStatuses.total_accounts || 0} duration={1}/></h2>
                          <p className="m-0">{tu('total_accounts')}</p>
                        </Link>
                      </div>
                      <div className="col-md-2 col-sm-12 col-xs-12">
                        <HrefLink href="https://www.coingecko.com/en/coins/midas-cash" target="_blank"
                                  className="hvr-underline-from-center hvr-underline-white text-muted">
                          {/*<h2><TRXPrice amount={priceExchangeData.rate} currency="USD" source="home"/></h2>*/}
                          <h2>
                            {
                              typeof priceExchangeData.rate !== 'undefined' && !isNaN(priceExchangeData.rate) ?
                                (
                                  <span>$
                                    <FormattedNumber
                                      value={priceExchangeData.rate}
                                      maximumFractionDigits={priceExchangeData.rate <= 0.01 ? 4 : 2}
                                    />
                                  </span>
                                ) : '-'
                            }
                            <img src={require('../../images/coingecko.png')} style={{width: 16, height: 16}}/>
                          </h2>
                          <p className="m-0">
                            {tu('pice_per_1trx')}&nbsp;
                            {
                              typeof priceExchangeData.percentChange24h !== 'undefined' && (
                                <span style={{ color: +priceExchangeData.percentChange24h >= 0 ? '#0fd586' : '#de4437' }}>
                                  {
                                    +priceExchangeData.percentChange24h >= 0 ? (
                                      <i className="fa fa-arrow-up" aria-hidden="true"/>
                                    ) : (
                                      <i className="fa fa-arrow-down" aria-hidden="true"/>
                                    )
                                  }
                                  <FormattedNumber
                                    value={Math.abs(priceExchangeData.percentChange24h || 0)}
                                    maximumFractionDigits={2}
                                  />%
                                </span>
                              )
                            }
                          </p>
                        </HrefLink>
                      </div>
                    </div>
                  </div>
                </div>
            }
          </div>
        </div>
        <div className="pb-3 pb-md-5">
          <div className="container">
            {/*{*/}
            {/*isMobile?*/}
            {/*<div className="row mt-0 mt-md-4 mb-3">*/}
            {/*<div className="col-md-6 mt-3 mt-md-0 ">*/}
            {/*<div className="card " style={styles.card}>*/}
            {/*<div className="card-header bg-tron-light pb-0" style={styles.card}>*/}
            {/*<h5 className="m-0 lh-150">*/}
            {/*<Link to="blockchain/stats/txOverviewStats">*/}
            {/*{tu("14_day_transaction_history")}*/}
            {/*</Link>*/}
            {/*</h5>*/}
            {/*</div>*/}
            {/*<div className="card-body pt-0">*/}

            {/*<div style={{minWidth: 255, height: 140}}>*/}
            {/*{*/}
            {/*txOverviewStats === null ?*/}
            {/*<TronLoader/> :*/}
            {/*<LineReactHighChartTx style={{minWidth: 255, height: 140}} data={txOverviewStats}*/}
            {/*intl={intl} source='home'/>*/}
            {/*}*/}
            {/*</div>*/}

            {/*</div>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*<div className="col-md-6 mt-3 mt-md-0 ">*/}
            {/*<div className="card" style={styles.card}>*/}
            {/*<div className="card-header bg-tron-light pb-0" style={styles.card}>*/}
            {/*<h5 className="m-0 lh-150">*/}
            {/*<Link to="blockchain/stats/addressesStats">*/}
            {/*{tu("14_day_address_growth")}*/}
            {/*</Link>*/}
            {/*</h5>*/}
            {/*</div>*/}
            {/*<div className="card-body pt-0">*/}

            {/*<div style={{minWidth: 255, height: 140}}>*/}
            {/*{*/}
            {/*addressesStats === null ?*/}
            {/*<TronLoader/> :*/}
            {/*<LineReactHighChartAdd style={{minWidth: 255, height: 140}} data={addressesStats}*/}
            {/*intl={intl} source='home'/>*/}
            {/*}*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*:*/}
            {/*<div className="row mt-0 mt-md-4 mb-3">*/}
            {/*<div className="col-md-6 mt-3 mt-md-0 ">*/}
            {/*<div className="card " style={styles.card}>*/}
            {/*<div className="card-header bg-tron-light color-grey-100 text-center pb-0" style={styles.card}>*/}
            {/*<h5 className="m-0 lh-150">*/}
            {/*<Link to="blockchain/stats/txOverviewStats">*/}
            {/*{tu("14_day_transaction_history")}*/}
            {/*</Link>*/}
            {/*</h5>*/}
            {/*</div>*/}
            {/*<div className="card-body pt-0" style={{paddingLeft: '2rem', paddingRight: '2rem'}}>*/}

            {/*<div style={{minWidth: 255, height: 200}}>*/}
            {/*{*/}
            {/*txOverviewStats === null ?*/}
            {/*<TronLoader/> :*/}
            {/*<LineReactHighChartTx style={{minWidth: 255, height: 200}} data={txOverviewStats}*/}
            {/*intl={intl} source='home'/>*/}
            {/*}*/}
            {/*</div>*/}

            {/*</div>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*<div className="col-md-6 mt-3 mt-md-0 ">*/}
            {/*<div className="card" style={styles.card}>*/}
            {/*<div className="card-header bg-tron-light color-grey-100 text-center pb-0" style={styles.card}>*/}
            {/*<h5 className="m-0 lh-150">*/}
            {/*<Link to="blockchain/stats/addressesStats">*/}
            {/*{tu("14_day_address_growth")}*/}
            {/*</Link>*/}
            {/*</h5>*/}
            {/*</div>*/}
            {/*<div className="card-body pt-0" style={{paddingLeft: '2rem', paddingRight: '2rem'}}>*/}

            {/*<div style={{minWidth: 255, height: 200}}>*/}
            {/*{*/}
            {/*addressesStats === null ?*/}
            {/*<TronLoader/> :*/}
            {/*<LineReactHighChartAdd style={{minWidth: 255, height: 200}} data={addressesStats}*/}
            {/*intl={intl} source='home'/>*/}
            {/*}*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*</div>*/}
            {/*}*/}

            <div className="row mt-0 mt-md-4">
              <div className="col-md-6 mt-0 mb-3 mt-md-0 text-center">
                <RecentBlocks/>
              </div>
              <div className="col-md-6 text-center">
                <RecentTransfers/>
              </div>
            </div>
          </div>
        </div>

      </main>
    )
  }
}
