import React from 'react'
import { FormattedNumber } from 'react-intl';
import { tu } from '../../utils/i18n';
import { getAccountBandwidth } from '../../utils/helper';

const Bandwidth = ({ currentWallet }) => {
  const bandwidth = getAccountBandwidth(currentWallet)

  return (
    <div className="row mt-3">
      <div className="col-md-12">
        <div className="card">
          <div className="card-body px-0 border-0">
            <h5 className="card-title text-center m-0">
              {tu('bandwidth')}
            </h5>
          </div>
          <table className="table m-0">
            <tbody>
            <tr>
              <th style={{width: 200}}>{tu('free_bandwidth')}</th>
              <td>
                  <span className="text-primary">
                    <FormattedNumber value={bandwidth.freeBandwidthUsed} className="text-success"/>&nbsp;
                  </span>/&nbsp;
                <span className="text-muted">
                    <FormattedNumber value={bandwidth.freeBandwidthLimit}/>
                  </span>
                <span className="float-right">
                    {Math.ceil(bandwidth.freeBandWidthPercentage)}%
                  </span>
                <div className="progress mt-1">
                  <div className="progress-bar bg-primary"
                       style={{width: bandwidth.freeBandWidthPercentage + '%'}}/>
                </div>
              </td>
            </tr>
            <tr>
              <th style={{width: 200}}>{tu('bandwidth')}</th>
              <td>
                    <span className="text-primary">
                      <FormattedNumber value={bandwidth.bandwidthUsed} className="text-primary"/>&nbsp;
                    </span>/&nbsp;
                <span className="text-muted">
                      <FormattedNumber value={bandwidth.bandwidthLimit}/>
                    </span>
                <span className="float-right">
                      {Math.ceil(bandwidth.bandWidthPercentage)}%
                    </span>
                <div className="progress mt-1">
                  <div className="progress-bar bg-primary"
                       style={{width: bandwidth.bandWidthPercentage + '%'}}/>
                </div>
              </td>
            </tr>
            {
              Object.keys(currentWallet.bandwidth.assets).length > 0 &&
              <tr>
                <td className="bg-light text-center" colSpan={2}>
                  Tokens
                </td>
              </tr>
            }
            {
              Object.entries(currentWallet.bandwidth.assets).map(([token, bandwidth]) => (
                <tr>
                  <th style={{width: 200}}>{token}</th>
                  <td>
                    <span className="text-primary">
                      <FormattedNumber value={bandwidth.netUsed} className="text-primary"/>&nbsp;
                    </span>
                    /&nbsp;
                    <span className="text-muted">
                      <FormattedNumber value={bandwidth.netLimit}/>
                    </span>
                    <span className="float-right">
                      {Math.ceil(bandwidth.netPercentage)}%
                    </span>
                    <div className="progress mt-1">
                      <div className="progress-bar bg-primary" style={{width: bandwidth.netPercentage + '%'}}/>
                    </div>
                  </td>
                </tr>
              ))
            }
            </tbody>
          </table>
        </div>
      </div>
    </div>
  );
}

export default Bandwidth
