import React from 'react'
import { FormattedDate, FormattedTime, injectIntl } from 'react-intl';
import SweetAlert from 'react-bootstrap-sweetalert';
import { tu } from '../../utils/i18n';
import { ONE_UNIT } from '../../constants';
import { withTronWeb } from '../../utils/mcashWeb';
import { connect } from 'react-redux';
import { reloadWallet } from '../../actions/wallet';
import { TRXPrice } from '../common/Price';
import Lockr from 'lockr';
import { transactionResultManager } from '../../utils/mcash';

@connect(
  state => ({
    currentWallet: state.wallet.current,
    walletApp: state.app.wallet
  }),
  {
    reloadWallet
  }
)
@injectIntl
@withTronWeb
export default class StakeTable extends React.Component {

  constructor () {
    super()
    this.state = {
      modal: null
    }
  }

  hideModal = () => {
    this.setState({
      modal: null,
    });
  }

  unstake = async () => {
    let {walletApp} = this.props;
    const walletType = walletApp ? walletApp.type : ''
    let res;
    this.hideModal();
    if (Lockr.get('islogin') || walletType === 'ACCOUNT_LEDGER' || walletType === 'ACCOUNT_MCASHLINK') {
      // const tronWebLedger = this.props.mcashWeb();
      const {mcashWeb} = this.props.account;

      try {
        if (this.props.walletType.type === 'ACCOUNT_LEDGER') {
          // let unSignTransaction;
          // unSignTransaction = await tronWebLedger.transactionBuilder.unfreezeBalance(delegateType, walletType.address).catch(e => false)
          // const {result} = await transactionResultManager(unSignTransaction, tronWebLedger);
          // res = result;
        }
        if (this.props.walletType.type === 'ACCOUNT_MCASHLINK') {
          let unSignTransaction;
          unSignTransaction = await mcashWeb.transactionBuilder.unstake(mcashWeb.defaultAddress.base58).catch(e => false);
          const {result} = await transactionResultManager(unSignTransaction, mcashWeb);
          res = result;
        }

      } catch (e) {
        console.log(e)
      }
    } else {
      // let {success} = await Client.unfreezeBalance(account.address, delegateType, '')(account.key);
      // res = success
    }

    if (res) {
      this.setState({
        modal: (
          <SweetAlert success title="MCASH Unstake" onConfirm={this.hideModal}>
            {'Successfully Unstake MCASH'}
          </SweetAlert>
        )
      });
      setTimeout(() => this.props.reloadWallet(), 1200);
    } else {
      this.setState({
        modal: (
          <SweetAlert warning title={'Unable to unstake'} onConfirm={this.hideModal}>
            {'Unable to unstake MCASH. This could be caused because the minimal stake period hasn\'t been reached yet.'}
          </SweetAlert>
        ),
      });
    }
  };

  showUnstakeModal = () => {
    this.setState({
      modal: (
        <SweetAlert
          info
          showCancel
          confirmBtnText={'Unstake'}
          confirmBtnBsStyle="danger"
          cancelBtnBsStyle="default"
          cancelBtnText={tu('cancel')}
          title={<span>{ 'Are you sure you want to unstake MCASH?' }</span>}
          onConfirm={this.unstake}
          onCancel={this.hideModal}
          style={{height: '300px'}}
        />
      )
    })
  };

  render () {
    const {stake, witness_stake} = this.props.currentWallet || {};
    const {modal} = this.state;
    return (
      <div className={'mt-4'} style={{overflow: 'auto'}}>
        {modal}

        <h5>{'My stake'}</h5>
        <table className="table m-0 temp-table">
          <thead className="thead-light">
          <tr>
            <th>{tu('total_amount')}</th>
            <th className="text-right">{'Unstake Time'}</th>
            <th></th>
          </tr>
          </thead>
          <tbody>
          {
            stake && Object.keys(stake).length > 0 && (
              <tr>
                <td>
                  <TRXPrice amount={(stake.stake_amount || 0) / ONE_UNIT}/>
                </td>
                <td className="text-right">
                  <span className="mr-1">{tu('After')}</span>
                  <FormattedDate value={stake.expiration_time}/>&nbsp;
                  <FormattedTime value={stake.expiration_time} hour='numeric' minute="numeric" second='numeric'
                                 hour12={false}/>
                </td>
                <td className="text-right">
                  <button
                    className="btn btn-danger mr-2"
                    style={{marginTop: '-5px', marginBottom: '-5px'}}
                    onClick={() => {
                      this.showUnstakeModal()
                    }}
                  >{'Unstake'}</button>
                </td>
              </tr>
            )
          }
          {
            witness_stake && Object.keys(witness_stake).length > 0 && (
              <tr>
                <td>
                  <TRXPrice amount={(witness_stake.stake_amount || 0) / ONE_UNIT}/>
                </td>
                <td className="text-right">
                  {
                    witness_stake.expiration_time > 0 && (
                      <div>
                        <span className="mr-1">{tu('After')}</span>
                        <FormattedDate value={witness_stake.expiration_time}/>&nbsp;
                        <FormattedTime value={witness_stake.expiration_time} hour='numeric' minute="numeric"
                                       second='numeric'
                                       hour12={false}/>
                      </div>
                    )
                  }
                </td>
                <td/>
              </tr>
            )
          }
          </tbody>
        </table>
      </div>
    )
  }

}
