import React from 'react'
import { tu } from '../../utils/i18n';
import { AddressLink, TokenLink, TokenTRC20Link } from '../common/Links';
import { FormattedDate, FormattedNumber, FormattedTime } from 'react-intl';
import { ONE_UNIT } from '../../constants';
import { TRXPrice } from '../common/Price';

const Tokens = ({items = []}) => {
  if (items.length === 0) {
    return (
      <div className="text-center d-flex justify-content-center p-4">
        {tu('no_tokens')}
      </div>
    );
  }
  return (
    <table className="table mt-3 temp-table">
      <thead className="thead-light">
      <tr>
        <th width="40%">{tu('name')}</th>
        <th>ID</th>
        <th>{tu('TRC20_decimals')}</th>
        <th className="text-right">{tu('balance')}</th>
      </tr>
      </thead>
      <tbody>
      {
        items.map((item, index) => {
          const token = item && item.token ? item.token : {}
          return (
            <tr key={`m1-${index}`}>
              <td className="text-nowrap">
                <TokenLink id={item.asset_id} name={token.name + ' (' + token.symbol + ')'}
                           address={token.address}/>
              </td>
              <td>
                <div className="tokenBalances_id">{item.asset_id}</div>
              </td>
              <td>
                <div>{token.decimals}</div>
              </td>
              <td className="text-right">
                <FormattedNumber value={item.balance / Math.pow(10, token.decimals || 0)}
                                 maximumFractionDigits={Number(token.decimals || 0)}/>
              </td>
            </tr>
          )
        })
      }
      </tbody>
    </table>
  )
}

const Tokens20 = ({items = []}) => {
  if (items.length === 0) {
    return (
      <div className="text-center d-flex justify-content-center p-4">
        {tu('no_tokens')}
      </div>
    );
  }
  return (
    <table className="table mt-3 temp-table">
      <thead className="thead-light">
      <tr>
        <th>{tu('name')}</th>
        <th className="text-right">{tu('balance')}</th>
      </tr>
      </thead>
      <tbody>
      {
        items.map((item, index) => {
          const token = item && item.token ? item.token : {}
          return (
            <tr key={`m20-${index}`}>
              <td className="text-nowrap">
                <TokenTRC20Link name={token.name} address={item.contract_address}
                                namePlus={token.name + ' (' + token.symbol + ')'}/>
              </td>
              <td className="text-right">
                <span>{item.balance / Math.pow(10, token.decimals || 0)}</span>
                {/*<FormattedNumber value={token.token20_balance} maximumFractionDigits={20}/>*/}
              </td>
            </tr>
          )
        })
      }
      </tbody>
    </table>
  )
}

const FrozenTokens = (props) => {
  const {frozenForBandwidth, frozenForEnergy, delegated, showUnfreezeModal = () => {}} = props;

  let receiveDelegateBandwidth = 0;
  if (delegated && delegated.received_delegated_bandwidth) {
    for (let i = 0; i < delegated.received_delegated_bandwidth.length; i++) {
      receiveDelegateBandwidth = receiveDelegateBandwidth + delegated.received_delegated_bandwidth[i]['frozen_balance_for_bandwidth'];
    }
  }

  const frozenBandwidth = frozenForBandwidth ? frozenForBandwidth.frozen_balance : 0

  let receiveDelegateResource = 0;
  if (delegated && delegated.received_delegated_resource) {
    for (let i = 0; i < delegated.received_delegated_resource.length; i++) {
      receiveDelegateResource = receiveDelegateResource + delegated.received_delegated_resource[i]['frozen_balance_for_energy'];
    }
  }

  const frozenEnergy = frozenForEnergy && frozenForEnergy.frozen_balance ? frozenForEnergy.frozen_balance : 0

  if (frozenEnergy === 0 && frozenBandwidth === 0 && receiveDelegateBandwidth === 0 && receiveDelegateResource === 0) {
    return null;
  }

  return (
    <div style={{overflow: 'auto'}}>
      <h5>{tu('my_account')}</h5>
      <table className="table m-0 temp-table">
        <thead className="thead-light">
        <tr>
          <th>{tu('freeze_type')}</th>
          <th>{tu('delegate_other')}</th>
          <th>{tu('freeze_self')}</th>
          <th>{tu('total_amount')}</th>
          <th className="text-right">{tu('unfreeze_time')}</th>
          <th />
        </tr>
        </thead>
        <tbody>
        {
          frozenBandwidth ? (
            <tr>
              <td>
                {tu('bandwidth')}
              </td>
              <td>
                <TRXPrice amount={receiveDelegateBandwidth / ONE_UNIT}/>
              </td>
              <td>
                <TRXPrice amount={frozenBandwidth / ONE_UNIT}/>
              </td>
              <td>
                <TRXPrice amount={(receiveDelegateBandwidth + frozenBandwidth) / ONE_UNIT}/>
              </td>
              <td className="text-right">
                <span className="mr-1">{tu('After')}</span>
                <FormattedDate value={frozenForBandwidth.expire_time}/>&nbsp;
                <FormattedTime value={frozenForBandwidth.expire_time} hour='numeric' minute="numeric" second='numeric'
                               hour12={false}/>
              </td>
              <td className="text-right">
                {
                  frozenBandwidth !== 0 && (
                    <button
                      className="btn btn-danger mr-2"
                      style={{marginTop: '-5px', marginBottom: '-5px'}}
                      onClick={() => {
                        showUnfreezeModal(0, false, '')
                      }}
                    >{tu('unfreeze')}</button>
                  )
                }

              </td>
            </tr>
          ) : null
        }
        {
          frozenEnergy ? (
            <tr>
              <td>
                {tu('energy')}
              </td>
              <td>
                <TRXPrice amount={receiveDelegateResource / ONE_UNIT}/>
              </td>
              <td>
                <TRXPrice amount={frozenEnergy / ONE_UNIT}/>
              </td>
              <td>
                <TRXPrice amount={(frozenEnergy + receiveDelegateResource) / ONE_UNIT}/>
              </td>
              <td className="text-right">
                <span className="mr-1">{tu('After')}</span>
                <FormattedDate value={frozenForEnergy.expire_time}/>&nbsp;
                <FormattedTime value={frozenForEnergy.expire_time} hour='numeric' minute="numeric" second='numeric'
                               hour12={false}/>
              </td>
              <td className="text-right">
                {
                  frozenEnergy !== 0 && (
                    <button className="btn btn-danger mr-2" style={{
                      marginTop: '-5px',
                      marginBottom: '-5px'
                    }} onClick={() => {
                      showUnfreezeModal(1, false, '')
                    }}>
                      {tu('unfreeze')}
                    </button>
                  )
                }
              </td>
            </tr>
          ) : null
        }
        </tbody>
      </table>
    </div>
  )
}

const DelegateFrozenTokens = ({delegated, showUnfreezeModal = () => {}}) => {
  if (!delegated || ((delegated.sent_delegated_bandwidth && delegated.sent_delegated_bandwidth.length === 0) && (delegated.sent_delegated_resource && delegated.sent_delegated_resource.length === 0))) {
    return null;
  }

  return (
    <div style={{overflow: 'auto'}}>
      <h5 style={{marginTop: '10px'}}>{tu('delegate_list')}</h5>
      <table className="table m-0 temp-table">
        <thead className="thead-light">
        <tr>
          <th>{tu('receive_list')}</th>
          <th>{tu('type')}</th>
          <th>{tu('amount')}</th>
          <th className="text-right">{tu('unfreeze_time')}</th>
          <th></th>
        </tr>
        </thead>
        <tbody>
        {
          delegated.sent_delegated_bandwidth.map((item, index) => {
            return <tr key={index}>
              <td>
                <AddressLink address={item.to} truncate={false}>
                  <span className="color-tron-100">{item.to}</span>
                </AddressLink>
              </td>
              <td>
                {tu('bandwidth')}
              </td>
              <td>
                <TRXPrice amount={item.frozen_balance_for_bandwidth / ONE_UNIT}/>
              </td>
              <td className="text-right">
                <span className="mr-1">{tu('After')}</span>
                <FormattedDate value={item.expire_time_for_bandwidth}/>&nbsp;
                <FormattedTime value={item.expire_time_for_bandwidth} hour='numeric' minute="numeric" second='numeric'
                               hour12={false}/>
              </td>
              <td className="text-right">
                {
                  <button className="btn btn-danger mr-2" style={{
                    marginTop: '-5px',
                    marginBottom: '-5px'
                  }} onClick={() => {
                    showUnfreezeModal(0, true, item.to)
                  }}>
                    {tu('unfreeze')}
                  </button>
                }
              </td>
            </tr>
          })
        }
        {
          delegated.sent_delegated_resource.map((item, index) => {
            return <tr key={index}>
              <td>
                <AddressLink address={item.to} truncate={false}>
                  <span className="color-tron-100">{item.to}</span>
                </AddressLink>
              </td>
              <td>
                {tu('energy')}
              </td>
              <td>
                <TRXPrice amount={item.frozen_balance_for_energy / ONE_UNIT}/>
              </td>
              <td className="text-right">
                <span className="mr-1">{tu('After')}</span>
                <FormattedDate value={item.expire_time_for_energy}/>&nbsp;
                <FormattedTime value={item.expire_time_for_energy} hour='numeric' minute="numeric" second='numeric'
                               hour12={false}/>
              </td>
              <td className="text-right">
                {
                  <button className="btn btn-danger mr-2" style={{
                    marginTop: '-5px',
                    marginBottom: '-5px'
                  }} onClick={() => {
                    showUnfreezeModal(1, true, item.to)
                  }}>
                    {tu('unfreeze')}
                  </button>
                }
              </td>
            </tr>
          })
        }
        </tbody>
      </table>
    </div>
  )
}

export {
  Tokens,
  Tokens20,
  FrozenTokens,
  DelegateFrozenTokens
}

