import React from 'react';
import { BigNumber } from 'bignumber.js';
import { tu } from '../../../utils/i18n';
import { FormattedNumber, injectIntl } from 'react-intl';
import { TokenLink, TokenTRC20Link } from '../../common/Links';
import SmartTable from '../../common/SmartTable.js'
import { upperFirst } from 'lodash';

class TokenBalances extends React.Component {

  constructor (props) {
    super(props);
    this.state = {
      hideSmallCurrency: false,
      balances: [],
      emptyState: props.emptyState
    };
  }

  componentDidMount () {
    this.load();
  }

  onChange = (page, pageSize) => {
    this.load(page, pageSize);
  };

  load = (page = 1) => {
    let {tokenBalances, assetsBalances} = this.props;
    this.setState({
      page,
      balances: assetsBalances.concat(tokenBalances)
    });
  };

  handleSwitch = (val) => {
    this.setState({
      hideSmallCurrency: val
    }, () => {
      this.load();
    });
  }

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: upperFirst(intl.formatMessage({id: 'token'})),
        key: 'map_token_name',
        width: '25%',
        align: 'left',
        className: 'ant_table',
        render: (text, record) => {
          const token = record.token || {}
          return (
            <div>
              <img width={20} height={20} src={token.logo_url} style={{marginRight: 5}}/>
              {
                token.type === 'M20' ? (
                    <TokenTRC20Link
                      name={token.name}
                      address={token.contract_address}
                      namePlus={token.name + (token.symbol ? ' (' + token.symbol + ')' : '')}/>
                  ) : (
                    <TokenLink
                      id={token.asset_id}
                      name={token.name + (token.symbol ? ' (' + token.symbol + ')' : '')}
                      address={token.contract_address}/>
                  )
              }
            </div>
          )
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'address_balance_token_type'})),
        key: 'tokenType',
        align: 'left',
        className: 'ant_table',
        render: (text, record) => {
          return <span>{record.token && record.token.type ? record.token.type : '-'}</span>
        }
      },
      {
        title: 'ID',
        key: 'map_token_id',
        align: 'left',
        className: 'ant_table',
        render: (text, record) => {
          return <span>{record.token && !isNaN(record.token.asset_id) ? record.token.asset_id : '-'}</span>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'TRC20_decimals'})),
        key: 'map_token_precision',
        align: 'left',
        className: 'ant_table',
        render: (text, record) => {
          return <span>{record.token && record.token.decimals ? record.token.decimals : '-'}</span>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'balance'})),
        dataIndex: 'balance',
        key: 'map_amount',
        width: '18%',
        align: 'left',
        className: 'ant_table',
        render: (text, record) => {
          const token = record.token || {}
          if (isNaN(token.decimals)) {
            return '-'
          }
          const balance = new BigNumber(text).div(Math.pow(10, token.decimals || 0))
          return (
            <FormattedNumber
              value={balance}
              maximumFractionDigits={Number(token.decimals || 0)}
            />
          )
        }
      },
      // {
      //   title: upperFirst(intl.formatMessage({id: 'pice_per_onetrx'})),
      //   dataIndex: 'priceInTrx',
      //   key: 'priceInTrx',
      //   align: 'left',
      //   className: 'ant_table',
      //   render: (text) => {
      //     return <span>{text ? text : '-'}</span>
      //   }
      // },
      // {
      //   title: upperFirst(intl.formatMessage({id: 'address_balance_token_price_TRX'})),
      //   dataIndex: 'mcashBalance',
      //   key: 'mcashBalance',
      //   align: 'left',
      //   className: 'ant_table',
      //   render: (text, record) => {
      //     return (
      //       <span>
      //         {record.mcashBalance ? <div>
      //           <div>
      //             {record.TRXBalance_toThousands} MCASH
      //           </div>
      //           <div className="small">
      //             <TRXPrice amount={record.mcashBalance}
      //                       currency="USD"
      //                       showPopup={false}/>
      //           </div>
      //
      //         </div> : '-'}
      //       </span>
      //     )
      //   }
      // },
    ];
  }

  render () {
    let {balances} = this.state;
    let column = this.customizedColumn();
    let {intl} = this.props;
    let tableInfo = intl.formatMessage({id: 'view_total'}) + ' ' + balances.length + ' ' + intl.formatMessage({id: 'token_unit'})
    let locale = {emptyText: intl.formatMessage({id: 'no_tokens_found'})}
    return (
      <div className="token_black table_pos">
        <div className=" d-flex justify-content-between" style={{left: 'auto'}}>
          <div className="table_pos_info d-md-block table_pos_info_addr d-none">
            {tableInfo}
          </div>
          {/*<div className={'table_pos_switch d-md-block ' + (balances.length ? 'table_pos_switch_addr4' : '')}>*/}
          {/*<SwitchToken handleSwitch={this.handleSwitch} text="hide_small_currency" hoverText="tokens_less_than_10"/>*/}
          {/*</div>*/}
        </div>
        <div>
          {
            Object.keys(balances).length === 0 ?
              <div className="text-center p-3 no-data">
                {tu('no_tokens_found')}
              </div>
              :
              <div className="mt-1">
                <SmartTable bordered={true} column={column} data={balances} total={balances.length}
                            locale={locale}
                            addr="address"/>
              </div>
          }
        </div>


      </div>
    )
  }
}

export default injectIntl(TokenBalances);

