/* eslint-disable no-undef */
import React, { Fragment } from 'react';
import { injectIntl } from 'react-intl';
import { NavLink, Route, Switch } from 'react-router-dom';
import { Client } from '../../../services/api';
import { tu } from '../../../utils/i18n';
import { FormattedNumber } from 'react-intl';
import TokenBalances from './TokenBalances';
import { ONE_UNIT } from '../../../constants';
import { AddressLink, ExternalLink } from '../../common/Links';
import { TRXPrice } from '../../common/Price';
import { TronLoader } from '../../common/loaders';
import Transactions from '../../common/Transactions';
import NewTransactions from '../../common/NewTransactions';
import Votes from '../../common/Votes';
import Transfers from '../../common/Transfers';
import TransfersTrc20 from '../../common/TransfersTrc20';
import Blocks from '../../common/Blocks';
import { Progress, Tooltip } from 'antd'
import BigNumber from 'bignumber.js'
// import { HrefLink } from '../../common/Links';
import { QuestionMark } from '../../common/QuestionMark';
import Rewards from '../../common/Rewards';
import {
  fromAmount, getAccountBandwidth, getAccountEnergy, mapContractTokenData,
  mapTokenData
} from '../../../utils/helper';

BigNumber.config({EXPONENTIAL_AT: [-20, 30]});

class Address extends React.Component {

  constructor () {
    super();
    this.state = {
      totalPower: 0,
      candidates: [],
      showQrCode: false,
      loading: true,
      blocksProduced: 0,
      votes: null,
      rank: 0,
      totalVotes: 0,
      address: {
        address: '',
        balance: 0,
        tokenBalances: {},
      },
      stats: {
        transactions: 0,
      },
      media: null,
      tabs: {
        transfers: {
          id: 'transfers',
          icon: 'fa fa-exchange-alt',
          path: '',
          label: <span>{tu('transfers')}</span>,
          cmp: () => (
            <TronLoader>
              {tu('loading_transfers')}
            </TronLoader>
          )
        },
        transactions: {
          id: 'transactions',
          icon: 'fas fa-handshake',
          path: '/transactions',
          label: <span>{tu('transactions')}</span>,
          cmp: () => (
            <TronLoader>
              {tu('loading_transactions')}
            </TronLoader>
          )
        },
      },
    };
  }

  componentDidMount () {
    let {match} = this.props;
    this.loadAddress(match.params.id);
  }

  componentDidUpdate (prevProps) {
    let {match} = this.props;

    if (match.params.id !== prevProps.match.params.id) {
      this.loadAddress(match.params.id);
    }
  }

  async refreshAddress (id) {
    let {intl} = this.props;
    let address = await Client.getAddress(id);

    this.setState(prevProps => ({
      address,
      tabs: {
        ...prevProps.tabs,
        token_balances: {
          id: 'token_balances',
          icon: 'fa fa-piggy-bank',
          // path: "/token-balances",
          path: '',
          label: <span>{tu('token_balances')}</span>,
          cmp: () => <TokenBalances tokenBalances={address.balances} intl={intl}/>,
        },
      }
    }))
  }

  async loadAddress (id) {
    let {intl} = this.props;
    this.setState({loading: true, address: {address: id}, media: null,});

    const address = await Client.getAddress(id);
    const tokenBalances = mapTokenData(address.token_balances, address.token_map);
    const assetsBalances = mapTokenData(address.assets_balances, address.token_map, 'asset_id');

    let {total: totalProducedBlocks} = await Client.getBlocks({
      producer: id,
      limit: 1,
    });
    if (address.witness) {
      this.setState(() => ({
        loading: false,
        address,
        blocksProduced: totalProducedBlocks,
        // stats,
        tabs: {

          // ...prevProps.tabs,
          token_balances: {
            id: 'token_balances',
            // icon: "fa fa-piggy-bank",
            // path: "/token-balances",
            path: '',
            label: <span>{tu('token_balances')}</span>,
            cmp: () => <TokenBalances tokenBalances={tokenBalances} assetsBalances={assetsBalances} intl={intl}/>,
          },
          transfers: {
            id: 'transfers',
            // icon: "fa fa-exchange-alt",
            path: '/transfers',
            label: <span>{tu('transfers')}</span>,
            cmp: () => <Transfers filter={{address: id}} address/>
          },
          transfers20: {
            id: 'transfers20',
            // icon: "fa fa-exchange-alt",
            path: '/20transfers',
            label: <span>{tu('20_transfers')}</span>,
            cmp: () => <TransfersTrc20 filter={{address: id}}/>
          },
          transactions: {
            id: 'transactions',
            // icon: "fas fa-handshake",
            path: '/transactions',
            label: <span>{tu('transactions')}</span>,
            cmp: () => <NewTransactions filter={{address: id}} address/>
          },
          intransactions: {
            id: 'intransactions',
            // icon: "fas fa-handshake",
            path: '/internal-transactions',
            label: <span>{tu('internal_transactions')}</span>,
            cmp: () => <Transactions filter={{address: id}} isinternal/>
          },
          blocks_produced: {
            id: 'blocks-produced',
            // icon: "fa fa-cube",
            path: '/blocks',
            label: <span>{tu('produced_blocks')}</span>,
            cmp: () => <Blocks filter={{witness_address: id}} intl={intl}/>,
          },
          // votes: {
          //   id: 'votes',
          //   // icon: "fa fa-bullhorn",
          //   path: '/votes',
          //   label: <span>{tu('votes')}</span>,
          //   cmp: () => <Votes
          //     filter={{voter: id}}
          //     showVoter={false}
          //     showVoterPercentage={false}
          //   />,
          // },
          voters: {
            id: 'voters',
            // icon: "fa fa-bullhorn",
            path: '/voters',
            label: <span>{tu('voters')}</span>,
            cmp: () => <Votes
              filter={{candidate: id}}
              showCandidate={false}
            />,
          },
          rewards: {
            id: 'rewards',
            path: '/rewards',
            label: <span>{tu('rewards')}</span>,
            cmp: () => <Rewards filter={{address: id}} />
          }
        }
      }));
    } else {
      this.setState(() => ({
        loading: false,
        address,
        blocksProduced: totalProducedBlocks,
        // stats,
        tabs: {
          // ...prevProps.tabs,
          token_balances: {
            id: 'token_balances',
            // icon: "fa fa-piggy-bank",
            // path: "/token-balances",
            path: '',
            label: <span>{tu('token_balances')}</span>,
            cmp: () => <TokenBalances tokenBalances={tokenBalances} assetsBalances={assetsBalances} intl={intl}/>,
          },
          transfers: {
            id: 'transfers',
            // icon: "fa fa-exchange-alt",
            path: '/transfers',
            label: <span>{tu('transfers')}</span>,
            cmp: () => <Transfers filter={{address: id}} address/>
          },
          transfers20: {
            id: 'transfers20',
            // icon: "fa fa-exchange-alt",
            path: '/20transfers',
            label: <span>{tu('20_transfers')}</span>,
            cmp: () => <TransfersTrc20 filter={{address: id}}/>
          },
          transactions: {
            id: 'transactions',
            // icon: "fas fa-handshake",
            path: '/transactions',
            label: <span>{tu('transactions')}</span>,
            cmp: () => <NewTransactions filter={{address: id}} address/>
          },
          intransactions: {
            id: 'intransactions',
            // icon: "fas fa-handshake",
            path: '/internal-transactions',
            label: <span>{tu('internal_transactions')}</span>,
            cmp: () => <Transactions filter={{address: id}} isinternal address/>
          },
          // votes: {
          //   id: 'votes',
          //   // icon: "fa fa-bullhorn",
          //   path: '/votes',
          //   label: <span>{tu('votes')}</span>,
          //   cmp: () => <Votes
          //     filter={{voter: id}}
          //     showVoter={false}
          //     showVoterPercentage={false}
          //   />,
          // },
          rewards: {
            id: 'rewards',
            path: '/rewards',
            label: <span>{tu('rewards')}</span>,
            cmp: () => <Rewards filter={{address: id}} />
          }
        }
      }));
    }

    if (address) {
      try {
        // let totalPower = sentDelegateBandwidth + frozenBandwidth + sentDelegateResource + frozenEnergy;
        const frozenBandwidth = address.frozen_for_bandwidth && address.frozen_for_bandwidth.frozen_balance ? address.frozen_for_bandwidth.frozen_balance : 0;
        const frozenEnergy = address.frozen_for_energy && address.frozen_for_energy.frozen_balance ? address.frozen_for_energy.frozen_balance : 0;
        const stakeAmount = address.stake && address.stake.stake_amount ? address.stake.stake_amount : 0;
        const totalPower = frozenBandwidth + frozenEnergy + stakeAmount;

        const bandwidth = getAccountBandwidth(address);
        const energy = getAccountEnergy(address);

        this.setState({
          totalPower: totalPower,
          availableBalance: fromAmount(address.balance),
          totalBalance: fromAmount(address.balance + totalPower),
          bandwidthUsed: bandwidth.totalBandwidthUsed,
          bandwidthLimit: bandwidth.totalBandwidthLimit,
          bandwidthRemaining: bandwidth.bandwidthRemaining,
          bandWidthPercentage: bandwidth.bandWidthPercentage,
          availableBandWidthPercentage: bandwidth.availableBandWidthPercentage,
          totalBandwidthRemaining: bandwidth.totalBandwidthRemaining,
          energyUsed: energy.energyUsed,
          energyLimit: energy.energyLimit,
          energyRemaining: energy.energyRemaining,
          energyPercentage: energy.energyPercentage,
          availableEnergyPercentage: energy.availableEnergyPercentage,
        });
      } catch (err) {
        console.error(err)
      }
    }

  }

  bandWidthCircle = () => {
    let {bandwidthUsed, bandwidthLimit, totalBandwidthRemaining, bandWidthPercentage} = this.state;
    let {intl} = this.props;
    let address_percentage_remaining = new BigNumber(100 - Number(bandWidthPercentage));
    let addressPercentageRemaining = totalBandwidthRemaining ? address_percentage_remaining.decimalPlaces(2) + '%' : '0%';
    let address_percentage_used = new BigNumber(bandWidthPercentage);
    let addressPercentageUsed = bandwidthUsed ? address_percentage_used.decimalPlaces(2) + '%' : '0%';
    return (
      <div>
        <div>
          {intl.formatMessage({id: 'address_netLimit'}) + ' : ' + bandwidthLimit}
        </div>
        <div>
          <span>{intl.formatMessage({id: 'address_netRemaining'}) + ' : ' + totalBandwidthRemaining}</span>&nbsp;&nbsp;
          <span>{intl.formatMessage({id: 'address_percentage'}) + ' : ' + addressPercentageRemaining}</span>
        </div>
        <div>
          <span>{intl.formatMessage({id: 'address_netUsed'}) + ' : ' + bandwidthUsed}</span>&nbsp;&nbsp;
          <span>{intl.formatMessage({id: 'address_percentage'}) + ' : ' + addressPercentageUsed}</span>
        </div>
      </div>
    )

  }

  energyCircle = () => {
    let {energyLimit, energyRemaining, energyUsed, energyPercentage} = this.state;
    let {intl} = this.props;
    let address_percentage_remaining = new BigNumber(100 - Number(energyPercentage));
    let addressPercentageRemaining = energyRemaining ? address_percentage_remaining.decimalPlaces(2) + '%' : '0%';
    let address_percentage_used = new BigNumber(energyPercentage);
    let addressPercentageUsed = energyUsed ? address_percentage_used.decimalPlaces(2) + '%' : '0%';
    return (
      <div>
        <div>
          {intl.formatMessage({id: 'address_energyLimit'}) + ' : ' + energyLimit}
        </div>
        <div>
          <span>{intl.formatMessage({id: 'address_energyRemaining'}) + ' : ' + energyRemaining}</span>&nbsp;&nbsp;
          <span>{intl.formatMessage({id: 'address_percentage'}) + ' : ' + addressPercentageRemaining}</span>
        </div>
        <div>
          <span>{intl.formatMessage({id: 'address_energyUsed'}) + ' : ' + energyUsed}</span>&nbsp;&nbsp;
          <span>{intl.formatMessage({id: 'address_percentage'}) + ' : ' + addressPercentageUsed}</span>
        </div>
      </div>
    )
  }

  render () {
    let {address, tabs, loading, blocksProduced, rank, totalVotes,
      bandwidthRemaining, totalBandwidthRemaining, energyRemaining, totalBalance, availableBalance, availableBandWidthPercentage,
      availableEnergyPercentage} = this.state;
    let {match} = this.props;
    let addr = match.params.id;
    // let uploadURL = API_URL + '/api/v2/node/info_upload?address=' + match.params.id

    if (!address) {
      return null;
    }

    // let pathname = this.props.location.pathname;
    // let tabName = ''
    //let rex = /[a-zA-Z0-9]{34}\/?([a-zA-Z\\-]+)$/
    // pathname.replace(rex, function (a, b) {
    //   tabName = b
    // })
    return (
      <main className="container header-overlap">
        <div className="row">
          <div className="col-md-12 ">
            {
              loading ? (
                  <div className="card">
                    <TronLoader>
                      {tu('loading_address')} {address.address}
                    </TronLoader>
                  </div>
                ) : (
                <Fragment>
                  <div className="card list-style-header">
                    {
                      address.witness &&
                      <div className="card-body">
                        <h5 className="card-title m-0">
                          <i className="fa fa-cube mr-2"/>
                          {tu('witnesss')}
                        </h5>
                      </div>
                    }
                    <div className="row">

                      <div className="col-md-6">
                        <table className="table m-0">
                          <tbody>
                          {
                            Boolean(rank) &&
                            <tr>
                              <th>{tu('rank_real_time')}:</th>
                              <td>
                                {rank}
                              </td>
                            </tr>
                          }
                          {
                            address.witness && (
                              <Fragment>
                                {
                                  address.witness.name ? (
                                    <tr>
                                      <th>{tu('name')}:</th>
                                      <td>{address.witness.name}</td>
                                    </tr>
                                  ) : null
                                }
                                <tr>
                                  <th>{tu('website')}:</th>
                                  <td>
                                    <ExternalLink url={address.witness.url}/>
                                  </td>
                                </tr>
                                <tr>
                                  <th>{tu('blocks_produced')}:</th>
                                  <td>
                                    <FormattedNumber value={blocksProduced}/>
                                  </td>
                                </tr>
                              </Fragment>
                            )
                          }
                          <tr>
                            <th>{tu('address')}:</th>
                            <td>
                              <AddressLink address={addr} includeCopy={true}/>
                            </td>
                          </tr>
                          {
                            address.witness && address.witness.owner_address && (
                              <tr>
                                <th>Owner:</th>
                                <td>
                                  <AddressLink address={address.witness.owner_address} includeCopy={true}/>
                                </td>
                              </tr>
                            )
                          }
                          {!address.witness && (
                            <tr>
                              <th>{tu('name')}:</th>
                              <td>
                                <span>{address.account_name ? address.account_name : '-'}</span>
                              </td>
                            </tr>
                          )}
                          {/*<tr>*/}
                          {/*<th>*/}
                          {/*<span className="mr-1">{tu('address_info_transactions')}</span>*/}
                          {/*<QuestionMark placement="top" text="address_transactions_tip"/>*/}
                          {/*<span className="ml-1">:</span>*/}
                          {/*</th>*/}
                          {/*<td>*/}
                          {/*<span>{address.total_transaction_count}</span>*/}
                          {/*</td>*/}
                          {/*</tr>*/}
                          {/*<tr>*/}
                          {/*<th>{tu('address_info_transfers')}:</th>*/}
                          {/*<td>*/}
                          {/*<div className="d-flex">*/}
                          {/*<div>*/}
                          {/*{stats.transactions_in + stats.transactions_out}*/}
                          {/*</div>*/}
                          {/*<div>*/}
                          {/*<span className="ml-1">(</span>*/}
                          {/*<i className="fa fa-arrow-down text-success"/>&nbsp;*/}
                          {/*<span>{stats.transactions_in}</span>&nbsp;*/}
                          {/*<i className="fa fa-arrow-up  text-danger"/>&nbsp;*/}
                          {/*<span>{stats.transactions_out}</span>&nbsp;*/}
                          {/*<span>)</span>*/}
                          {/*</div>*/}
                          {/*</div>*/}
                          {/*</td>*/}
                          {/*</tr>*/}
                          {/*<tr>*/}
                          {/*<th>{tu("balance")}:</th>*/}
                          {/*<td>*/}
                          {/*<ul className="list-unstyled m-0">*/}
                          {/*<li>*/}
                          {/*<TRXPrice amount={address.balance / ONE_UNIT}/>*/}
                          {/*</li>*/}
                          {/*</ul>*/}
                          {/*</td>*/}
                          {/*</tr>*/}
                          {/*<tr>*/}
                          {/*<th>*/}
                          {/*<span className="mr-1">{  tu('tron_power')}</span>*/}
                          {/*<QuestionMark placement="top" text="address_tron_power_tip" className="ml-1"/>*/}
                          {/*<span className="ml-1">:</span>*/}
                          {/*</th>*/}
                          {/*<td>*/}
                          {/*<ul className="list-unstyled m-0">*/}
                          {/*<li className="d-flex">*/}
                          {/*<TRXPrice showCurreny={false}*/}
                          {/*amount={(totalPower) / ONE_UNIT}/>*/}
                          {/*<div>*/}
                          {/*<span*/}
                          {/*className="ml-1">(</span> {tu('address_tron_power_used')}: {address.voteTotal}&nbsp;   {tu('address_tron_power_remaining')}: {(totalPower) / ONE_UNIT - address.voteTotal}*/}
                          {/*<span>)</span>*/}
                          {/*</div>*/}
                          {/*</li>*/}
                          {/*</ul>*/}
                          {/*</td>*/}
                          {/*</tr>*/}
                          <tr>
                            <th>
                              <span className="mr-1">{ 'Balance' }</span>
                              {/*<QuestionMark placement="top" text="address_total_balance_tip" className="ml-1"/>*/}
                              <span className="ml-1">:</span>
                            </th>
                            <td>
                              <ul className="list-unstyled m-0">
                                <li>
                                  <div>
                                    <TRXPrice amount={availableBalance}/>&nbsp;
                                    <span className="small">(<TRXPrice
                                      amount={availableBalance}
                                      currency="USD"
                                      showPopup={false}/>)</span>
                                  </div>

                                  {/*<div>*/}
                                  {/*<span className="small">*/}
                                  {/*{tu('address_total_balance_info_sources')}：*/}
                                  {/*</span>*/}
                                  {/*<span className="small">*/}
                                  {/*<HrefLink*/}
                                  {/*href={*/}
                                  {/*intl.locale === 'zh'*/}
                                  {/*? 'https://trx.market/zh/'*/}
                                  {/*: 'https://trx.market/'*/}
                                  {/*}*/}
                                  {/*>TRXMarket</HrefLink>*/}
                                  {/*</span>*/}
                                  {/*<img width={15} height={15} style={{marginLeft: 5}}*/}
                                  {/*src={require('../../../images/svg/market.png')} alt=""/>*/}
                                  {/*</div>*/}
                                </li>
                              </ul>
                            </td>
                          </tr>
                          {
                            address.allowance ? (
                              <tr>
                                <th>{ 'Pending Reward:' }</th>
                                <td>
                                  <ul className="list-unstyled m-0">
                                    <li>
                                      <div>
                                        <TRXPrice amount={fromAmount(address.allowance)}/>&nbsp;
                                        <span className="small">(<TRXPrice
                                          amount={fromAmount(address.allowance)}
                                          currency="USD"
                                          showPopup={false}/>)</span>
                                      </div>
                                    </li>
                                  </ul>
                                </td>
                              </tr>
                            ) : null
                          }
                          {
                            Boolean(totalVotes) &&
                            <tr>
                              <th>{tu('total_votes')}:</th>
                              <td>
                                <ul className="list-unstyled m-0">
                                  <li>
                                    <FormattedNumber value={totalVotes}/>
                                  </li>
                                </ul>
                              </td>
                            </tr>
                          }
                          </tbody>
                        </table>

                      </div>
                      <div className="col-md-6 d-flex address-circle">
                        <div className="address-circle-bandwidth d-flex">
                          <Tooltip title={this.bandWidthCircle} overlayStyle={{'maxWidth': '500px'}}>
                            <Progress
                              width={82}
                              strokeWidth={10}
                              showInfo={false}
                              type="circle"
                              strokeColor="#FFA30B"
                              strokeLinecap="square"
                              percent={availableBandWidthPercentage}
                            />
                          </Tooltip>

                          <div className="circle-info">
                            <div>{tu('address_netRemaining')}</div>
                            <h2 style={(totalBandwidthRemaining >= 1e9 || energyRemaining >= 1e9) ? {fontSize: '1.2rem'} : {}}>
                              {!isNaN(totalBandwidthRemaining) ? <FormattedNumber value={totalBandwidthRemaining}/> : 'NaN'}
                            </h2>
                          </div>
                        </div>
                        <div className="address-circle-line"/>
                        <div className="address-circle-energy d-flex">
                          <Tooltip title={this.energyCircle} overlayStyle={{'maxWidth': '500px'}}>
                            <Progress
                              width={82}
                              strokeWidth={10}
                              showInfo={false}
                              type="circle"
                              strokeColor="#4A90E2"
                              strokeLinecap="square"
                              percent={availableEnergyPercentage}
                            />
                          </Tooltip>
                          <div className="circle-info">
                            <div>{tu('address_energyRemaining')}</div>
                            <h2 style={(totalBandwidthRemaining >= 1e9 || energyRemaining >= 1e9) ? {fontSize: '1.2rem'} : {}}>
                              {!isNaN(energyRemaining) ? <FormattedNumber value={energyRemaining}/> : 'NaN'}
                            </h2>
                          </div>
                        </div>
                      </div>
                      {
                        /*
                        <div className={address.witness ? 'col-md-6 mt-3 mt-md-0' : ''}>
                          {
                          address.witness && votes.length &&
                          <h4 className="text-center mt-3">Top {votes.length} {tu("voters")} {tu("addresses")}</h4>
                        }
                          {
                          address.witness &&
                        <PieReact style={{height: 340}} data={votes}/>
                        }
                        </div>
                       */
                      }
                    </div>
                  </div>
                  <div className="card mt-3 list-style-body">
                    <div className="card-header list-style-body__header">
                      <ul className="nav nav-tabs card-header-tabs">
                        {
                          Object.values(tabs).map(tab => (
                            <li key={tab.id} className="nav-item">
                              <NavLink exact to={match.url + tab.path} className="nav-link text-dark">
                                <i className={tab.icon + ' mr-2'}/>
                                {tab.label}
                              </NavLink>
                            </li>
                          ))
                        }
                      </ul>
                    </div>
                    <div className="card-body p-0 list-style-body__body">
                      <Switch>
                        {
                          Object.values(tabs).map(tab => (
                            <Route key={tab.id} exact path={match.url + tab.path}
                                   render={(props) => (<tab.cmp block={address}/>)}/>
                          ))
                        }
                      </Switch>
                    </div>

                  </div>
                  {/*{*/}
                  {/*tabName === '' ?*/}
                  {/*<div style={{marginTop: 20, float: 'right'}}><i size="1" style={{fontStyle: 'normal'}}>[*/}
                  {/*Download <a href={uploadURL} style={{color: '#C23631'}}><b>CSV Export</b></a>&nbsp;*/}
                  {/*<span className="glyphicon glyphicon-download-alt"/> ]</i>&nbsp;*/}
                  {/*</div> : null*/}
                  {/*}*/}

                </Fragment>
              )
            }
          </div>
        </div>
      </main>
    )
  }
}

// function mapStateToProps (state) {
//   return {
//     tokens20: state.account.tokens20,
//   };
// }
//
// const mapDispatchToProps = {};

export default injectIntl(Address);
//export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(Address));
