/* eslint-disable no-undef */
import React from 'react';
import {loadTokens} from '../../actions/tokens';
import {connect} from 'react-redux';
import TimeAgo from 'react-timeago';
import moment from 'moment';
import {FormattedNumber, injectIntl} from 'react-intl';
import {Client} from '../../services/api';
import {AddressLink, BlockNumberLink} from '../common/Links';
import SmartTable from '../common/SmartTable.js'
import {upperFirst} from 'lodash';
import {TronLoader} from '../common/loaders';
import TotalInfo from '../common/TableTotal';
import {CHAIN_SYMBOL, ONE_UNIT} from '../../constants';

class Blocks extends React.Component {
  _mounted = false

  constructor() {
    super();
    this.start = '';
    this.end = '';
    this.state = {
      loading: false,
      blocks: [],
      total: 0,
    };
  }

  componentDidMount() {
    this._mounted = true
    return this.loadBlocks();
  }

  componentWillUnmount() {
    this._mounted = false
  }

  onChange = (page, pageSize) => {
    this.loadBlocks(page, pageSize);
  };

  loadBlocks = async (page = 1, pageSize = 20) => {
    this.setState({
      loading: true,
      page: page,
      pageSize: pageSize,
    });
    let result = await Client.getBlocks({
      limit: pageSize,
      start: (page - 1) * pageSize,
    });
    if (!this._mounted) return
    this.setState({
      loading: false,
      blocks: result.items,
      total: result.total,
      rangeTotal: result['range_total']
    });
  };

  setProducedName = (record) => {
    let ele = null;
    if (record['witness_name']) {
      ele = <span>{record['witness_name']}</span>
    } else {
      if (record.number) {
        ele = <span>{record['witness_address']}</span>
      } else {
        ele = <span/>
      }
    }
    return ele
  }

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: upperFirst(intl.formatMessage({id: 'height'})),
        dataIndex: 'number',
        key: 'number',
        align: 'center',
        className: 'ant_table',
        width: '12%',
        render: (text) => {
          return <BlockNumberLink number={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'age'})),
        dataIndex: 'timestamp',
        key: 'timestamp',

        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <TimeAgo date={text} title={moment(text).format('MMM-DD-YYYY HH:mm:ss A')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'transactions'})),
        dataIndex: 'total_transactions',
        key: 'total_transactions',
        align: 'center',
        render: (text) => {
          return <FormattedNumber value={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'produced_by'})),
        dataIndex: 'witness_name',
        key: 'witness_name',
        align: 'left',
        width: '40%',
        className: 'ant_table',
        render: (text, record) => {
          //return <AddressLink address={text}/>
          return <AddressLink address={record['witness_address']}>{this.setProducedName(record)}</AddressLink>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'rewards'})),
        dataIndex: 'reward_per_blocks',
        key: 'reward_per_blocks',
        align: 'right',
        className: 'ant_table',
        render: (text) => {
          if (!text) return <span>-</span>
          return (
            <span><FormattedNumber value={text / ONE_UNIT}/>&nbsp;{CHAIN_SYMBOL}</span>
          )
        }
      },
      // {
      //   title: upperFirst(intl.formatMessage({id: 'bytes'})),
      //   dataIndex: 'size',
      //   key: 'size',
      //   align: 'center',
      //   className: 'ant_table',
      //   render: (text) => {
      //     return <FormattedNumber value={text}/>
      //   },
      // },
      // {
      //     title: upperFirst(intl.formatMessage({id: 'status'})),
      //     dataIndex: 'confirmed',
      //     key: 'confirmed',
      //     align: 'center',
      //     className: 'ant_table',
      //     render: (text, record, index) => {
      //         return record.confirmed?
      //             <span className="badge badge-success text-uppercase">{intl.formatMessage({id:'Confirmed'})}</span> :
      //             <span className="badge badge-danger text-uppercase">{intl.formatMessage({id: 'Unconfirmed'})}</span>
      //     },
      // }
    ]
  };

  // onDateOk (start, end) {
  //   this.start = start.valueOf();
  //   this.end = end.valueOf();
  //   let {page, pageSize} = this.state;
  //   this.loadBlocks(page, pageSize);
  // }

  render() {
    let {blocks, total, rangeTotal, loading} = this.state;
    let column = this.customizedColumn();
    return (
      <main className="container header-overlap pb-3 token_black">
        {loading && <div className="loading-style"><TronLoader/></div>}
        {
          <div className="row">
            <div className="col-md-12 table_pos">
              {total ? <TotalInfo total={total} rangeTotal={rangeTotal} typeText="block_unit"/> : ''}
              {/*<DateRange onDateOk={(start, end) => this.onDateOk(start, end)}/>*/}
              <SmartTable bordered={true} loading={loading} column={column} data={blocks} total={total}
                          onPageChange={(page, pageSize) => {
                            return this.loadBlocks(page, pageSize)
                          }}/>
            </div>
          </div>
        }
      </main>
    )
  }

}

function mapStateToProps() {
  return {};
}

const mapDispatchToProps = {
  loadTokens,
};

export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(Blocks));
