import React from 'react'
import { TronLoader } from '../../common/loaders'
import { AddressLink } from '../../common/Links'
import xhr from 'axios/index'
import { API_URL } from '../../../constants'
import { serializeQuery } from '../../../utils/helper'

class ReadContract extends React.Component {
  _mounted = false

  constructor () {
    super()
    this.state = {
      loading: false,
      data: []
    }
  }

  componentDidMount() {
    this._mounted = true
    this.getDataFromApi()
  }

  componentWillUnmount() {
    this._mounted = false
  }

  getDataFromApi = async () => {
    this.setState({
      loading: true
    })
    let {data} = await xhr.get(API_URL + '/api/contracts/' + self.contract + '/read')
    if (this._mounted) {
      this.setState({
        data: data.data,
        loading: false
      })
    }
  }

  callFunction = async (functionName, signature, inputElement, outputElement) => {
    let params = {
      functionName: functionName,
      signature: signature
    }
    let strParams = ''
    let elements = document.querySelectorAll('.' + inputElement)
    for (let i = 0; i < elements.length; i++) {
      if (i === 0) {
        strParams = this.add0xforAddress(elements[0].value)
      } else {
        strParams = strParams + ',' + elements[i].value
      }
      if (elements[i].value === '') {
        document.getElementsByClassName(inputElement)[0].focus()
        alert('Input value cannot be empty')
        return false
      }
    }
    params.strParams = strParams
    let query = serializeQuery(params)
    let output = await xhr.get(API_URL + '/api/contracts/' + this.contract + '/call/?' + query)
    if (this._mounted) {
      document.getElementById(outputElement).innerHTML =
        `<div class="tomo-contract-info__response">
          <p>[<strong>${functionName}</strong> method response]</p>${this.formatOuputs(output.data)}
        </div>`
    }
  }

  add0xforAddress (straddress) {
    straddress = straddress.trim()
    if (!straddress.startsWith('0x') && straddress.length === 40) {
      straddress = '0x' + straddress
    }
    return straddress
  }

  formatOuputs (output) {
    let response = ''
    for (let i = 0; i < output.length; i++) {
      response += '<p>'
      response += `<i class="response${output[i].name === 'Error' ? '--error' : '--success'}" />&nbsp;`
      response += `<strong>${output[i].name}</strong>&nbsp;
                    <em class="tomo-contract-info__type">${output[i].type}</em> :
                    <span>${this.formatResult(output[i].value, output[i].type)}</span>`
      response += '</p>'
    }
    return response
  }

  formatResult (strResult, resulttype) {
    if (resulttype.startsWith('uint')) {
      return this.formatNumber(strResult)
    } else if (resulttype === 'string') {
      return strResult
    } else if (resulttype === 'address') {
      return '<a href=\'/address/' + strResult + '\'>' + strResult + '</a>'
    } else {
      return strResult
    }
  }

  resetInput (e) {
    e.preventDefault()
    let inputs = document.querySelectorAll('.tomo-contract-info input')
    let ouputs = document.querySelectorAll('.tomo-contract-info__func.with-input .tomo-contract-info__result')
    for (let i = 0; i < inputs.length; i++) {
      inputs[i].value = ''
    }
    for (let i = 0; i < ouputs.length; i++) {
      ouputs[i].innerHTML = ''
    }
  }

  render () {
    const {loading, data} = this.state

    if (loading) {
      return <TronLoader/>
    }

    return (
      <div className='card tomo-card tomo-card--contract-info'>
        {
          data.length === 0 && (
            <div className="tomo-empty">
              <i className="fa fa-book tomo-empty__icon"/>
              <p className="tomo-empty__description">Can not read this smart contract</p>
            </div>
          )
        }

        <div className="tomo-card__header mb-3">
          <h6 className="d-md-inline-block"><i className="fa fa-book mr-2"/>Read Contract Information</h6>
          <a href="#" className="ml-md-5" onClick={this.resetInput}><i className="fa fa-refresh mr-1"/>Reset</a>
        </div>

        <table class="tomo-card__table tomo-contract-info">
          {
            data.map((func, index) => (
              <tr key={index}>
                <td className={`tomo-contract-info__func ${func.inputs.length ? 'with-input' : 'no-input'}`}>
                  <span class="tomo-contract-info__name">
                    {index + 1}. <i class="fa fa-caret-right"/> {func.name}&nbsp;
                  </span>
                  {
                    typeof func.result !== 'undefined' && !func.inputs.length && (
                      <i className="fa fa-long-arrow-right ml-2 mr-2"/>
                    )
                  }
                  {
                    typeof func.result !== 'undefined' && !func.inputs.length && (
                      <span className={'tomo-contract-info__result'}>
                        {
                          func.outputs[0].type === 'address' ? (
                            <AddressLink address={func.result}/>
                          ) : (
                            <span>{func.result === '' ? '\'\'' : func.result}</span>
                          )
                        }
                        <em class="tomo-contract-info__type">{func.outputs[0].type}</em>
                      </span>
                    )
                  }
                  {
                    func.inputs.length && (
                      <div className="tomo-contract-info__inputs">
                        {
                          func.inputs.map((input, idx) => (
                            <input
                              key={idx}
                              name={input.name}
                              placeholder={input.name + '(' + input.type + ')'}
                              className={'form-control mr-2 ' + func.name + '_' + index}
                              type="text"/>
                          ))
                        }
                        <button
                          class="btn btn-primary"
                          type="button"
                          onClick={() => this.callFunction(func.name, func.signature, func.name + '_' + index, 'output_' + func.name + '_' + index)}
                        >Call
                        </button>
                      </div>
                    )
                  }
                  {
                    func.outputs.length && typeof func.result === 'undefined' && (
                      <div className="tomo-contract-info__outputs">
                        <i class="fa fa-angle-double-right"/>
                        {
                          func.outputs.map((output, idx2) => (
                            <span key={idx2}>
                              <span class="tomo-contract-info__param-name">{output.name}</span>
                              <em class="tomo-contract-info__type">{output.type}</em>
                              {
                                idx2 < func.outputs.length - 1 && (<span>,&nbsp;</span>)
                              }
                            </span>
                          ))
                        }
                        <div
                          id={'output_' + func.name + '_' + index}
                          className={'tomo-contract-info__result'}
                        />
                      </div>
                    )
                  }
                </td>
              </tr>
            ))
          }
        </table>
      </div>
    )
  }

}

export default ReadContract
