/* eslint-disable no-undef */
import React, { Fragment } from 'react';
import { injectIntl } from 'react-intl';
import { NavLink, Route, Switch } from 'react-router-dom';
import { Client } from '../../../services/api';
import { tu } from '../../../utils/i18n';
import { AddressLink, TransactionHashLink, TokenTRC20Link } from '../../common/Links';
import { TRXPrice } from '../../common/Price';
import { ONE_UNIT } from '../../../constants';
import { TronLoader } from '../../common/loaders';
import Transactions from './Txs';
import InternalTransactions from '../../common/Transactions';
import Code from './Code';
// import Events from './Events';
import Transfers from './Transfers';
// import Energy from './Energy';
// import Call from './Call';
import { upperFirst } from 'lodash';
import { Truncate } from '../../common/text';
import { CONTRACT_ADDRESS_USDT } from '../../../constants';
import { Tooltip } from 'antd'
import moment from 'moment/moment';
import TimeAgo from 'react-timeago';

class SmartContract extends React.Component {

  constructor () {
    super();
    this.state = {
      showQrCode: false,
      loading: true,
      blocksProduced: 0,
      address: {
        address: '',
        balance: 0,
        tokenBalances: {},
      },
      token20: null
    }
  }

  componentDidMount () {
    let {match} = this.props;
    this.loadContract(match.params.id);
  }

  componentDidUpdate (prevProps) {
    let {match} = this.props;

    if (match.params.id !== prevProps.match.params.id) {
      this.loadContract(match.params.id);
    }
  }

  async loadContract (id) {
    this.setState({loading: true, address: {address: id}});
    let contract =  await Client.getContractByAddress(id, {'show_code': true});
    if (contract['token_type']) {
      this.setState(prevProps => ({
        loading: false,
        token20: contract,
        contract: contract,
        tabs: {
          ...prevProps.tabs,
          transactions: {
            id: 'transactions',
            path: '',
            label: <span>{tu('transactions')}</span>,
            cmp: () => <Transactions filter={{contract_address: id}}/>
          },
          intransactions: {
            id: 'intransactions',
            // icon: "fas fa-handshake",
            path: '/internal-transactions',
            label: <span>{tu('internal_transactions')}</span>,
            cmp: () => <InternalTransactions filter={{contract_address: id}} isinternal/>
          },
          // Txns: {
          //   id: "Txns",
          //   path: "/Txns",
          //   label: <span>{tu('token_txns')}</span>,
          //   cmp: () => <Txhash filter={{address: id}} />,
          // },
          Transfers: {
            id: 'Transfers',
            path: '/transfers',
            label: <span>{tu('TRC20_transfers')}</span>,
            cmp: () => <Transfers filter={{token: id}} token={contract}/>,
          },
          voters: {
            id: 'code',
            path: '/code',
            label: <span>{tu('Code')}</span>,
            cmp: () => <Code {...contract}/>,
          },
          // readContract: {
          //   id: 'readContract',
          //   path: '/read-contract',
          //   label: <span>{ tu('read_contract') }</span>,
          //   cmp: () => <Code filter={{address: id}}/>,
          // },
          // events: {
          //   id: 'events',
          //   path: '/events',
          //   label: <span>{tu('Events')}</span>,
          //   cmp: () => <Events filter={{address: id}}/>,
          // },
          // energy: {
          //   id: 'energy',
          //   path: '/energy',
          //   label: <span>{tu('energy')}</span>,
          //   cmp: () => <Energy filter={{address: id}}/>,
          // },
          // call: {
          //   id: 'call',
          //   path: '/call',
          //   label: <span>{tu('call')}</span>,
          //   cmp: () => <Call filter={{address: id}}/>,
          // }
        }
      }));
    } else {
      this.setState(prevProps => ({
        loading: false,
        token20: contract,
        contract: contract,
        tabs: {
          ...prevProps.tabs,
          transactions: {
            id: 'transactions',
            path: '',
            label: <span>{tu('transactions')}</span>,
            cmp: () => <Transactions filter={{contract_address: id}}/>
          },
          intransactions: {
            id: 'intransactions',
            // icon: "fas fa-handshake",
            path: '/internal-transactions',
            label: <span>{tu('internal_transactions')}</span>,
            cmp: () => <InternalTransactions filter={{contract_address: id}} isinternal/>
          },
          // Txns: {
          //   id: "Txns",
          //   path: "/Txns",
          //   label: <span>{tu('token_txns')}</span>,
          //   cmp: () => <Txhash filter={{address: id}} />,
          // },
          // Transfers: {
          //     id: "Transfers",
          //     path: "/transfers",
          //     label: <span>{tu('TRC20_transfers')}</span>,
          //     cmp: () => <Transfers filter={{token: id}} token={token20}/>,
          // },
          voters: {
            id: 'code',
            path: '/code',
            label: <span>{tu('Code')}</span>,
            cmp: () => <Code {...contract}/>,
          },
          // readContract: {
          //   id: 'readContract',
          //   path: '/read-contract',
          //   label: <span>{ tu('read_contract') }</span>,
          //   cmp: () => <Code filter={{address: id}}/>,
          // },
          // events: {
          //   id: 'events',
          //   path: '/events',
          //   label: <span>{tu('Events')}</span>,
          //   cmp: () => <Events filter={{address: id}}/>,
          // },
          // energy: {
          //   id: 'energy',
          //   path: '/energy',
          //   label: <span>{tu('energy')}</span>,
          //   cmp: () => <Energy filter={{address: id}}/>,
          // },
          // call: {
          //   id: 'call',
          //   path: '/call',
          //   label: <span>{tu('call')}</span>,
          //   cmp: () => <Call filter={{address: id}}/>,
          // }
        }
      }));
    }
  }

  render () {
    let {contract, tabs, loading, token20} = this.state;
    let {match, intl} = this.props;

    if (!contract) {
      return null;
    }

    return (
      <main className="container header-overlap token_black">
        <div className="row">
          <div className="col-md-12 ">
            {
              loading ? (
                  <div className="card">
                    <TronLoader>
                      {tu('loading_address')} {contract.address}
                    </TronLoader>
                  </div>
                ) :
                <Fragment>
                  <div className="card list-style-header">
                    <div className="contract-header">
                      <h6><AddressLink address={contract.address} isContract={true} includeCopy={true}/></h6>

                      <div className="d-flex contract-header_list">
                        <div className="contract-header__item">
                          <h6 className="contract-header__title">{tu('contract_overview')}</h6>
                          <ul>
                            <li><p>{upperFirst(intl.formatMessage({id: 'balance'}))}: </p><TRXPrice
                              amount={parseInt(contract.balance) / ONE_UNIT}/></li>
                            {/* <li><p>{tu('trx_value')}: </p><TRXPrice amount={1} currency="USD" source="home"/></li> */}
                            {/*<li><p>{upperFirst(intl.formatMessage({id: 'transactions'}))}: </p>*/}
                            {/*<p className="contract_trx_count">*/}
                            {/*{contract.trxCount}*/}
                            {/*<Tooltip placement="top" title={intl.formatMessage({id: 'Normal_Transactions'})}>*/}
                            {/*<span className="ml-1"> txns </span>*/}
                            {/*</Tooltip>*/}
                            {/*</p>*/}
                            {/*</li>*/}
                            {
                              token20 && (
                                <li>
                                  <p>{tu('token_tracker')}: </p>
                                  <img width={20} height={20} src={token20['logo_url']} style={{marginRight: 10}}/>
                                  <TokenTRC20Link name={token20.name} address={token20.address}
                                                  namePlus={token20.name + ' (' + token20.symbol + ')'}/>
                                </li>
                              )
                            }
                          </ul>
                        </div>
                        <div className="contract-header__item">
                          <h6 className="contract-header__title">{tu('Misc')}</h6>
                          <ul>
                            <li>
                              <p>{tu('contract_creator')}:</p>
                              {contract.creator &&
                              <div className="d-flex">
                                    <span style={{width: '30%'}}>
                                    <Truncate><AddressLink address={contract.creator.owner_address}/></Truncate></span>
                                <span className="px-1">{tu('at_txn')}</span>
                                <span style={{width: '30%'}}>
                                    <Truncate>
                                      <TransactionHashLink
                                        hash={contract.creator['transaction_hash']}>{contract.creator['transaction_hash']}</TransactionHashLink>
                                      </Truncate>
                                    </span>
                              </div>
                              }
                            </li>
                            <li>
                              <p>{ 'Created Time:' }</p>
                              {
                                contract['created_time'] && (
                                  <div className="d-flex">
                                    <TimeAgo
                                      date={contract['created_time']}
                                      title={moment(contract['created_time']).format('MMM-DD-YYYY HH:mm:ss A')}
                                    />
                                  </div>
                                )
                              }
                            </li>
                          </ul>
                        </div>
                      </div>
                    </div>
                  </div>
                  <div className="card mt-3 list-style-body">
                    <div className="card-header list-style-body__header">
                      <ul className="nav nav-tabs card-header-tabs">
                        {
                          Object.values(tabs).map(tab => (
                            <li key={tab.id} className="nav-item">
                              <NavLink exact to={match.url + tab.path} className="nav-link text-dark">
                                <i className={tab.icon + ' mr-2'}/>
                                {tab.label}
                              </NavLink>
                            </li>
                          ))
                        }
                      </ul>
                    </div>
                    <div className="card-body p-0 list-style-body__body">
                      <Switch>
                        {
                          Object.values(tabs).map(tab => (
                            <Route key={tab.id} exact path={match.url + tab.path}
                                   render={(props) => (<tab.cmp/>)}/>
                          ))
                        }
                      </Switch>
                    </div>
                  </div>
                </Fragment>
            }
          </div>
        </div>
      </main>
    )
  }
}

export default injectIntl(SmartContract);
