/* eslint-disable no-undef */
import React from 'react';
import { injectIntl } from 'react-intl';
import { loadTokens } from '../../actions/tokens';
import { connect } from 'react-redux';
import TimeAgo from 'react-timeago';
import { Client } from '../../services/api';
import { AddressLink, BlockNumberLink, TransactionHashLink } from '../common/Links';
import { getQueryParams } from '../../utils/url';
import { Truncate } from '../common/text';
import { upperFirst } from 'lodash';
import SmartTable from '../common/SmartTable.js'
import { TronLoader } from '../common/loaders';
import { TRXPrice } from '../common/Price';
import TotalInfo from '../common/TableTotal';
// import DateRange from '../common/DateRange';
import moment from 'moment';
import { fromAmount } from '../../utils/helper';

class ContractTrans extends React.Component {

  constructor () {
    super();
    // this.start = moment([2018, 5, 25]).startOf('day').valueOf();
    // this.end = moment().valueOf();
    this.state = {
      transactions: [],
      total: 0,
    };
  }

  // componentWillReceiveProps() {
  //   setTimeout(() => {
  //     this.loadTriggers();
  //   }, 0)
  // }

  componentDidMount () {
    this.loadTriggers();
  }

  componentDidUpdate () {
    // checkPageChanged(this, this.loadTriggers);
  }

  onChange = (page, pageSize) => {
    this.loadTriggers(page, pageSize);
  };

  loadTriggers = async (page = 1, pageSize = 20) => {
    let {location, match} = this.props;
    let date_to = match.params.date;
    let date_start = parseInt(match.params.date) - 24 * 60 * 60 * 1000;

    this.setState(
      {
        loading: true,
        page: page,
        pageSize: pageSize,
      }
    );

    let searchParams = {};

    for (let [key, value] of Object.entries(getQueryParams(location))) {
      switch (key) {
        case 'address':
        case 'block':
          searchParams[key] = value;
          break;
        default:
          break;
      }
    }
    let result = null;
    if (date_start) {
      result = await Client.getContractTriggers({
        date_start: date_start,
        date_to: date_to
      });
    }
    else {
      result = await Client.getContractTriggers({
        limit: pageSize,
        start: (page - 1) * pageSize,
        // start_timestamp: this.start,
        // end_timestamp: this.end,
        ...searchParams,
      });
    }
    this.setState({
      transactions: result.items,
      loading: false,
      total: result.total,
      rangeTotal: result.range_total,
    });
  };

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: '#',
        dataIndex: 'hash',
        key: 'hash',
        align: 'left',
        className: 'ant_table',
        width: '12%',
        render: (text) => {
          return <Truncate>
            <TransactionHashLink hash={text}>{text}</TransactionHashLink>
          </Truncate>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'block'})),
        dataIndex: 'block_number',
        key: 'block_number',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <BlockNumberLink number={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'age'})),
        dataIndex: 'timestamp',
        key: 'timestamp',
        align: 'left',
        render: (text) => {
          return <TimeAgo date={text} title={moment(text).format('MMM-DD-YYYY HH:mm:ss A')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'contract_triggers_owner'})),
        dataIndex: 'owner_address',
        key: 'owner_address',
        align: 'left',
        width: '30%',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'contract_address'})),
        dataIndex: 'to_address',
        key: 'to_address',
        align: 'left',
        width: '30%',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text} isContract={true}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'value'})),
        dataIndex: 'amount',
        key: 'amount',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <TRXPrice amount={fromAmount(text)}/>
        }
      },

    ];
  };

  render () {
    let {transactions, total, rangeTotal, loading} = this.state;
    let column = this.customizedColumn();
    return (
      <main className="container header-overlap pb-3 token_black">
        {loading && <div className="loading-style"><TronLoader/></div>}
        <div className="row">
          <div className="col-md-12 table_pos">
            {total ? <TotalInfo total={total} rangeTotal={rangeTotal} typeText="contract_triggers_total"
                                markName="table-question-mark-triggers"/> : ''}
            {/*{*/}
            {/*total ? <DateRange onDateOk={(start, end) => this.onDateOk(start, end)}/> : ''*/}
            {/*}*/}
            <SmartTable bordered={true} loading={loading}
                        column={column} data={transactions} total={total}
                        onPageChange={(page, pageSize) => {
                          this.loadTriggers(page, pageSize)
                        }}/>
          </div>
        </div>
      </main>
    )
  }
}

function mapStateToProps () {
  return {};
}

const mapDispatchToProps = {
  loadTokens,
};

export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(ContractTrans));
