/* eslint-disable no-undef */
import React from 'react';
import {injectIntl} from 'react-intl';
import {loadTokens} from '../../actions/tokens';
import {connect} from 'react-redux';
import TimeAgo from 'react-timeago';
import {Client} from '../../services/api';
import {AddressLink, BlockNumberLink, TransactionHashLink} from '../common/Links';
import {getQueryParams} from '../../utils/url';
import {Truncate} from '../common/text';
import {ContractTypes} from '../../utils/protocol';
import {upperFirst} from 'lodash';
import SmartTable from '../common/SmartTable.js'
import {TronLoader} from '../common/loaders';
import TotalInfo from '../common/TableTotal';
import moment from 'moment';

class Transactions extends React.Component {
  constructor() {
    super();
    this.state = {
      transactions: [],
      total: 0,
    };
  }


  componentDidMount() {
    return this.loadTransactions();
  }

  componentDidUpdate(prevProps) {
    let {location} = this.props;
    if (location.search !== prevProps.location.search) {
      return this.loadTransactions();
    }
  }

  onChange = (page, pageSize) => {
    return this.loadTransactions(page, pageSize);
  };

  loadTransactions = async (page = 1, pageSize = 20) => {
    let {location} = this.props;
    this.setState({
      loading: true,
      page: page,
      pageSize: pageSize,
    });

    let searchParams = {};
    for (let [key, value] of Object.entries(getQueryParams(location))) {
      switch (key) {
        case 'address':
        case 'block_number':
          searchParams[key] = value;
          break;
        default:
          break;
      }
    }
    const result = await Client.getTransactions({
      limit: pageSize,
      start: (page - 1) * pageSize,
      ...searchParams,
    });
    this.setState({addressLock: true});


    this.setState({
      transactions: result.items,
      loading: false,
      total: result.total,
      rangeTotal: result['range_total'],
    });
  };

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: '#',
        dataIndex: 'hash',
        key: 'hash',
        align: 'left',
        className: 'ant_table',
        width: '12%',
        render: (text) => {
          return <Truncate>
            <TransactionHashLink hash={text}>{text}</TransactionHashLink>
          </Truncate>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'block'})),
        dataIndex: 'block_number',
        key: 'block_number',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <BlockNumberLink number={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'created'})),
        dataIndex: 'timestamp',
        key: 'timestamp',
        align: 'left',
        render: (text) => {
          return <TimeAgo date={text} title={moment(text).format('MMM-DD-YYYY HH:mm:ss A')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'address'})),
        dataIndex: 'owner_address',
        key: 'owner_address',
        align: 'left',
        width: '30%',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'contract'})),
        dataIndex: 'contract_type',
        key: 'contract_type',
        align: 'right',
        className: 'ant_table',
        render: (text) => {
          return <span>{text}</span>
        },
      },
    ];
  }


  render() {
    let {transactions, total, rangeTotal, loading, addressLock} = this.state;
    let column = this.customizedColumn();
    return (
      <main className="container header-overlap pb-3 token_black">
        {loading && <div className="loading-style"><TronLoader/></div>}
        <div className="row">
          <div className="col-md-12 table_pos">
            {total ?
              <TotalInfo total={total} rangeTotal={rangeTotal} typeText="transactions_unit" common={addressLock}/> : ''}
            <SmartTable bordered={true} loading={loading}
                        column={column} data={transactions} total={total}
                        onPageChange={(page, pageSize) => {
                          this.loadTransactions(page, pageSize)
                        }}/>
          </div>
        </div>
      </main>
    )
  }
}

function mapStateToProps() {
  return {};
}

const mapDispatchToProps = {
  loadTokens,
};

export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(Transactions));
