import React, { Component } from 'react';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { withTimers } from '../../utils/timing';
import { HrefLink } from '../common/Links';
// import isMobile from '../../utils/isMobile';
import { Client } from '../../services/api';
import { TRXPrice } from './Price';
import { fromAmount } from '../../utils/helper';

class Footer extends Component {

  constructor () {
    super();
    this.state = {
      data: {}
    };
  }

  componentDidMount() {
    this.getSystemBalance();
  }

  getSystemBalance() {
    Client.getSystemBalance()
      .then(result => {
        if (result) {
          this.setState({
            data: result
          })
        }
      })
  }

  render () {
    const { data = {} } = this.state;
    const socials = [
      {url: 'https://t.me/mcashchain', icon: 'fab fa-telegram'},
      {url: 'https://twitter.com/mcashchain', icon: 'fab fa-twitter'},
      {url: 'https://medium.com/mcashchain', icon: 'fab fa-medium-m'},
    ];

    return (
      <footer className="home pb-0">
        <div className="pt-5 home-footer">
          <div className="container">
            <div className="row">
              <div className="col-md-6 col-xs-12">
                <p>MCashScan © 2019 - <a href="javascript:">v0.1.0</a></p>
              </div>
              <div className="col-md-6 col-xs-12">
                <div className='c-list-group c-list-group__horizontal justify-content-md-end home-footer__social'>
                  {socials.map((item, index) => (
                    <div key={index} className='c-list-group__item mr-3 mr-md-0 ml-md-4'>
                      <HrefLink href={item.url}>
                        <i className={`${item.icon} fa-2x`} />
                      </HrefLink>
                    </div>
                  ))}
                </div>
              </div>
            </div>
          </div>
        </div>

        <div className="footer-grid mb-5">
          <div className="container">
            <div className="row justify-content-md-between">
              <div className="col-lg-5 d-none d-md-block"/>
              <div className="col-lg-3 col-6">
                <h5>Company</h5>
                <div className="footer-grid__list">
                  <a href="https://mcash.network/" target="_blank">About Us</a>
                </div>
              </div>
              <div className="col-lg-3 col-6">
                <h5>Supply</h5>
                <div className="footer-grid__list">
                  <a href="javascript:void(0)">Total supply: <TRXPrice amount={data.total_supply ? fromAmount(data.total_supply) : 0} decimalsFormatter={0} /></a>
                  <a href="javascript:void(0)">Total stake: <TRXPrice amount={data.total_stake ? fromAmount(data.total_stake) : 0} decimalsFormatter={0} /></a>
                  <a href="javascript:void(0)">Circulation: <TRXPrice amount={data.circulating_supply ? fromAmount(data.circulating_supply) : 0} decimalsFormatter={0} /></a>
                </div>
              </div>
            </div>
          </div>
        </div>
      </footer>
    );
  }
}

function mapStateToProps (state) {
  return {
    activeLanguage: state.app.activeLanguage
  };
}

const mapDispatchToProps = {};

export default connect(
  mapStateToProps,
  mapDispatchToProps
)(withTimers(injectIntl(Footer)));
