import React from 'react';
import xhr from 'axios/index';
import { FormattedNumber } from 'react-intl';
import { BigNumber } from 'bignumber.js'
// import { Tooltip } from 'reactstrap';
// import { alpha } from '../../utils/str';
import { connect } from 'react-redux';
import { CHAIN_DECIMAL, CHAIN_SYMBOL } from '../../constants';
import { fromAmount, showAmount } from '../../utils/helper';

let PriceContext = React.createContext({
  priceBTC: 0,
  priceUSD: 0
});

let {Provider, Consumer} = PriceContext;

class PriceProviderCmp extends React.PureComponent {
  constructor (props) {
    super(props);

    this.state = {
      prices: {
        BTC: 0,
        EUR: 0,
        USD: 0,
        MCASH: 1
      },
      priceShown: props.activeCurrency || CHAIN_SYMBOL,
      currencies: {},
      activePrice: 1
    };

    for (let currency of props.currencies) {
      this.state.currencies[currency.id.toUpperCase()] = currency;
    }
  }

  componentWillReceiveProps (nextProps) {
    this.setState(state => ({
      priceShown: nextProps.activeCurrency,
      activePrice: state.prices[nextProps.activeCurrency.toUpperCase()]
    }));
  }

  async loadPrices () {
    let {data} = await xhr.get(
      `https://api.coinmarketcap.com/v1/ticker/tronix/?convert=EUR`
    );
    let {data: dataEth} = await xhr.get(
      `https://api.coinmarketcap.com/v1/ticker/tronix/?convert=ETH`
    );

    let newPrices = {
      BTC: parseFloat(data[0].price_btc),
      EUR: parseFloat(data[0].price_eur),
      USD: parseFloat(data[0].price_usd),
      ETH: parseFloat(dataEth[0].price_eth),
      MCASH: 1
    };

    this.setState(state => ({
      prices: newPrices,
      activePrice: newPrices[state.priceShown.toUpperCase()]
    }));
  }

  componentDidMount () {
    // this.loadPrices();
  }

  render () {
    let {children} = this.props;

    return <Provider value={this.state}>{children}</Provider>;
  }
}

function mapStateToProps (state) {
  return {
    activeCurrency: state.app.activeCurrency,
    currencies: state.app.currencyConversions
  };
}

const mapDispatchToProps = {};

export let PriceProvider = connect(
  mapStateToProps,
  mapDispatchToProps
)(PriceProviderCmp);

export class TRXPrice extends React.PureComponent {

  constructor () {
    super();
    this.state = {
      // open: false,
      // id: alpha(24),
      priceExchangeData: null,
      toCode: 'USD'
    };
  }
  //
  // renderPrice (value, priceValues) {
  //   let {currency = ''} = this.props;
  //   let id = currency || priceValues.priceShown;
  //   return priceValues.prices[id.toUpperCase()] * value;
  // }

  componentDidMount() {
    if (this.props.currency === this.state.toCode) {
      this.loadPrice()
    }
  }

  async loadPrice() {
    const fromCode = CHAIN_SYMBOL
    const toCode = this.state.toCode
    const {data} = await xhr.get(`https://api.midasprotocol.com/token/v1/prices?fromCodes=${fromCode}&toCodes=${toCode}`)
    const res = data ? data.data : {}
    const items = res ? res.items : []
    if (items.length > 0) {
      const priceData = items[0] && items[0].rates && items[0].rates[toCode]
      this.setState({
        priceExchangeData: priceData ? priceData : {}
      })
    }
  }

  render () {
    // let {open, id} = this.state;
    let {
      source,
      name,
      amount = 0,
      currency = '',
      showCurreny = true,
      showPopup = true,
      valueRange = false,
      decimalsFormatter,
      ...props
    } = this.props;
    // let ele = null;
    const {priceExchangeData, toCode} = this.state
    let canExchange = true

    if (currency === toCode) {
      canExchange = priceExchangeData && typeof priceExchangeData.rate !== 'undefined'
      amount = canExchange ? new BigNumber(amount).times(priceExchangeData.rate || 1).toFixed() : 0
    }

    //=======================
    switch (source) {
      case 'transfers':
        return (
          <FormattedNumber
            value={amount}
            maximumFractionDigits={12}
          >
            {value => (
              <span {...props}>
                {name === CHAIN_SYMBOL ? fromAmount(amount) : amount} {name}
              </span>
            )}
          </FormattedNumber>
        )
      case 'home':
        return (
          <FormattedNumber
            value={amount}
            maximumFractionDigits={3}
          >
            {value => (
              <span {...props}>{canExchange ? value : '-'}&nbsp;{(showCurreny && currency.toUpperCase()) || CHAIN_SYMBOL}</span>
            )}
          </FormattedNumber>
        )
      default:
        if (valueRange) {
          return <span {...props}>{canExchange ? showAmount(amount, CHAIN_DECIMAL) : '-'}&nbsp;{(showCurreny && currency.toUpperCase()) || CHAIN_SYMBOL}</span>
        }
        return (
          <FormattedNumber
            value={amount}
            maximumFractionDigits={typeof decimalsFormatter !== 'undefined' ? decimalsFormatter : CHAIN_DECIMAL}
          >
            {value => (
              <span {...props}>{canExchange ? value : '-'}&nbsp;{(showCurreny && currency.toUpperCase()) || CHAIN_SYMBOL}</span>
            )}
          </FormattedNumber>
        )
    }
    //=======================

    // return (
    //   <Consumer>
    //     {priceValues => {
    //       switch (source) {
    //         case 'transfers':
    //           ele = (
    //             <Fragment>
    //               <FormattedNumber
    //                 value={this.renderPrice(amount, priceValues)}
    //                 maximumFractionDigits={12}
    //               >
    //                 {value => (
    //                   <span
    //                     id={id}
    //                     onMouseEnter={() => this.setState({open: true})}
    //                     onMouseLeave={() => this.setState({open: false})}
    //                     {...props}
    //                   >
    //                     {name === CHAIN_SYMBOL ? amount / ONE_UNIT : amount} {name}
    //                   </span>
    //                 )}
    //               </FormattedNumber>
    //               {showPopup && (
    //                 <Tooltip placement="top" isOpen={open} target={id}>
    //                   {CHAIN_SYMBOL + ' '}
    //                   <FormattedNumber
    //                     value={amount / ONE_UNIT}
    //                     maximumFractionDigits={6}
    //                     minimumFractionDigits={6}
    //                   />{' '}
    //                   <br/>
    //                   BTC{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.BTC * (amount / ONE_UNIT)}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.BTC.fractions || 2
    //                     }
    //                   />
    //                   <br/>
    //                   ETH{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.ETH * (amount / ONE_UNIT)}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.ETH.fractions || 2
    //                     }
    //                   />
    //                   <br/>
    //                   USD{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.USD * (amount / ONE_UNIT)}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.USD.fractions || 3
    //                     }
    //                   />
    //                   <br/>
    //                   EUR{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.EUR * (amount / ONE_UNIT)}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.EUR.fractions || 2
    //                     }
    //                   />
    //                 </Tooltip>
    //               )}
    //             </Fragment>
    //           );
    //           break;
    //         case 'home':
    //           ele = (
    //             <Fragment>
    //               <FormattedNumber
    //                 value={this.renderPrice(amount, priceValues)}
    //                 maximumFractionDigits={
    //                   priceValues.currencies[
    //                   currency.toUpperCase() ||
    //                   priceValues.priceShown.toUpperCase()
    //                     ].fractions || 3
    //                 }
    //               >
    //                 {value => (
    //                   <span
    //                     id={id}
    //                     onMouseOver={() => this.setState({open: true})}
    //                     onMouseOut={() => this.setState({open: false})}
    //                     {...props}
    //                   >
    //                     {value}{' '}
    //                     {showCurreny && (currency.toUpperCase() || priceValues.priceShown.toUpperCase())}
    //                   </span>
    //                 )}
    //               </FormattedNumber>
    //               {showPopup && (
    //                 <Tooltip placement="top" isOpen={open} target={id}>
    //                   {CHAIN_SYMBOL + ' '}
    //                   <FormattedNumber
    //                     value={amount}
    //                     maximumFractionDigits={6}
    //                     minimumFractionDigits={6}
    //                   />{' '}
    //                   <br/>
    //                   <span className="text-capitalize">satoshi</span>{' '}
    //                   <FormattedNumber
    //                     value={
    //                       priceValues.prices.BTC * amount * Math.pow(10, 8)
    //                     }
    //                     maximumFractionDigits={
    //                       priceValues.currencies.BTC.fractions || 2
    //                     }
    //                   />
    //                   <br/>
    //                   ETH{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.ETH * amount}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.ETH.fractions || 2
    //                     }
    //                   />
    //                   <br/>
    //                   USD{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.USD * amount}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.USD.fractions || 3
    //                     }
    //                   />
    //                   <br/>
    //                   EUR{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.EUR * amount}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.EUR.fractions || 6
    //                     }
    //                   />
    //                 </Tooltip>
    //               )}
    //             </Fragment>
    //           );
    //           break;
    //         default:
    //           ele = (
    //             <Fragment>
    //               <FormattedNumber
    //                 value={this.renderPrice(amount, priceValues)}
    //                 maximumFractionDigits={
    //                   priceValues.currencies[
    //                   currency.toUpperCase() ||
    //                   priceValues.priceShown.toUpperCase()
    //                     ].fractions || 2
    //                 }
    //               >
    //                 {value => (
    //                   <span
    //                     id={id}
    //                     onMouseEnter={() => this.setState({open: true})}
    //                     onMouseLeave={() => this.setState({open: false})}
    //                     {...props}
    //                   >
    //                     {value}{' '}
    //                     {showCurreny && (currency.toUpperCase() || priceValues.priceShown.toUpperCase())}
    //                   </span>
    //                 )}
    //               </FormattedNumber>
    //               {showPopup && (
    //                 <Tooltip placement="top" isOpen={open} target={id}>
    //                   {CHAIN_SYMBOL + ' '}
    //                   <FormattedNumber
    //                     value={amount}
    //                     maximumFractionDigits={6}
    //                     minimumFractionDigits={6}
    //                   />{' '}
    //                   <br/>
    //                   BTC{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.BTC * amount}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.BTC.fractions || 2
    //                     }
    //                   />
    //                   <br/>
    //                   ETH{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.ETH * amount}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.ETH.fractions || 2
    //                     }
    //                   />
    //                   <br/>
    //                   USD{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.USD * amount}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.USD.fractions || 3
    //                     }
    //                   />
    //                   <br/>
    //                   EUR{' '}
    //                   <FormattedNumber
    //                     value={priceValues.prices.EUR * amount}
    //                     maximumFractionDigits={
    //                       priceValues.currencies.EUR.fractions || 2
    //                     }
    //                   />
    //                 </Tooltip>
    //               )}
    //             </Fragment>
    //           );
    //       }
    //       return ele;
    //     }}
    //   </Consumer>
    // );
  }
}
