import React from 'react'
import _, { trim } from 'lodash';
import { injectIntl } from 'react-intl';
import { withRouter } from 'react-router-dom';
import { Truncate } from './text';
// import $ from 'jquery';
import xhr from 'axios/index';
import { API_URL } from '../../constants';
import { toastr } from 'react-redux-toastr';
import { validateAddress } from '../../utils/helper';

const styles = {
  search: {
    fontSize: 13,
    minWidth: 260,
    height: 36,
  },
  searchType: {
    fontSize: 13,
  }
};

class SearchBar extends React.Component {

  constructor () {
    super();
    this.callAjax = _.debounce(this.callAjax, 500);
    this.isSearching = true;
    this.state = {
      search: '',
      searchResults: [],
      popup: null,
      isShowResults: false
    };
  }

  onSearchKeyDown = (ev) => {
    if (ev.keyCode === 13) {
      this.doSearch();
      // $('#_searchBox').css({display: 'none'});
      this.setState({
        isShowResults: false
      })
    }
  }

  doSearch = async () => {
    this.isSearching = true;
    let {intl} = this.props;
    let {searchResults} = this.state;
    if (searchResults && searchResults.length) {
      if (searchResults[0].desc === 'Block') {
        this.afterSearch('#/block/' + _.trim(searchResults[0].value));
      }
      if (searchResults[0].desc === 'Token-TRC20') {
        this.afterSearch('#/token20/' + _.trim(searchResults[0].value.split(' ')[searchResults[0].value.split(' ').length - 1]));
      }
      if (searchResults[0].desc === 'Token-TRC10') {
        this.afterSearch('#/token/' + _.trim(searchResults[0].value.split(' ')[searchResults[0].value.split(' ').length - 1]));
      }
      if (searchResults[0].desc === 'Address' && validateAddress(_.trim(searchResults[0].value))) {
        this.afterSearch('#/address/' + _.trim(searchResults[0].value));
      }
      if (searchResults[0].desc === 'Contract') {
        this.afterSearch('#/contract/' + _.trim(searchResults[0].value));
      }
      if (searchResults[0].desc === 'TxHash') {
        this.afterSearch('#/transaction/' + _.trim(searchResults[0].value));
      }
    }
    else {
      toastr.warning(intl.formatMessage({id: 'warning'}), intl.formatMessage({id: 'search_not_found'}));
    }
  };

  onSearchChange = (ev) => {
    this.setState({search: ev.target.value});
    ev.persist();
    this.callAjax(ev.target.value);
  }

  callAjax = async (value) => {
    this.isSearching = true;
    let {search} = this.state;
    if (search === '') {
      this.setState({
        searchResults: [],
        isShowResults: false
      });
      // $('#_searchBox').css({display: 'none'});
      return;
    }

    let result = await xhr.get(API_URL + '/api/search?term=' + _.trim(search));
    let results = result.data ? result.data.data : [];
    this.isSearching = false;
    /*let results = [
      {desc: 'Token-TRC10', value: "IGG 1000029"},
      {desc: 'Token-TRC20', value: "IGG 1000029"},
      {desc: 'Block', value: "1000029"},
      {desc: 'Address', value: "TVethjgashn8t4cwKWfGA3VvSgMwVmHKNM"},
      {desc: 'Contract', value: "TVethjgashn8t4cwKWfGA3VvSgMwVmHKNM"},
      {desc: 'TxHash', value: "9073aca5dfacd63c8e61f6174c98ab3f350bc9365df6ffc3bc7a70a252711d6f"}
    ];*/

    this.setState({
      searchResults: results || [],
      isShowResults: results.length > 0
    });
    // if (results.length) {
    //   $('#_searchBox').css({display: 'block'});
    // } else {
    //   $('#_searchBox').css({display: 'none'});
    // }
  }

  afterSearch = (hash) => {
    window.location.hash = hash;
    this.setState({
      searchResults: [],
      search: ''
    });
  }

  render() {
    let {intl} = this.props;
    const {search, searchResults, isShowResults} = this.state;
    return (
      <div className="hidden-mobile nav-searchbar">
        <div className="input-group dropdown">
          <input type="text"
                 className="form-control p-2 bg-white border-0 box-shadow-none"
                 style={styles.search}
                 value={search}
                 onKeyDown={this.onSearchKeyDown}
                 onChange={this.onSearchChange}
                 onClick={this.callAjax}
                 placeholder={intl.formatMessage({id: 'search_description1'})}/>
          <div className="input-group-append" style={{marginLeft: 0}}>
            <button className="btn box-shadow-none" disabled={this.isSearching} onClick={this.doSearch}>
              <i className="fa fa-search"/>
            </button>
          </div>
          <div className="dropdown-menu" id="_searchBox"
               style={{width: '100%', maxHeight: '300px', overflow: 'auto', display: isShowResults ? 'block' : 'none'}}>
            {
              Array.isArray(searchResults) && searchResults.map((result, index) => {
                  if (result.desc === 'Block') {
                    return <a className="dropdown-item text-uppercase" onClick={() => {
                      this.afterSearch('#/block/' + _.trim(result.value))
                    }}
                              key={index}>{result.desc + ': '}<Truncate><strong>{result.value}</strong></Truncate></a>
                  }
                  if (result.desc === 'Token-TRC20') {
                    return <a className="dropdown-item text-uppercase" onClick={() => {
                      this.afterSearch('#/token20/' + _.trim(result.value.split(' ')[result.value.split(' ').length - 1]))
                    }}
                              key={index}>{result.desc + ': '}<Truncate><strong>{result.value}</strong></Truncate></a>
                  }
                  if (result.desc === 'Token-TRC10') {
                    return <a className="dropdown-item text-uppercase" onClick={() => {
                      this.afterSearch('#/token/' + _.trim(result.value.split(' ')[result.value.split(' ').length - 1]))
                    }}
                              key={index}>{result.desc + ': '}<Truncate><strong>{result.value}</strong></Truncate></a>
                  }
                  if (result.desc === 'Address') {
                    if (validateAddress(trim(result.value))) {
                      return <a className="dropdown-item text-uppercase" onClick={() => {
                        this.afterSearch('#/address/' + trim(result.value))
                      }}
                                key={index}>{result.desc + ': '}<Truncate><strong>{result.value}</strong></Truncate></a>
                    } else {
                      return (
                        <span className="dropdown-item text-uppercase" key={index}>
                          <Truncate><strong>{result.value}</strong></Truncate>
                        </span>
                      )
                    }
                  }
                  if (result.desc === 'Contract') {
                    return <a className="dropdown-item text-uppercase" onClick={() => {
                      this.afterSearch('#/contract/' + _.trim(result.value))
                    }}
                              key={index}>{result.desc + ': '}<Truncate><strong>{result.value}</strong></Truncate></a>
                  }
                  if (result.desc === 'TxHash') {
                    return <a className="dropdown-item text-uppercase" onClick={() => {
                      this.afterSearch('#/transaction/' + _.trim(result.value))
                    }}
                              key={index}>{result.desc + ': '}<Truncate><strong>{result.value}</strong></Truncate></a>
                  }
                  return ''
                }
              )
            }
          </div>
        </div>
      </div>
    )
  }

}

export default withRouter(injectIntl(SearchBar))
