import React from 'react';
import { injectIntl } from 'react-intl';
import { Client } from '../../services/api';
import { TransactionHashLink, AddressLink } from './Links';
import { tu } from '../../utils/i18n';
import TimeAgo from 'react-timeago';
import { TronLoader } from './loaders';
import { Truncate } from './text';
import SmartTable from './SmartTable.js'
import { upperFirst } from 'lodash';
import TotalInfo from './TableTotal';
import moment from 'moment';
import { mapAssetTokenData } from '../../utils/helper';
import { FormatNumberByDecimals } from '../../utils/number';

class Transactions extends React.Component {

  constructor (props) {
    super(props);
    this.start = moment([2018, 5, 25]).startOf('day').valueOf();
    this.end = moment().valueOf();
    this.state = {
      filter: {},
      transactions: [],
      total: 0,
      emptyState: props.EmptyState || (
        <TronLoader>
          Loading Transactions
        </TronLoader>
      )
    };
  }

  componentDidMount () {
    this.loadTransactions();
  }

  componentDidUpdate (prevProps) {
    let {filter, page} = this.props;
    if (prevProps.filter.address !== filter.address && page.router === 'account') {
      this.loadTransactions();
    }
  }

  onChange = (page, pageSize) => {
    this.loadTransactions(page, pageSize);
  };

  loadTransactions = async (page = 1, pageSize = 20) => {
    let {filter, isinternal = false} = this.props;

    this.setState({
      loading: true,
      page: page,
      pageSize: pageSize,
    });

    let transactions, total, rangeTotal = 0;

    if (!isinternal) {
      const result = await Client.getTransactions({
        limit: pageSize,
        start: (page - 1) * pageSize,
        total: this.state.total,
        ...filter,
      });
      transactions = result.items;
      total = result.total
      rangeTotal = result.range_total

    } else {
      // TODO internal transactions

      const result = await Client.getInternalTransaction({
        limit: pageSize,
        start: (page - 1) * pageSize,
        ...filter
      });

      transactions = mapAssetTokenData(result);
      total = result.total
      rangeTotal = result.range_total
    }

    this.setState({
      transactions,
      total,
      rangeTotal,
      loading: false,
    });
  };

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: upperFirst(intl.formatMessage({id: 'hash'})),
        dataIndex: 'hash',
        key: 'hash',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <Truncate>
            <TransactionHashLink hash={text}>
              {text}
            </TransactionHashLink>
          </Truncate>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'age'})),
        dataIndex: 'timestamp',
        key: 'timestamp',
        align: 'left',
        className: 'ant_table',
        width: '14%',
        render: (text) => {
          return <TimeAgo date={text} title={moment(text).format('MMM-DD-YYYY HH:mm:ss A')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'contract_type'})),
        dataIndex: 'contract_type',
        key: 'contract_type',
        align: 'right',
        width: '20%',
        className: 'ant_table _text_nowrap',
        render: (text) => {
          return <span>{text}</span>
        }
      },
      // {
      //     title: upperFirst(intl.formatMessage({id: 'status'})),
      //     dataIndex: 'confirmed',
      //     key: 'confirmed',
      //     align: 'center',
      //     className: 'ant_table',
      //     render: (text, record, index) => {
      //         return record.confirmed?
      //             <span className="badge badge-success text-uppercase">{intl.formatMessage({id:'Confirmed'})}</span> :
      //             <span className="badge badge-danger text-uppercase">{intl.formatMessage({id: 'Unconfirmed'})}</span>
      //     },
      // }
    ];
  }

  token20CustomizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: upperFirst(intl.formatMessage({id: 'hash'})),
        dataIndex: 'transaction_hash',
        key: 'transaction_hash',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return (
            <Truncate>
              <TransactionHashLink hash={text}>
                {text}
              </TransactionHashLink>
            </Truncate>
          )
        }
      },
      // {
      //   title: upperFirst(intl.formatMessage({id: 'age'})),
      //   dataIndex: 'timestamp',
      //   key: 'timestamp',
      //   align: 'left',
      //   className: 'ant_table',
      //   width: '14%',
      //   render: (text, record, index) => {
      //     return <TimeAgo date={text}/>
      //   }
      // },
      {
        title: upperFirst(intl.formatMessage({id: 'from'})),
        dataIndex: 'from_address',
        key: 'from_address',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        }
      },
      {
        title: '',
        className: 'ant_table',
        width: '30px',
        render: () => {
          return <img src={require('../../images/arrow.png')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'to'})),
        dataIndex: 'to_address',
        key: 'to_address',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'trc20_my_trans_header_status'})),
        dataIndex: 'rejected',
        key: 'rejected',
        align: 'left',
        className: 'ant_table _text_nowrap',
        render: (text) => {
          return (
            <span>
              {
                text ? <img style={{width: '20px', height: '20px'}} src={require('../../images/internal_error.png')}/> :
                  <img style={{width: '20px', height: '20px'}} src={require('../../images/internal_success.png')}/>
              }
            </span>
          )
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'amount'})),
        dataIndex: 'amount',
        key: 'amount',
        align: 'right',
        className: 'ant_table _text_nowrap',
        render: (text, record) => {
          // return record.valueInfoList && record.valueInfoList.map((item, index) => {
          //   return (
          //     <span key={index}>
          //       <NameWithId value={item}/>
          //       <span className={index === record.valueInfoList.length - 1 ? 'd-none' : ''}>, </span>
          //     </span>
          //   )
          // })
          const token = record.token ? record.token : {}
          return (
            <span>{FormatNumberByDecimals(text, token.decimals) + ' ' + token.symbol}</span>
          )
        }
      },
      // {
      //   title: upperFirst(intl.formatMessage({id: 'contract_type'})),
      //   dataIndex: 'contractType',
      //   key: 'contractType',
      //   align: 'right',
      //   width: '20%',
      //   className: 'ant_table _text_nowrap',
      //   render: (text, record, index) => {
      //     return <span>{ContractTypes[text]}</span>
      //   }
      // },
    ];
  }

  // onDateOk (start, end) {
  //   this.start = start.valueOf();
  //   this.end = end.valueOf();
  //   let {page, pageSize} = this.state;
  //   this.loadTransactions(page, pageSize);
  // }

  render () {
    let {transactions, total, rangeTotal, loading} = this.state;
    let {isinternal, address = false} = this.props;
    let column = !isinternal ? this.customizedColumn() : this.token20CustomizedColumn();
    // let tableInfo = intl.formatMessage({id: 'view_total'}) + ' ' + total + ' ' + intl.formatMessage({id: 'transactions_unit'})

    // if (!loading && transactions && transactions.length === 0) {
    //   if (!EmptyState) {
    //     return (
    //         <div className="p-3 text-center no-data">{tu("no_transactions")}</div>
    //     );
    //   }
    //   return <EmptyState/>;
    // }

    return (
      <div className={'token_black table_pos ' + (address ? 'mt-5' : '')}>
        {loading && <div className="loading-style"><TronLoader/></div>}
        {total ? <TotalInfo total={total} rangeTotal={rangeTotal} typeText="transactions_unit" common={!address}/> : ''}
        {/*{*/}
        {/*address ?*/}
        {/*<DateRange onDateOk={(start, end) => this.onDateOk(start, end)} dateClass="date-range-box-address"/> : ''*/}
        {/*}*/}
        {
          (!loading && transactions.length === 0) ?
            <div className="p-3 text-center no-data">{tu('no_transactions')}</div> :
            <SmartTable bordered={true} loading={loading} column={column} data={transactions} total={total}
                        onPageChange={(page, pageSize) => {
                          this.loadTransactions(page, pageSize)
                        }}/>
        }

      </div>
    )
  }
}

export default (injectIntl(Transactions));
