import React from 'react';
import { injectIntl } from 'react-intl';
import { Client } from '../../services/api';
import { AddressLink, TransactionHashLink } from './Links';
import { tu } from '../../utils/i18n';
import TimeAgo from 'react-timeago';
import { Truncate } from './text';
import { withTimers } from '../../utils/timing';
import SmartTable from './SmartTable.js'
import { upperFirst, upperCase } from 'lodash';
import { TronLoader } from './loaders';
import TotalInfo from './TableTotal';
import moment from 'moment';
import { NameWithId } from './names';
import { CHAIN_SYMBOL } from '../../constants';
import { mapAssetTokenData } from '../../utils/helper';

class Transfers extends React.Component {

  constructor (props) {
    super(props);
    this.start = moment([2018, 5, 25]).startOf('day').valueOf();
    this.end = moment().valueOf();
    this.state = {
      filter: {},
      transfers: [],
      page: 1,
      total: 0,
      pageSize: 20,
      showTotal: props.showTotal !== false,
      emptyState: props.emptyState,
      autoRefresh: props.autoRefresh || false,
      hideSmallCurrency: false,
      tokenName: ''
    };
  }

  componentDidMount () {
    let {page, pageSize} = this.state;
    this.load(page, pageSize);

    if (this.state.autoRefresh !== false) {
      this.props.setInterval(() => this.load(page, pageSize), this.state.autoRefresh);
    }
  }

  onChange = (page, pageSize) => {
    this.setState({
      page: page,
      pageSize: pageSize
    }, () => {
      this.load(page, pageSize);
    });

  };

  load = async (page, pageSize) => {
    let {filter} = this.props;
    let {hideSmallCurrency, tokenName} = this.state;
    this.setState(
      {
        loading: true,
        page: page,
        pageSize: pageSize,
      }
    );
    const result = await Client.getTransfers({
      limit: pageSize,
      start: (page - 1) * pageSize,
      total: this.state.total,
      token: tokenName,
      ...filter,
    });

    const rebuildTransfers = mapAssetTokenData(result);
    rebuildTransfers.map(item => {
      if (filter.address) {
        item.fromtip = !(item.owner_address === filter.address)
        item.totip = !(item.to_address === filter.address)
      } else {
        item.fromtip = true
        item.totip = true
      }
      return item
    })

    let transfersData;
    if (hideSmallCurrency) {
      // transfersData = _(rebuildTransfers)
      //   .filter(tb => tb.tokenName === '_' || upperCase(tb.tokenName) === CHAIN_SYMBOL)
      //   .value();
      transfersData = rebuildTransfers.filter(tb => tb.token_name === '_' || upperCase(tb.token_name) === CHAIN_SYMBOL)
    } else {
      transfersData = rebuildTransfers
    }

    this.setState({
      page,
      transfers: transfersData,
      total: result.total,
      rangeTotal: result.range_total,
      loading: false,
    });
  };

  handleSwitch = (val) => {
    if (val) {
      this.setState({
        hideSmallCurrency: val,
        tokenName: '_',
      }, () => {
        this.load(1, 20);
      });
    } else {
      this.setState({
        hideSmallCurrency: val,
        tokenName: '',
      }, () => {
        this.load(1, 20);
      });
    }
  }

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: upperFirst(intl.formatMessage({id: 'hash'})),
        dataIndex: 'hash',
        key: 'hash',
        align: 'left',
        width: '20%',
        className: 'ant_table',
        render: (text) => {
          return (
            <Truncate>
              <TransactionHashLink hash={text}>{text}</TransactionHashLink>
            </Truncate>
          )
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'age'})),
        dataIndex: 'timestamp',
        key: 'timestamp',
        align: 'left',
        className: 'ant_table',
        width: '14%',
        render: (text) => {
          return <TimeAgo date={text} title={moment(text).format('MMM-DD-YYYY HH:mm:ss A')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'from'})),
        dataIndex: 'owner_address',
        key: 'owner_address',
        align: 'left',
        className: 'ant_table',
        render: (text, record) => {
          return record.fromtip ?
            <AddressLink address={text}/> :
            <Truncate><span>{text}</span></Truncate>
        }
      },
      {
        title: '',
        className: 'ant_table',
        width: '30px',
        render: () => {
          return <img src={require('../../images/arrow.png')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'to'})),
        dataIndex: 'to_address',
        key: 'to_address',
        align: 'left',
        className: 'ant_table',
        render: (text, record) => {
          return record.totip ?
            <AddressLink address={text}/> :
            <Truncate><span>{text}</span></Truncate>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'amount'})),
        dataIndex: 'amount',
        key: 'amount',
        align: 'right',
        className: 'ant_table _text_nowrap',
        render: (text, record) => {
          return <NameWithId value={record}/>
        }
      },
      // {
      //     title: upperFirst(intl.formatMessage({id: 'status'})),
      //     dataIndex: 'confirmed',
      //     key: 'confirmed',
      //     align: 'center',
      //     className: 'ant_table',
      //     render: (text, record, index) => {
      //         return record.confirmed?
      //             <span className="badge badge-success text-uppercase">{intl.formatMessage({id:'Confirmed'})}</span> :
      //             <span className="badge badge-danger text-uppercase">{intl.formatMessage({id: 'Unconfirmed'})}</span>
      //     },
      // }
    ];
  }

  // onDateOk (start, end) {
  //   this.start = start.valueOf();
  //   this.end = end.valueOf();
  //   let {page, pageSize} = this.state;
  //   this.load(page, pageSize);
  // }

  render () {
    // let {transfers, total, rangeTotal = 0, loading, emptyState: EmptyState = null} = this.state;
    let {transfers, total, rangeTotal = 0, loading} = this.state;
    let column = this.customizedColumn();
    let {intl, address = false} = this.props;

    // let tableInfo = intl.formatMessage({id: 'view_total'}) + ' ' + total + ' ' + intl.formatMessage({id: 'transfers_unit'})
    let locale = {emptyText: intl.formatMessage({id: 'no_transfers'})}
    // if (!loading && transfers.length === 0) {
    //   if (!EmptyState) {
    //     return (
    //         <div className="p-3 text-center no-data">{tu("no_transfers")}</div>
    //     );
    //   }
    //
    //   return <EmptyState/>;
    // }

    return (
      <div className="token_black table_pos">
        {loading && <div className="loading-style"><TronLoader/></div>}
        {
          transfers.length ? (
            <div className="d-flex justify-content-between" style={{left: 'auto'}}>
              <TotalInfo total={total} rangeTotal={rangeTotal} typeText="transactions_unit" common={!address}
                         divClass="table_pos_info_addr"/>
              {/*<div className="table_pos_switch d-md-block table_pos_switch_addr table_pos_switch_addr_transfers">*/}
              {/*<SwitchToken handleSwitch={this.handleSwitch} text={`Only ${CHAIN_SYMBOL} transfers`} isHide={false}/>*/}
              {/*</div>*/}
              {/*{address && (*/}
              {/*<div className="transactions-rangePicker table_pos_picker transfers_pos_picker"*/}
              {/*style={{width: '360px'}}>*/}
              {/*<DateRange onDateOk={(start, end) => this.onDateOk(start, end)}*/}
              {/*dateClass="date-range-box-address-transfer"/>*/}
              {/*</div>*/}
              {/*)}*/}
            </div>
          ) : (
            <div className="d-flex justify-content-between" style={{left: 'auto'}}>
              {/*<div className="table_pos_info d-md-block table_pos_info_addr2">{tableInfo}<span> <QuestionMark placement="top" text="to_provide_a_better_experience"></QuestionMark></span></div>*/}
              <TotalInfo total={total} rangeTotal={rangeTotal} typeText="transactions_unit" common={!address}
                         divClass="table_pos_info_addr2"/>
              {/*<div className="table_pos_switch d-md-block table_pos_switch_addr2">*/}
              {/*<SwitchToken handleSwitch={this.handleSwitch} text={`Only ${CHAIN_SYMBOL} transfers`} isHide={false}/>*/}
              {/*</div>*/}
              {/*{address && (*/}
              {/*<div className="transactions-rangePicker table_pos_picker" style={{width: '360px'}}>*/}
              {/*<DateRange onDateOk={(start, end) => this.onDateOk(start, end)}*/}
              {/*dateClass="date-range-box-address-nodata"/>*/}
              {/*</div>*/}
              {/*)}*/}
            </div>
          )
        }

        {
          (!loading && transfers.length === 0) ?
            <div className="p-3 text-center no-data">{tu('no_transfers')}</div>
            :
            <SmartTable bordered={true} loading={loading} column={column} data={transfers} total={total} locale={locale}
                        addr="address" transfers="address"
                        onPageChange={(page, pageSize) => {
                          this.onChange(page, pageSize)
                        }}/>
        }
      </div>
    )
  }
}

export default withTimers(injectIntl(Transfers));
