import React, { Component } from 'react';
import { FormattedNumber, injectIntl } from 'react-intl';
import { Client } from '../../../services/api';
import { tu } from '../../../utils/i18n';
import { TokenLink, TokenTRC20Link } from '../../common/Links';
import { upperFirst } from 'lodash';
import { TronLoader } from '../../common/loaders';
import { withTronWeb } from '../../../utils/mcashWeb';
import { Table, Radio } from 'antd';
import TotalInfo from '../../common/TableTotal';

@withTronWeb
class TokenList extends Component {

  constructor (props) {
    super(props);
    this.state = {
      tokens: [],
      loading: false,
      total: 0,
      filter: {
        type: 'all'
      },
      pagination: {
        showQuickJumper: true,
        position: 'both',
        showSizeChanger: true,
        defaultPageSize: 20,
        total: 0
      },
    };
  }

  loadPage = async (page = 1, pageSize = 20) => {
    this.setState({loading: true})
    const {filter} = this.state
    const result = await Client.getTokens({
      start: (page - 1) * pageSize,
      limit: pageSize,
      ...filter
    });

    let count = 0
    result.items.forEach((item, index) => {
      if (!item.isTop) {
        item.index = count + 1
        count++
      } else {
        item.c_index = index + 1
      }

      // if (typeof item.gain !== 'undefined') {
      //   item.gain = item.gain * 10000
      //   if (item.gain < 0) {
      //     item.color = 'col-red'
      //     item.gain = parseInt(item.gain) / 100 + '%'
      //   }
      //   if (item.gain > 0) {
      //     item.color = 'col-green'
      //     item.gain = '+' + parseInt(item.gain) / 100 + '%'
      //   }
      //   if (item.gain === 0) {
      //     item.color = 'col-green'
      //     item.gain = parseInt(item.gain) / 100 + '%'
      //   }
      // } else {
      //   item.gain = '-'
      // }
    })

    this.setState({
      loading: false,
      tokens: result.items,
      pagination: {
        ...this.state.pagination,
        total: result.total
      },
      total: result.total
    });
  };

  componentDidMount () {
    this.loadPage();
  }

  onChange = (e) => {
    this.setState({
      filter: {
        ...this.state.filter,
        type: e.target.value
      }
    }, () => this.loadPage())
  };

  handleTableChange = (pagination, filters, sorter) => {
    const pager = {...this.state.pagination};
    pager.current = pagination.current;
    pager.pageSize = pagination.pageSize;

    const map = {
      descend: 'desc',
      ascend: 'asc'
    }
    const sortMap = {
      nrOfTokenHolders: 'holderCount',
      gain: 'gain',
      priceInTrx: 'priceInTrx',
      volume24hInTrx: 'volume24hInTrx',
      marketcap: 'marketcap'
    }
    this.setState({
      pagination: pager,
      filter: {
        ...this.state.filter,
        sort: sortMap[sorter.columnKey] || 'marketcap',
        order: map[sorter.order] || 'desc',
        order_current: sorter.order
      }
    }, () => this.loadPage(pager.current, pager.pageSize));
  }

  customizedColumn = () => {
    let {filter} = this.state
    let {intl} = this.props;
    return [
      {
        title: '#',
        dataIndex: 'index',
        key: 'index',
        width: '48px',
        align: 'center',
        className: 'ant_table _text_nowrap',
        render: (text, record) => {
          return (
            <span>
              {
                record.isTop ? (
                    <div>
                      <span className="starbeat"><i className="fas fa-star"/> </span>
                      <span className="star-tip"/>
                    </div>
                  ) : (<span>{text}</span>)
              }
            </span>
          )
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'token'})),
        dataIndex: 'name',
        key: 'name',
        width: '50%',
        render: (text, record) => {
          return <div className="table-imgtext">
            {record.logo_url ?
              <div style={{width: '42px', height: '42px', marginRight: '18px'}}>
                <img style={{width: '42px', height: '42px'}} src={record.logo_url}/>
              </div> :
              <div style={{width: '42px', height: '42px', marginRight: '18px'}}>
                <img style={{width: '42px', height: '42px'}} src={require('../../../images/logo_default.png')}/>
              </div>
            }

            <div>
              <h5>
                <div>
                  {
                    record.token_type === 'M1' &&
                    <TokenLink
                      name={record.name}
                      id={record.asset_id}
                      namePlus={record.name + (record.symbol ? ' (' + record.symbol + ')' : '')}/>
                  }
                  {
                    record.token_type === 'M20' &&
                    <TokenTRC20Link
                      name={record.name}
                      namePlus={record.name + (record.symbol ? ' (' + record.symbol + ')' : '')}
                      address={record.contract_address}/>
                  }
                </div>
              </h5>
              <p style={{wordBreak: 'break-all'}}>{record.description}</p>
            </div>
          </div>
        }
      },
      // {
      //   title: intl.formatMessage({id: 'price'}) + ' (MCASH)',
      //   dataIndex: 'priceInTrx',
      //   key: 'priceInTrx',
      //   sorter: true,
      //   sortOrder: filter.sort === 'priceInTrx' && filter.order_current,
      //   align: 'center',
      //   className: 'ant_table d-none d-md-table-cell _text_nowrap'
      // },
      // {
      //   title: intl.formatMessage({id: 'gain'}),
      //   sorter: true,
      //   sortOrder: filter.sort === 'gain' && filter.order_current,
      //   dataIndex: 'gain',
      //   key: 'gain',
      //   render: (text, record, index) => {
      //     return <div className={record.color}>{record.gain}</div>
      //   },
      //   align: 'center',
      //   className: 'ant_table d-none d-md-table-cell _text_nowrap'
      // },
      // {
      //   title: intl.formatMessage({id: 'volume_24_trx'}),
      //   dataIndex: 'volume24hInTrx',
      //   key: 'volume24hInTrx',
      //   align: 'center',
      //   className: 'ant_table',
      //   sorter: true,
      //   sortOrder: filter.sort === 'volume24hInTrx' && filter.order_current,
      //   render: (text) => {
      //     return text > 0 ? <FormattedNumber value={text} maximumFractionDigits={2}/> : '-'
      //   }
      // },
      // {
      //   title: intl.formatMessage({id: 'market_capitalization_trx'}),
      //   dataIndex: 'marketcap',
      //   key: 'marketcap',
      //   sorter: true,
      //   sortOrder: filter.sort === 'marketcap' && filter.order_current,
      //   render: (text) => {
      //     return text > 0 ? <FormattedNumber value={text}/> : '-'
      //   },
      //   align: 'center',
      //   className: 'ant_table _text_nowrap'
      // },
      {
        title: intl.formatMessage({id: 'token_holders'}),
        dataIndex: 'total_holders',
        key: 'total_holders',
        sorter: true,
        sortOrder: filter.sort === 'holderCount' && filter.order_current,
        render: (text) => {
          return text > 0 ? <FormattedNumber value={text}/> : '-'
        },
        align: 'center',
        className: 'ant_table d-none d-sm-table-cell'
      }
    ];
  }

  render () {
    let {tokens, alert, loading, total, filter} = this.state;
    let {intl} = this.props;
    let column = this.customizedColumn();
    // let tableInfo = intl.formatMessage({id: 'part_total'}) + ' ' + total + '/' + totalAll + ' ' + intl.formatMessage({id: 'part_pass'})
    let url = 'https://trx.market/launchBase?utm_source=TS3'
    if (intl.locale === 'zh') {
      url = 'https://trx.market/zh/launchBase?utm_source=TS3'
    }
    return (
      <main className="container header-overlap token_black">
        {alert}
        {loading && <div className="loading-style"><TronLoader/></div>}
        {
          <div className="row">
            <div className="col-md-12 table_pos trc20-ad-bg pt-5 pt-md-0">
              {total ? <TotalInfo total={total} rangeTotal={total} typeText="token_unit"/> : ''}
              {/*{total ?*/}
              {/*<div className="table_pos_info d-none d-md-block" style={{left: 'auto'}}>*/}
              {/*<div>*/}
              {/*{tableInfo}&nbsp;*/}
              {/*<QuestionMark placement="top" text="newly_issued_token_by_tronscan"*/}
              {/*className="token-list-info"/>*/}
              {/*&nbsp;&nbsp;*/}
              {/*/!*<Link to="/exchange/trc10">{t('Trade_on_TRXMarket')}></Link>*!/*/}
              {/*</div>*/}
              {/*</div> : ''}*/}
              <div className="d-md-flex apply-trc20 apply-all align-items-center">
                {/*<div className="d-flex align-items-center mr-md-4 mb-2 mb-md-0">*/}
                <div className="d-flex align-items-center mb-2 mb-md-0">
                  <Radio.Group size="Small" value={filter.type} onChange={this.onChange}>
                    <Radio.Button value="all">{tu('all')}</Radio.Button>
                    <Radio.Button value="M1">M1</Radio.Button>
                    <Radio.Button value="M20">M20</Radio.Button>
                  </Radio.Group>
                </div>
                {/*<a className="ml-md-2" href="https://goo.gl/forms/PiyLiDeaXv3uesSE3" target="_blank">*/}
                {/*<button className="btn btn-default" style={{lineHeight: '18px'}}>*/}
                {/*{tu('application_entry')}*/}
                {/*</button>*/}
                {/*</a>*/}
              </div>

              <Table
                columns={column}
                rowKey={(record, index) => index}
                dataSource={tokens}
                loading={loading}
                onChange={this.handleTableChange}
                pagination={this.state.pagination}
                bordered={true}
                rowClassName={record => {
                  if (record.isTop) {
                    return 'trc20-star-ad'
                  }
                }}
              />
            </div>
          </div>
        }
      </main>

    )
  }
}

export default injectIntl(TokenList)
