import React from 'react';
import { tu } from '../../../utils/i18n';
import { FormattedNumber } from 'react-intl';
import { AddressLink, ExternalLink } from '../../common/Links';
import { Link } from 'react-router-dom';
import { toLower } from 'lodash';
import moment from 'moment';

export function Information ({token, currentTotalSupply}) {
  let social_display = 0;
  // let lowerText = token.reputation ? toLower(token.reputation) + '_active.png' : '';
  let issuer_address = <AddressLink address={token.owner_address} includeCopy={true}/>
  token && token['social_media'] && token['social_media'].forEach((media, index) => {
    if (media.url) {
      social_display++;
    }
  })
  // let issued = token.decimals ? token.issued / Math.pow(10, token.decimals) : token.issued
  // let currentTotal = token.id == 1002000 ? currentTotalSupply : issued;
  let currentTotal = token.decimals ? token.issued / Math.pow(10, token.decimals) : token.issued

  const tokenList = [
    {
      name: 'total_supply',
      content: <FormattedNumber
        value={token.decimals ? token.total_supply / Math.pow(10, token.decimals) : token.total_supply}/>
    }, {
      name: 'ID',
      content: <span>{token.asset_id}</span>
    }, {
      name: 'TRC20_decimals',
      content: <span>{token.decimals === 0 || token.decimals ? token.decimals : '_'}</span>
    }, {
      name: 'reputation',
      content: (
        <Link to={`/rating`} style={{display: 'flex', alignItems: 'center'}}>
          {token.reputation ? tu(toLower(token.reputation)) : ''}
        </Link>
      )
    }, {
      name: 'circulating_supply',
      content: <span>{currentTotal ? <FormattedNumber value={currentTotal}/> : '-'}</span>
    },
    {
      name: 'website',
      content: <ExternalLink url={token.url}/>
    }, {
      name: 'token_holders',
      content: <FormattedNumber value={token.total_holders}/>
    }, {
      name: 'issuer',
      content: issuer_address
    }, {
      name: 'nr_of_Transfers',
      content: <FormattedNumber value={token.total_transfers}/>
    }, {
      name: 'white_paper',
      content: token.white_paper !== 'no_message' ?
        <ExternalLink url={token.white_paper && tu(token.white_paper)} _url={token.white_paper}/> :
        <span style={{color: '#d8d8d8'}}>-</span>
    }, {
      name: 'created',
      content: (
        <span>{token.created_time ? moment(token.created_time).format('MM/DD/YYYY HH:mm:ss') : ''}</span>
      )
    }, {
      name: 'GitHub',
      content: token.github !== 'no_message' ?
        <ExternalLink url={token.github && tu(token.github)} _url={token.github}/> :
        <span style={{color: '#d8d8d8'}}>-</span>
    },
    // {
    //   name: 'contract_address',
    //   content:  <span style={{color: '#d8d8d8'}}>-</span>
    // },
    {
      name: 'social_link',
      content: <div className="d-flex">
        {token['social_media'] && token['social_media'].map((media, index) => {
          return (media.url !== '' && <div key={index} style={{marginRight: '10px'}}>
            <a href={media.url}>
              <img src={require('../../../images/' + media.name + '.png')}/>
            </a>
          </div>)
        })}
        {!social_display && <span style={{color: '#d8d8d8'}}>-</span>}
      </div>
    }]

  return (
    <div className="information-bg">{
      tokenList.map((item, index) => {
        return (
          <div key={index} className={index % 2 === 0 ? 'information-bg-item' : 'information-bg-item ml'}>
            <span>{item.name ? tu(item.name) : ''}</span>
            <div style={{width: '75%'}}>{item.content}</div>
          </div>)
      })
    }</div>
  );
}
