import React, { Fragment } from 'react';
import { tu } from '../../../utils/i18n';
import { AddressLink } from '../../common/Links';
import { Client } from '../../../services/api';
// import { ONE_UNIT } from '../../../constants';
import SmartTable from '../../common/SmartTable.js'
import { FormattedNumber, injectIntl } from 'react-intl';
import { TronLoader } from '../../common/loaders';
import { upperFirst, upperCase } from 'lodash';
import { Tooltip } from 'antd';
import { FormatNumberByDecimals } from '../../../utils/number';
import { QuestionMark } from '../../common/QuestionMark';

class TokenHolders extends React.Component {

  constructor (props) {
    super(props);

    this.state = {
      filter: {},
      addresses: [],
      page: 0,
      total: 0,
      pageSize: 25
    };
  }

  componentDidMount () {
    this.loadTokenHolders();
  }

  onChange = (page, pageSize) => {
    this.loadTokenHolders(page, pageSize);
  };

  loadTokenHolders = async (page = 1, pageSize = 20) => {
    let {filter} = this.props;

    this.setState({loading: true});

    const result = await Client.getTokenHolders(filter.asset_id, {
      limit: pageSize,
      start: (page - 1) * pageSize,
      count: true
    });
    if (Array.isArray(result.items)) {
      result.items.forEach((item, index) => {
        item.index = index + 1;
      })
    }

    this.setState({
      page,
      addresses: result.items,
      total: result.total,
      rangeTotal: result.range_total,
      loading: false,
    });
  };

  customizedColumn = () => {
    let {intl, token, tokenPrecision} = this.props;
    return [
      {
        title: '#',
        dataIndex: 'index',
        key: 'index',
        width: '10%',
        align: 'left',
        className: 'ant_table',
      },
      {
        title: intl.formatMessage({id: 'address'}),
        dataIndex: 'address',
        key: 'address',
        render: (text, record) => {
          return record.ico ?
            <Tooltip placement="topLeft" title={upperCase(intl.formatMessage({id: record.ico}))}>
                <span className="d-flex align-items-center">
                  <img src={require('../../../images/' + record.ico + '-logo.png')}
                       style={{width: '14px', marginLeft: '-20px', marginRight: '6px'}}
                  />
                <AddressLink address={record.address}/>
              </span>
            </Tooltip> :
            <AddressLink address={record.address}/>

        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'quantity'})),
        dataIndex: 'hash',
        key: 'hash',
        width: '20%',
        align: 'right',
        className: 'ant_table',
        render: (text, record) => {
          //return <FormattedNumber value={record.balance/ Math.pow(10,tokenPrecision.precision)}/>
          return <span>{FormatNumberByDecimals(record.balance, tokenPrecision.decimals)}</span>
        }
      },
      {
        title: intl.formatMessage({id: 'percentage'}),
        dataIndex: 'percentage',
        key: 'percentage',
        width: '18%',
        align: 'right',
        className: 'ant_table',
        render: (text, record) => {
          return (
            <div>
              {/*<FormattedNumber*/}
              {/*value={(((record.balance) / token.total_supply) * 100)}*/}
              {/*maximumFractionDigits={6}*/}
              {/*/>*/}
              <FormattedNumber value={text} maximumFractionDigits={6}/>&nbsp;%
            </div>
          )
        }
      }
    ];
  }

  render () {
    let {addresses, total, rangeTotal, loading} = this.state;
    let {intl} = this.props
    let column = this.customizedColumn();
    // let tableInfo = intl.formatMessage({id: 'a_totle'}) + ' ' + total + ' ' + intl.formatMessage({id: 'hold_addr'})
    let tableInfoTip = intl.formatMessage({id: 'table_info_holders_tip1'}) + ' ' + rangeTotal + ' ' + intl.formatMessage({id: 'table_info_holders_tip2'});

    if (!loading && addresses.length === 0) {
      return (
        <div className="p-3 text-center no-data">{tu('no_holders_found')}</div>
      );
    }
    return (
      <Fragment>
        {loading && <div className="loading-style" style={{marginTop: '-20px'}}><TronLoader/></div>}
        <div className="row transfers">
          <div className="col-md-12 table_pos">
            {/* {total?<div className="table_pos_info d-none d-md-block">{tableInfo}</div>: ''} */}
            <div style={{paddingLeft: 20}}>
              {total ? <div className="table_pos_info d-none d-md-block" style={{left: 'auto'}}>
                <div>{tu('view_total')} {rangeTotal} {tu('hold_addr')}
                  {rangeTotal >= 10000 ? <QuestionMark placement="top" info={tableInfoTip}/> : ''}
                  <br/>
                  {rangeTotal >= 10000 ? <span>({tu('table_info_big')})</span> : ''}
                </div>
              </div> : ''}
            </div>
            <SmartTable border={false} loading={loading} column={column} data={addresses} total={total}
                        onPageChange={(page, pageSize) => {
                          this.loadTokenHolders(page, pageSize)
                        }}/>
          </div>
        </div>
      </Fragment>
    )
  }

}


export default injectIntl(TokenHolders);
