import React, { Fragment } from 'react';
import { Client } from '../../../services/api';
import { AddressLink, TransactionHashLink } from '../../common/Links';
import { tu } from '../../../utils/i18n';
import TimeAgo from 'react-timeago';
import moment from 'moment';
import { Truncate } from '../../common/text';
import { withTimers } from '../../../utils/timing';
import { injectIntl } from 'react-intl';
import SmartTable from '../../common/SmartTable.js'
import { upperFirst } from 'lodash';
import { TronLoader } from '../../common/loaders';
import { NameWithId } from '../../common/names';
import TotalInfo from '../../common/TableTotal';
// import DateRange from '../../common/DateRange';
import { mapAssetTokenData } from '../../../utils/helper';

class Transfers extends React.Component {

  constructor (props) {
    super(props);

    this.start = moment([2018, 5, 25]).startOf('day').valueOf();
    this.end = moment().valueOf();
    this.state = {
      filter: {},
      transfers: [],
      page: 0,
      total: 0,
      pageSize: 25,
      emptyState: props.emptyState,
      autoRefresh: props.autoRefresh || false
    };
  }

  componentDidMount () {
    this.loadPage();

    if (this.state.autoRefresh !== false) {
      this.props.setInterval(() => this.load(), this.state.autoRefresh);
    }
  }

  onChange = (page, pageSize) => {
    this.loadPage(page, pageSize);
  };

  loadPage = async (page = 1, pageSize = 20) => {
    let {filter} = this.props;
    this.setState(
      {
        loading: true,
        page: page,
        pageSize: pageSize,
      }
    );

    const result = await Client.getAssetTransfers({
      limit: pageSize,
      start: (page - 1) * pageSize,
      asset_id: filter.asset_id
    });
    // let transfers = rebuildList(list, 'assetId', 'amount');
    let transfers = mapAssetTokenData(result)

    for (let index in transfers) {
      transfers[index].index = parseInt(index) + 1;
    }

    this.setState({
      page,
      transfers,
      total: result.total,
      rangeTotal: result.range_total,
      loading: false,
    });
  };

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: upperFirst(intl.formatMessage({id: 'hash'})),
        dataIndex: 'hash',
        key: 'hash',
        className: 'ant_table',
        render: (text) => {
          return (
            <Truncate>
              <TransactionHashLink hash={text}>
                {text}
              </TransactionHashLink>
            </Truncate>
          )
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'age'})),
        dataIndex: 'timestamp',
        key: 'timestamp',
        width: '150px',
        className: 'ant_table',
        render: (text, record) => {
          return <TimeAgo date={record.timestamp} title={moment(record.timestamp).format('MMM-DD-YYYY HH:mm:ss A')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'from'})),
        dataIndex: 'owner_address',
        key: 'owner_address',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        }
      },
      {
        title: '',
        className: 'ant_table',
        width: '30px',
        render: () => {
          return <img src={require('../../../images/arrow.png')}/>
        },
      },
      {
        title: upperFirst(intl.formatMessage({id: 'to'})),
        dataIndex: 'to_address',
        key: 'to_address',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        },
      },
      {
        title: upperFirst(intl.formatMessage({id: 'amount'})),
        dataIndex: 'amount',
        key: 'amount',
        width: '200px',
        align: 'right',
        className: 'ant_table',
        render: (text, record) => {
          return <NameWithId value={record} br/>
        }
      }
    ];
  }

  // onDateOk (start, end) {
  //   this.start = start.valueOf();
  //   this.end = end.valueOf();
  //   let {page, pageSize} = this.state;
  //   this.loadPage(page, pageSize);
  // }

  render () {
    let {transfers, total, rangeTotal, loading} = this.state;
    let column = this.customizedColumn();

    return (
      <Fragment>
        {loading && <div className="loading-style" style={{marginTop: '-20px'}}><TronLoader/></div>}

        <div className="row transfers">
          <div className="col-md-12 table_pos">
            <div className="d-flex justify-content-between pl-3 pr-3" style={{left: 'auto'}}>
              {total ? <TotalInfo total={total} rangeTotal={rangeTotal} typeText="transaction_info"
                                  divClass="table_pos_info_addr"/> : ''}
              {/*<DateRange onDateOk={(start, end) => this.onDateOk(start, end)} dateClass="date-range-box-token"/>*/}
            </div>
            {
              (!loading && transfers.length === 0) ?
                <div className="pt-5 pb-5 text-center no-data transfers-bg-white">{tu('no_transfers')}</div>
                :
                <SmartTable border={false} loading={loading} column={column} data={transfers} total={total}
                            addr="address" transfers="token"
                            onPageChange={(page, pageSize) => {
                              this.loadPage(page, pageSize)
                            }}/>
            }

          </div>
        </div>
      </Fragment>
    )
  }
}

export default withTimers(injectIntl(Transfers));
