import React from 'react';
import { Client } from '../../../services/api';
import { t, tu } from '../../../utils/i18n';
import { injectIntl } from 'react-intl';
import TokenHolders from './TokenHolders';
import { NavLink, Route, Switch } from 'react-router-dom';
import { TronLoader } from '../../common/loaders';
import Transfers from './Transfers.js';
import { Information } from './Information.js';
import { ONE_UNIT, API_URL, CHAIN_SYMBOL } from '../../../constants';
import { login } from '../../../actions/app';
import { reloadWallet } from '../../../actions/wallet';
import { connect } from 'react-redux';
import SweetAlert from 'react-bootstrap-sweetalert';
import { pkToAddress } from 'mcashscan/src/utils/crypto';
import { transactionResultManager } from '../../../utils/mcash';
import xhr from 'axios/index';
import Lockr from 'lockr';
import { withTronWeb } from '../../../utils/mcashWeb';

@withTronWeb
class TokenDetail extends React.Component {

  constructor () {
    super();

    this.state = {
      privateKey: '',
      loading: true,
      token: {},
      tabs: [],
      buyAmount: 0,
      alert: null,
      currentTotalSupply: '',
    };
  }

  componentDidMount () {
    let {match} = this.props;
    if (isNaN(Number(match.params.id))) {
      this.props.history.push('/tokens/list')
    } else {
      this.loadToken(decodeURI(match.params.id));
    }
  }

  componentDidUpdate (prevProps) {
    let {match} = this.props;

    if (match.params.id !== prevProps.match.params.id) {
      if (isNaN(Number(match.params.id))) {
        this.props.history.push('/tokens/list')
      } else {
        this.loadToken(decodeURI(match.params.id));
      }
    }
  }

  // loadTotalTRXSupply = async () => {
  //   const {funds} = await Client.getBttFundsSupply();
  //   this.setState({
  //     currentTotalSupply: parseInt(funds.totalTurnOver),
  //   });
  // }

  loadToken = async (id) => {
    this.setState({loading: true});

    const token = await Client.getTokenById(id);
    if (!token) {
      this.setState({loading: false, token: null});
      this.props.history.push('/tokens/list');
      return;
    }
    this.setState({
      loading: false,
      token,
      tabs: [
        // {
        //   id: 'tokenInfo',
        //   icon: '',
        //   path: '',
        //   label: <span>{tu('issue_info')}</span>,
        //   cmp: () => <TokenInfo token={token}/>
        // },
        {
          id: 'transfers',
          icon: '',
          path: '',
          // path: '/transfers',
          label: <span>{tu('token_transfers')}</span>,
          cmp: () => <Transfers filter={{token: token.name, asset_id: token.asset_id, address: token.owner_address}}/>
        },
        {
          id: 'holders',
          icon: '',
          path: '/holders',
          label: <span>{tu('token_holders')}</span>,
          cmp: () => <TokenHolders filter={{token: token.name, address: token.owner_address, asset_id: token.asset_id}}
                                   token={{total_supply: token.total_supply}}
                                   tokenPrecision={{decimals: token.decimals}}/>
        },
      ]
    });
  };

  submit = async (token) => {
    let price = ((token.trxNum / token.num) * Math.pow(10, token.precision));
    let {account, currentWallet} = this.props;
    let {buyAmount} = this.state;

    let res;
    if (Lockr.get('islogin') || this.props.walletType.type === 'ACCOUNT_LEDGER' || this.props.walletType.type === 'ACCOUNT_MCASHLINK') {
      const tronWebLedger = this.props.mcashWeb();
      const {mcashWeb} = this.props.account;
      try {
        if (this.props.walletType.type === 'ACCOUNT_LEDGER') {
          const unSignTransaction = await tronWebLedger.transactionBuilder.purchaseToken(token.owner_address, token.id + '', parseInt((buyAmount * price).toFixed(0)), this.props.walletType.address).catch(e => false);
          const {result} = await transactionResultManager(unSignTransaction, tronWebLedger);
          res = result;
        }
        if (this.props.walletType.type === 'ACCOUNT_MCASHLINK') {
          const unSignTransaction = await mcashWeb.transactionBuilder.purchaseToken(token.owner_address, token.id + '', parseInt((buyAmount * price).toFixed(0)), mcashWeb.defaultAddress.hex).catch(e => false);
          const {result} = await transactionResultManager(unSignTransaction, mcashWeb);
          res = result;
        }
      } catch (e) {
        console.log(e)
      }
    } else {
      let isSuccess = await Client.participateAsset(
        currentWallet.address,
        token.owner_address,
        token.id + '',
        parseInt((buyAmount * price).toFixed(0)))(account.key);
      res = isSuccess.success
    }
    if (res) {
      this.setState({
        activeToken: null,
        confirmedParticipate: true,
        participateSuccess: res,
        buyAmount: 0,
      });
      this.props.reloadWallet();
      return true;
    } else {
      return false;
    }
  };

  onInputChange = (value) => {
    let {account} = this.props;
    if (value && value.length === 64) {
      this.privateKey.className = 'form-control';
      if (pkToAddress(value) !== account.address)
        this.privateKey.className = 'form-control is-invalid';
    }
    else {
      this.privateKey.className = 'form-control is-invalid';
    }
    this.setState({privateKey: value})
    this.privateKey.value = value;
  }

  confirmPrivateKey = () => {
    let {token} = this.state;
    let {account} = this.props;

    let reConfirm = () => {
      if (this.privateKey.value && this.privateKey.value.length === 64) {
        if (pkToAddress(this.privateKey.value) === account.address)
          this.buyTokens(token);
      }
    }

    this.setState({
      alert: (
        <SweetAlert
          info
          showCancel
          cancelBtnText={tu('cancel')}
          confirmBtnText={tu('confirm')}
          confirmBtnBsStyle="success"
          cancelBtnBsStyle="default"
          title={tu('confirm_private_key')}
          onConfirm={reConfirm}
          onCancel={() => this.setState({alert: null})}
          // style={{marginLeft: '-240px', marginTop: '-195px'}}
        >
          <div className="form-group">
            <div className="input-group mb-3">
              <input type="text"
                     ref={ref => this.privateKey = ref}
                     onChange={(ev) => {
                       this.onInputChange(ev.target.value)
                     }}
                     className="form-control is-invalid"
              />
              <div className="invalid-feedback">
                {tu('fill_a_valid_private_key')}
              </div>
            </div>
          </div>
        </SweetAlert>
      )
    });
  }

  isBuyValid = () => {
    return (this.state.buyAmount > 0);
  };

  onBuyInputChange = (value, price, max) => {
    let {intl} = this.props;
    if (value > max) {
      value = max;
    }
    value = value.replace(/^0|[^\d*]/g, '')
    this.setState({buyAmount: value});
    this.buyAmount.value = value;
    let priceTRX = value * (price);
    this.priceTRX.innerHTML = intl.formatNumber(priceTRX, {
      maximumFractionDigits: 6,
    });
  }

  preBuyTokens = (token) => {
    let {wallet} = this.props;

    if (!wallet.isOpen) {
      this.setState({
        alert: (
          <SweetAlert
            info
            showConfirm={false}
            // style={{ width: '30rem', height: '18.75rem',left:'50%',marginLeft:'-15rem'}}
          >
            <div className="token-sweet-alert">
              <a className="close" onClick={() => {
                this.setState({alert: null})
              }}><i className="fa fa-times" ariaHidden="true"/></a>
              <span>{tu('login_first')}</span>
              <button className="btn btn-danger btn-block mt-3" onClick={() => {
                this.setState({alert: null})
              }}>{tu('OK')}</button>
            </div>

          </SweetAlert>
        ),
      });
    } else {
      this.setState({
        alert: (
          <SweetAlert
            showConfirm={false}
            // style={{marginLeft: '-240px', marginTop: '-195px', width: '450px', height: '300px'}}
          >
            <div className="mt-5 token-sweet-alert" style={{textAlign: 'left'}}>
              <a style={{float: 'right', marginTop: '-45px'}} onClick={() => {
                this.setState({alert: null})
              }}><i className="fa fa-times" ariaHidden="true"/></a>
              <h5 style={{color: 'black'}}>{tu('buy_token_info')}</h5>
              {token.remaining === 0 && <span> {tu('no_token_to_buy')}</span>}
              <div className="input-group mt-5">
                <input
                  type="number"
                  ref={ref => this.buyAmount = ref}
                  className="form-control"
                  max={token.remaining}
                  min={1}
                  onKeyUp={(e) => {
                    e.target.value = e.target.value.replace(/^0|[^\d*]/g, '')
                  }}
                  onChange={(e) => {
                    this.onBuyInputChange(e.target.value, ((token.trx_num / token.num) * Math.pow(10, token.precision)) / ONE_UNIT, token.remaining)
                  }}
                />
              </div>
              <div className="text-center mt-3 text-muted">
                <b>= <span ref={ref => this.priceTRX = ref}>0</span> {CHAIN_SYMBOL}</b>
              </div>
              <button className="btn btn-danger btn-block mt-3" onClick={() => {
                this.buyTokens(token)
              }}>{tu('participate')}</button>
            </div>
          </SweetAlert>
        ),
      });
    }
  }

  buyTokens = (token) => {
    let price = ((token.trx_num / token.num) * Math.pow(10, token.precision));
    let {buyAmount} = this.state;
    if (buyAmount <= 0) {
      return;
    }
    let {currentWallet} = this.props;
    let tokenCosts = buyAmount * (price / ONE_UNIT);

    if ((currentWallet.balance / ONE_UNIT) < tokenCosts) {
      this.setState({
        alert: (
          <SweetAlert
            warning
            showConfirm={false}
            // style={{marginLeft: '-240px', marginTop: '-195px', width: '450px', height: '300px'}}
          >
            <div className="mt-5 token-sweet-alert">
              <a style={{float: 'right', marginTop: '-155px'}} onClick={() => {
                this.setState({alert: null})
              }}><i className="fa fa-times" ariaHidden="true"/></a>
              <span>
                  {tu('not_enough_trx_message')}
                </span>
              <button className="btn btn-danger btn-block mt-3" onClick={() => {
                this.setState({alert: null})
              }}>{tu('confirm')}</button>
            </div>
          </SweetAlert>
        ),
      });
    } else {
      this.setState({
        alert: (
          <SweetAlert
            warning
            showConfirm={false}
            // style={{marginLeft: '-240px', marginTop: '-195px', width: '450px', height: '300px'}}
          >
            <div className="mt-5 token-sweet-alert">
              <a style={{float: 'right', marginTop: '-155px'}} onClick={() => {
                this.setState({alert: null})
              }}><i className="fa fa-times" ariaHidden="true"/></a>
              <p className="ml-auto buy_confirm_message">{tu('buy_confirm_message_1')}</p>
              <span>
                {buyAmount} {token.name} {t('for')} {parseFloat((buyAmount * (price / ONE_UNIT)).toFixed(6))} {CHAIN_SYMBOL}?
                </span>
              <button className="btn btn-danger btn-block mt-3" onClick={() => {
                this.confirmTransaction(token)
              }}>{tu('confirm')}</button>
            </div>
          </SweetAlert>
        ),
      });
    }
  };

  confirmTransaction = async (token) => {
    let {intl} = this.props;

    this.setState({
      alert: (
        <SweetAlert
          showConfirm={false}
          showCancel={false}
          cancelBtnBsStyle="default"
          title={intl.formatMessage({id: 'transferring'})}
          // style={{marginLeft: '-240px', marginTop: '-195px', width: '450px', height: '300px'}}
        />
      ),
    });

    if (await this.submit(token)) {
      this.setState({
        alert: (
          <SweetAlert
            success
            showConfirm={false}
            // style={{marginLeft: '-240px', marginTop: '-195px', width: '450px', height: '300px'}}
          >
            <div className="mt-5 token-sweet-alert">
              <a style={{float: 'right', marginTop: '-155px'}} onClick={() => {
                this.setState({alert: null})
              }}><i className="fa fa-times" ariaHidden="true"/></a>
              <h5 style={{color: 'black'}}>{tu('transaction')} {tu('confirm')}</h5>
              <span>
                {tu('success_receive')} {token.name} {tu('tokens')}
              </span>
              <button className="btn btn-danger btn-block mt-3" onClick={() => {
                this.setState({alert: null})
              }}>{tu('OK')}</button>
            </div>
          </SweetAlert>
        )
      });
    } else {
      this.setState({
        alert: (
          <SweetAlert danger title="Error" onConfirm={() => this.setState({alert: null})}>
            {tu('fail_transaction')}
          </SweetAlert>
        )
      });
    }
  };

  render () {
    let {match} = this.props;
    let {token, tabs, loading, alert, currentTotalSupply} = this.state;

    return (
      <main className="container header-overlap token_black mc-donalds-coin">
        {alert}
        {
          loading ? (
              <div className="card">
                <TronLoader>
                  {tu('loading_token')} {token.name}
                </TronLoader>
              </div>
            ) : (
            <div className="row">
              {token &&
              <div className="col-sm-12">
                <div className="card">
                  <div className="card-body">
                    <div className="d-flex">
                      {token && token.logo_url && token.asset_id ?
                        <div>
                          <img className='token-logo' src={token.logo_url} alt=''/>
                        </div>
                        : <img className='token-logo' src={require('../../../images/logo_default.png')} alt=''/>
                      }
                      <div style={{width: '70%'}} className="token-description">
                        <h5 className="card-title">
                          {token.name} ({token.symbol})
                        </h5>
                        <p className="card-text">{token.description}</p>
                      </div>
                      {/*<div className="ml-auto">*/}
                      {/*{(!(token.end_time < new Date() || token.issued_percentage === 100 || token.start_time > new Date() || token.is_black) && (token.can_show !== 3)) &&*/}
                      {/*<button className="btn btn-default btn-xs d-inline-block"*/}
                      {/*onClick={() => this.preBuyTokens(token)}>{tu('participate')}</button>*/}
                      {/*}*/}
                      {/*</div>*/}
                    </div>
                  </div>
                  {token && <Information token={token} currentTotalSupply={currentTotalSupply}/>}
                </div>

                <div className="card mt-3 border_table">
                  <div className="card-header">
                    <ul className="nav nav-tabs card-header-tabs" style={{marginTop: '-12px', marginLeft: '-20px'}}>
                      {
                        tabs.map(tab => (
                          <li key={tab.id} className="nav-item">
                            <NavLink exact to={match.url + tab.path} className="nav-link text-dark">
                              <i className={tab.icon + ' mr-2'}/>
                              {tab.label}
                            </NavLink>
                          </li>
                        ))
                      }
                    </ul>
                  </div>
                  <div className="card-body p-0">
                    <Switch>
                      {
                        tabs.map(tab => (
                          <Route key={tab.id} exact path={match.url + tab.path} render={() => (<tab.cmp/>)}/>
                        ))
                      }
                    </Switch>
                  </div>
                </div>
              </div>
              }
            </div>
          )

        }

      </main>
    )
  }
}


function mapStateToProps (state) {
  return {
    tokens: state.tokens.tokens,
    wallet: state.wallet,
    currentWallet: state.wallet.current,
    account: state.app.account,
    walletType: state.app.wallet,
  };
}

const mapDispatchToProps = {
  login,
  reloadWallet
};

export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(TokenDetail));
