import React, {Fragment} from 'react';
import {ONE_UNIT} from '../../../constants';
import {AddressLink, ExternalLink} from '../../common/Links';
import Field from './Field';
import {TRXPrice} from '../../common/Price';
import {tu} from '../../../utils/i18n';
import {FormattedNumber, FormattedDate} from 'react-intl';
import {toUtf8} from 'mcashweb'
import {NameWithId} from '../../common/names';
import {FormatNumberByDecimals} from '../../../utils/number';
import {isHex} from '../../../utils/helper';
import TriggerSmartContract from './TriggerSmartContract';
import StakeContract from './StakeContract';
import WithdrawBalanceContract from './WithdrawBalanceContract';
import CostViewer from "./CostViewer";

export default function Contract({contract}) {
  let parametersArr = [
    'MAINTENANCE_TIME_INTERVAL',
    'ACCOUNT_UPGRADE_COST',
    'CREATE_ACCOUNT_FEE',
    'TRANSACTION_FEE',
    'ASSET_ISSUE_FEE',
    'WITNESS_PAY_PER_BLOCK',
    'WITNESS_STANDBY_ALLOWANCE',
    'CREATE_NEW_ACCOUNT_FEE_IN_SYSTEM_CONTRACT',
    'CREATE_NEW_ACCOUNT_BANDWIDTH_RATE',
    'ALLOW_CREATION_OF_CONTRACTS',
    'REMOVE_THE_POWER_OF_THE_GR',
    'ENERGY_FEE',
    'EXCHANGE_CREATE_FEE',
    'MAX_CPU_TIME_OF_ONE_TX',
    'ALLOW_UPDATE_ACCOUNT_NAME',
    'ALLOW_SAME_TOKEN_NAME'
  ];
  let proposal, proposalVal;
  if (contract.parameters) {
    for (let item in contract.parameters) {
      for (let i in parametersArr) {
        if (item === i) {
          proposal = parametersArr[item];
          proposalVal = contract.parameters[item]
        }
      }
    }
  }

  let TokenIDList = [];
  TokenIDList.push(contract)
  if (contract.contractType) {
    switch (contract.contractType.toUpperCase()) {
      case 'TRANSFERCONTRACT':

        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('transfer_contract')}
                <small>{tu('TRX_transfer_between_addresses')}</small>
              </h5>
            </div>
            <div className="table-responsive">
              <table className="table">
                <tbody>
                <Field label="from"><AddressLink address={contract['owner_address']}/></Field>
                <Field label="to"><AddressLink address={contract['to_address']}/></Field>
                <Field label="amount"><TRXPrice amount={contract.amount / ONE_UNIT}/></Field>
                {
                  contract.contract_note ?
                    <Field label="note">{decodeURIComponent(contract.contract_note)}</Field>
                    : null
                }
                </tbody>
              </table>
            </div>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );

      case 'TRANSFERASSETCONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('transfer_asset_contract')}
                <small>{tu('token_transfer_between_addresses')}</small>
              </h5>
            </div>
            <div className="table-responsive">
              <table className="table">
                <tbody>
                <Field label="from"><AddressLink address={contract['owner_address']}/></Field>
                <Field label="to"><AddressLink address={contract['to_address']}/></Field>
                <Field label="amount">
                  {contract.token ? FormatNumberByDecimals(contract.amount, contract.token.decimals) : contract.amount}
                </Field>
                <Field label="token"><NameWithId value={contract} notamount totoken/></Field>
                </tbody>
              </table>
            </div>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );

      case 'PARTICIPATEASSETISSUECONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('participate_asset_issue_contract')}
                <small>{tu('participate_token_between_addresses')}</small>
              </h5>
            </div>

            <div className="table-responsive">
              <table className="table">
                <tbody>
                <Field label="to"><AddressLink address={contract['owner_address']}/></Field>
                <Field label="issuer"><AddressLink address={contract['to_address']}/></Field>
                <Field label="amount">{contract.amount / ONE_UNIT}</Field>
                <Field label="token"><NameWithId value={contract} notamount totoken/></Field>
                </tbody>
              </table>
            </div>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );
      case 'WITNESSUPDATECONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('witness_update_contract')}
                <small>{tu('updates_a_witness')}</small>
              </h5>
            </div>
            <table className="table">
              <tbody>
              <Field label="owner_address"><AddressLink address={contract['owner_address']}/></Field>
              <Field label="URL">{contract.url}</Field>
              </tbody>
            </table>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );
      case 'WITNESSCREATECONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('witness_create_contract')}
                <small>{tu('create_a_witness')}</small>
              </h5>
            </div>
            <table className="table">
              <tbody>
              <Field label="owner_address"><AddressLink address={contract['owner_address']}/></Field>
              <Field label="URL">{contract.url}</Field>
              </tbody>
            </table>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );

      case 'ACCOUNTUPDATECONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('account_update_contract')}
                <small>{tu('update_account_name')}</small>
              </h5>
            </div>
            <table className="table">
              <tbody>
              <Field label="owner_address"><AddressLink address={contract['owner_address']}/></Field>
              <Field label="account_name">{toUtf8(contract['account_name'])}</Field>
              </tbody>
            </table>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );
      case 'ACCOUNTCREATECONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('account_create_contract')}
                <small>{tu('account_create')}</small>
              </h5>
            </div>
            <table className="table">
              <tbody>
              <Field label="create_from_address"><AddressLink address={contract['owner_address']}/></Field>
              <Field label="create_to_address"><AddressLink address={contract['account_address']}/></Field>
              </tbody>
            </table>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );
      case 'WITHDRAWBALANCECONTRACT':
        return <WithdrawBalanceContract data={contract}/>;

      case 'FREEZEBALANCECONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('freeze_balance_contract')}
                <small>{tu('freeze_TRX')}</small>
              </h5>
            </div>
            <table className="table">
              <tbody>
              <Field label="owner_address"><AddressLink address={contract['owner_address']}/></Field>
              {contract['receiver_address'] &&
              <Field label="receive_list"><AddressLink address={contract['receiver_address']}/></Field>}
              {contract['resource'] ? <Field label="type">{contract['resource']}</Field> :
                <Field label="type">Bandwidth</Field>}
              <Field label="frozen_balance">{contract['frozen_balance'] / ONE_UNIT}</Field>
              <Field label="frozen_days">{contract['frozen_duration']}</Field>
              </tbody>
            </table>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );

      case 'UNFREEZEBALANCECONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('unfreeze_balance_contract')}
                <small>{tu('unfreeze_TRX')}</small>
              </h5>
            </div>
            <table className="table">
              <tbody>
              <Field label="owner_address"><AddressLink address={contract['owner_address']}/></Field>
              {contract['receiver_address'] &&
              <Field label="receive_list"><AddressLink address={contract['receiver_address']}/></Field>}
              </tbody>
            </table>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );

      case 'VOTEWITNESSCONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('vote_witness_contract')}
                <small>{tu('vote_for_a_witness')}</small>
              </h5>
            </div>
            <table className="table">
              <tbody>
              <Field label="owner_address"><AddressLink address={contract['owner_address']}/></Field>
              <tr>
                <th>{tu('votes')}</th>
                <td>
                  <ul>
                    <li>
                      <AddressLink address={contract['vote_address']} truncate={false}/>
                      {tu('counts')} : {contract['resultInfo'] ? contract['resultInfo']['voteCount'] : ''}
                    </li>
                  </ul>
                </td>
              </tr>
              </tbody>
            </table>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );

      case 'ASSETISSUECONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('asset_issue_contract')}
                <small>{tu('issue_a_new_asset')}</small>
              </h5>
            </div>
            <table className="table">
              <tbody>
              <Field label="owner_address"><AddressLink address={contract['owner_address']}/></Field>
              <Field label="token_name">{isHex(contract.name) ? toUtf8(contract.name) : ''}</Field>
              <Field label="total_supply"><FormattedNumber
                value={contract.total_supply / (contract.precision ? Math.pow(10, contract.precision) : 1)}/></Field>
              <Field label="TRX_exchange_rate">{contract.trx_num / ONE_UNIT} : {contract.num}</Field>
              <Field label="start_time">
                {contract.end_time - contract.start_time > 1000 ? <FormattedDate value={contract.start_time}/> : '-'}
              </Field>
              <Field label="end_time">
                {contract.end_time - contract.start_time > 1000 ? <FormattedDate value={contract.end_time}/> : '-'}
              </Field>
              <Field label="description">{isHex(contract.description) ? toUtf8(contract.description) : ''}</Field>
              <Field label="URL">{isHex(contract.url) ? <ExternalLink url={toUtf8(contract.url)}/> : ''}</Field>
              </tbody>
            </table>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );
      case 'PROPOSALCREATECONTRACT':
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                {tu('proposal_create_contract')}
                <small>{tu('proposal_create')}</small>
              </h5>
            </div>
            <table className="table">
              <tbody>
              <Field label="owner_address"><AddressLink address={contract['owner_address']}/></Field>
              <tr>
                <th>{proposal}</th>
                <td>{proposalVal}</td>
              </tr>
              </tbody>
            </table>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );
      case 'TRIGGERSMARTCONTRACT':
        return <TriggerSmartContract data={contract}/>;
      case 'STAKECONTRACT':
        return <StakeContract data={contract}/>
      default:
        return (
          <Fragment>
            <div className="card-body table-title">
              <h5>
                <i className="fa fa-exchange-alt"/>
                &nbsp;&nbsp;
                {contract.contractType}
              </h5>
            </div>
            <table className="table">
              <tbody>
              {
                contract['owner_address'] ?
                  <Field label="owner_address"><AddressLink address={contract['owner_address']}/></Field>
                  : ''
              }
              </tbody>
            </table>
            <CostViewer cost={contract.cost}/>
          </Fragment>
        );
    }
  }

  return (
    <div>
      {
        contract.contractType && <div className="card-body">
          {JSON.stringify(contract)}
        </div>
      }
    </div>
  );
}
