import React from 'react';
import {FormatNumberByDecimals} from '../../../utils/number';
import {CONTRACT_ADDRESS_USDT, ONE_UNIT} from '../../../constants';
import {tu} from '../../../utils/i18n';
import {gets} from '../../../utils/helper';
import {AddressLink, ContractLink, TokenTRC20Link} from '../../common/Links';
import {TRXPrice} from '../../common/Price';
import CostViewer from "./CostViewer";

class TriggerSmartContract extends React.Component {

  render() {
    const {data} = this.props;
    if (!data) {
      return <div/>
    }

    const tokenTransfer = data && Array.isArray(data.tokenTransferInfo) && data.tokenTransferInfo.length > 0 ? data.tokenTransferInfo[0] : {}
    return (
      <React.Fragment>
        <div className="card-body table-title">
          <h5>
            <i className="fa fa-exchange-alt"/>
            {tu('trigger_smart_contract')}
            <small>{tu('normal_address_trigger_smart_contract')}</small>
          </h5>
        </div>
        <div className="content">
          <div className="content_pos">
            <div className="d-flex border-bottom pt-2">
              <div className="content_box_name">{tu('Basic_info')}</div>
              <div className="flex1">
                <div className="d-flex border-bottom content_item">
                  <div className="content_name">{tu('contract_triggers_owner_address')}:</div>
                  <div className="flex1"><AddressLink address={data.owner_address}/></div>
                </div>
                <div className="d-flex border-bottom content_item">
                  <div className="content_name">{tu('contract_address')}:</div>
                  <div className="flex1">
                    <AddressLink address={data.contract_address} isContract={true}/>
                  </div>
                </div>
                <div className="d-flex content_item">
                  <div className="content_name">{tu('value')}:</div>
                  {
                    data['call_value'] ?
                      <TRXPrice amount={data['call_value'] / ONE_UNIT}/>
                      : <TRXPrice amount={0}/>
                  }
                </div>
              </div>
            </div>

            {
              Array.isArray(data.tokenTransferInfo) && data.tokenTransferInfo.length > 0 && (
                data.tokenTransferInfo.length > 1 ? (
                  <div className="d-flex border-bottom pt-2">
                    <div className="content_box_name"><span>{'M20 transfers'}</span></div>
                    <div className="flex1">
                      {
                        data.tokenTransferInfo.map((item, index) => {
                          const token = item.token ? item.token : {}
                          return (
                            <div key={index} className={'d-flex align-items-center content_item'}>
                              <div className="d-flex">
                                <div className="mr-1">
                                  <TokenTRC20Link
                                    name={gets(tokenTransfer, 'token.name')}
                                    address={gets(tokenTransfer, 'token.contract_address')}
                                    namePlus={gets(tokenTransfer, 'token.name') + ' (' + gets(tokenTransfer, 'token.symbol') + ')'}
                                  />{' - '}
                                </div>
                                <div className="mr-1">{tu('transfers')}</div>
                                <div className="mr-1">
                                  {FormatNumberByDecimals(item.amount, token.decimals) + ' ' + token.symbol}
                                </div>
                                <div className="mr-1">{tu('from')}</div>
                                <div className="mr-1" style={{width: '150px'}}>
                                  <ContractLink address={item['owner_address']}/>
                                </div>
                                <div className="mr-1">{tu('to')}</div>
                                <div className="mr-1" style={{width: '150px'}}>
                                  <ContractLink address={item['to_address']}/>
                                </div>
                              </div>
                            </div>
                          )
                        })
                      }
                    </div>
                  </div>
                ) : (
                  <div className="d-flex border-bottom pt-2">
                    <div className="content_box_name"><span>{'M20 transfers'}</span></div>
                    <div className="flex1">
                      <div className="d-flex border-bottom content_item">
                        <div className="content_name">{tu('from')}:</div>
                        <div className="flex1"><AddressLink address={tokenTransfer['owner_address']}/></div>
                      </div>
                      <div className="d-flex border-bottom content_item">
                        <div className="content_name">{tu('to')}:</div>
                        <div className="flex1"><AddressLink address={tokenTransfer['to_address']}/></div>
                      </div>
                      <div className="d-flex border-bottom content_item">
                        <div className="content_name">{tu('amount')}:</div>
                        <div className="flex1">
                          {FormatNumberByDecimals(tokenTransfer.amount, gets(tokenTransfer, 'token.decimals'))}
                        </div>
                      </div>
                      <div className="d-flex border-bottom content_item">
                        <div className="content_name">{tu('token_txs_info')}:</div>
                        <div className="flex1">
                          {
                            tokenTransfer['contract_address'] === CONTRACT_ADDRESS_USDT ? (
                              <b className="token-img-top" style={{marginRight: 5}}>
                                <img width={20} height={20} src={gets(tokenTransfer, 'token.logo_url')}/>
                                <i style={{width: 10, height: 10, bottom: -5}}/>
                              </b>
                            ) : (
                              <img
                                src={gets(tokenTransfer, 'token.logoUrl')}
                                width={20} height={20} style={{marginRight: 5}}
                              />
                            )
                          }
                          <TokenTRC20Link
                            name={gets(tokenTransfer, 'token.name')}
                            address={gets(tokenTransfer, 'token.contract_address')}
                            namePlus={gets(tokenTransfer, 'token.name') + ' (' + gets(tokenTransfer, 'token.symbol') + ')'}
                          />
                        </div>
                      </div>
                    </div>
                  </div>
                )
              )
            }

            {Array.isArray(data.internalTransactions) && data.internalTransactions.length > 0 && (
              <div className="d-flex border-bottom pt-2">
                <div className="content_box_name">{tu('Internal_txns')}</div>
                <div className="flex1">
                  {
                    data.internalTransactions.map((item, index) => {
                      const token = item.token ? item.token : {}
                      return (
                        <div key={index} className={'d-flex align-items-center content_item'}>
                          <div className="d-flex">
                            <div className="mr-1">{tu('transfers')}</div>
                            {/*<div className="mr-1">{item.amount + ' ' + tokenInfo.map_token_name_abbr}</div>*/}
                            <div className="mr-1">
                              {FormatNumberByDecimals(item.amount, token.decimals) + ' ' + token.symbol}
                            </div>
                            <div className="mr-1">{tu('from')}</div>
                            <div className="mr-1" style={{width: '150px'}}>
                              <ContractLink address={item['from_address']}/>
                            </div>
                            <div className="mr-1">{tu('to')}</div>
                            <div className="mr-1" style={{width: '150px'}}>
                              <ContractLink address={item['to_address']}/>
                            </div>
                          </div>
                        </div>
                      )
                    })
                  }
                </div>
              </div>
            )}
            <CostViewer cost={data.cost}/>
            {
              data.method && (
                <div className="d-flex border-bottom pt-2">
                  <div className="content_box_name">{tu('Method_calling')}</div>
                  <div className="flex1">
                    <div className="d-flex border-bottom content_item">
                      <div className="content_name">{tu('contract_method')}:</div>
                      <div className="flex1">{data.method}</div>
                    </div>
                    {
                      Array.isArray(data.parameters) && data.parameters.map((item, index) => {
                        return (
                          <div key={index} className="d-flex border-bottom content_item">
                            <div className="content_name">{item.name}:</div>
                            <div className="flex1">{item.value}</div>
                          </div>
                        )
                      })
                    }
                  </div>
                </div>
              )
            }
          </div>
        </div>
      </React.Fragment>
    )
  }

}

export default TriggerSmartContract
