/* eslint-disable no-restricted-globals */
import { connect } from 'react-redux';
import React, { Fragment } from 'react';
import { FormattedNumber, injectIntl } from 'react-intl';
import { tu } from '../../../utils/i18n';
import { Client } from '../../../services/api';
import { isAddressValid } from 'mcashscan/src/utils/crypto';
import _, { find, round } from 'lodash';
import { CHAIN_DECIMAL, ONE_UNIT, TOKENS_CONFIG, CHAIN_TOKEN } from '../../../constants';
import { Alert } from 'reactstrap';
import { reloadWallet } from '../../../actions/wallet';
import SweetAlert from 'react-bootstrap-sweetalert';
import { TronLoader } from '../../common/loaders';
import { login } from '../../../actions/app';
import Lockr from 'lockr';
import { Select } from 'antd';
import isMobile from '../../../utils/isMobile';
import { withTronWeb } from '../../../utils/mcashWeb';
import { transactionResultManager } from '../../../utils/mcash'
import BigNumber from 'bignumber.js'

BigNumber.config({EXPONENTIAL_AT: [-20, 30]});

const {Option, OptGroup} = Select;

@withTronWeb
class SendForm extends React.Component {

  constructor (props) {
    super(props);

    this.state = {
      privateKey: '',
      to: props.to || '',
      errors: {
        to: false,
        amount: false
      },
      token: '',
      amount: '',
      note: '',
      sendStatus: 'waiting',
      isLoading: false,
      toAccount: null,
      modal: null,
      tokens20: [],
      decimals: '',
      balance: '',
    };
  }

  /**
   * Check if the form is valid
   * @returns {*|boolean}
   */
  isValid = () => {
    let {to, token, amount, balance} = this.state;
    let {account} = this.props;
    /*
       if (!privateKey || privateKey.length !== 64) {
         return false;
       }

      if(privateKey && privateKey.length === 64 && pkToAddress(privateKey) !== account.address){
         return false;
       }
    */
    return isAddressValid(to) && token !== '' && balance >= amount && amount > 0 && to !== account.address;
  };

  /**
   * Send the transaction
   */
  send = async () => {
    let {token} = this.state;
    const arr = token.split('-');
    const TokenType = token === TOKENS_CONFIG.MCASH.type ? TOKENS_CONFIG.M1.type : arr[arr.length - 1];
    switch (TokenType) {
      case TOKENS_CONFIG.M1.type:
        if (Lockr.get('islogin') || this.props.wallet.type === 'ACCOUNT_LEDGER' || this.props.wallet.type === 'ACCOUNT_MCASHLINK') {
          await this.tokenSendWithTronLink();
        } else {
          await this.token10Send();
        }
        break;
      case TOKENS_CONFIG.M20.type:
        await this.token20Send();
        break;
      default:
        break;
    }
  };

  tokenSendWithTronLink = async () => {
    const { mcashWeb } = this.props.account;
    let {to, token, amount, decimals} = this.state;
    let list = token.split('-');
    let tokenId = token === TOKENS_CONFIG.MCASH.type ? 0 : list[1];
    let {onSend, wallet} = this.props;

    let result, success;
    this.setState({isLoading: true, modal: null});

    if (token === TOKENS_CONFIG.MCASH.type) {
      amount = this.Mul(amount, ONE_UNIT);
      if (this.props.wallet.type === 'ACCOUNT_LEDGER') {
        result = await this.props.mcashWeb().mcash.sendTransaction(to, amount, {address: wallet.address}, false).catch(function (e) {
          console.error(e)
        });
      } else if (this.props.wallet.type === 'ACCOUNT_MCASHLINK') {
        result = await mcashWeb.mcash.sendTransaction(to, amount, {address: wallet.address}, false).catch(function (e) {
          console.error(e)
        });
      }
      success = result ? result.result : false
    } else {
      //amount = amount * Math.pow(10, decimals);
      amount = this.Mul(amount, Math.pow(10, decimals));
      if (this.props.wallet.type === 'ACCOUNT_LEDGER') {
        result = await this.props.mcashWeb().mcash.sendToken(to, amount, +tokenId, {address: wallet.address}).catch(function (e) {
          console.error(e)
        });
      } else if (this.props.wallet.type === 'ACCOUNT_MCASHLINK') {
        result = await mcashWeb.mcash.sendToken(to, amount, +tokenId, {address: wallet.address}).catch(function (e) {
          console.error(e)
        });
      }
      success = result ? result.result : false
    }

    if (success) {
      this.refreshTokenBalances();
      //two work flows!

      this.setState({
        sendStatus: 'success',
        isLoading: false,
      }, () => {
        onSend && onSend();
      });
    } else {
      this.setState({
        sendStatus: 'failure',
        isLoading: false,
      });

      setTimeout(() => {
        this.setState({
          sendStatus: 'waiting',
        });
      }, 2000);
    }
  };

  Mul (arg1, arg2) {
    let r1 = arg1.toString(), r2 = arg2.toString(), m, resultVal, d = arguments[2];
    m = (r1.split('.')[1] ? r1.split('.')[1].length : 0) + (r2.split('.')[1] ? r2.split('.')[1].length : 0);
    resultVal = Number(r1.replace('.', '')) * Number(r2.replace('.', '')) / Math.pow(10, m);
    return typeof d !== 'number' ? Number(resultVal) : Number(resultVal.toFixed(parseInt(d)));
  }

  token10Send = async () => {
    let {to, token, amount, note, decimals} = this.state;
    let list = token.split('-');
    let TokenName = list[1];
    let {account, onSend} = this.props;

    this.setState({isLoading: true, modal: null});

    if (TokenName === '_') {
      amount = this.Mul(amount, ONE_UNIT);
    } else {

      amount = this.Mul(amount, Math.pow(10, decimals))
    }

    let {success} = await Client.sendWithNote(TokenName, account.address, to, amount, note)(account.key);

    if (success) {
      this.refreshTokenBalances();

      onSend && onSend();
      //two work flows!

      this.setState({
        sendStatus: 'success',
        isLoading: false,
      });
    } else {
      this.setState({
        sendStatus: 'failure',
        isLoading: false,
      });

      setTimeout(() => {
        this.setState({
          sendStatus: 'waiting',
        });
      }, 2000);
    }
  };

  token20Send = async () => {
    const {to, token, amount, decimals} = this.state;
    const TokenName = token.split('-')[0];
    let {onSend, tokens20} = this.props;
    let mcashWeb;
    let transactionId;
    this.setState({isLoading: true, modal: null});
    const tokenFinder = tokens20.find(t => t.token && t.token.name === TokenName);
    const contractAddress = tokenFinder ? tokenFinder.contract_address : '';

    if (this.props.wallet.type === 'ACCOUNT_LEDGER') {
      mcashWeb = this.props.mcashWeb();
      // Send M20
      let unSignTransaction = await mcashWeb.transactionBuilder.triggerSmartContract(
        mcashWeb.address.toHex(contractAddress),
        'transfer(address,uint256)',
        10000000, 0,
        [
          {type: 'address', value: mcashWeb.address.toHex(to)},
          {type: 'uint256', value: new BigNumber(amount).shiftedBy(decimals).toString()}
        ],
        mcashWeb.address.toHex(this.props.wallet.address),
      );
      if (unSignTransaction.transaction !== undefined)
        unSignTransaction = unSignTransaction.transaction;
      unSignTransaction.extra = {
        to,
        decimals: decimals,
        token_name: TokenName,
        amount: amount,
      };
      transactionId = await transactionResultManager(unSignTransaction, mcashWeb);
    } else if (this.props.wallet.type === 'ACCOUNT_MCASHLINK' || this.props.wallet.type === 'ACCOUNT_PRIVATE_KEY') {
      mcashWeb = this.props.account.mcashWeb;
      let contractInstance = await mcashWeb.contract().at(contractAddress);
      transactionId = await contractInstance.transfer(to, new BigNumber(amount).shiftedBy(decimals).toString()).send()
        .catch((e) => {
          console.error(e);
        });
    }
    if (transactionId) {
      this.refreshTokenBalances();
      onSend && onSend();
      //two work flows!

      this.setState({
        sendStatus: 'success',
        isLoading: false,
      });
    } else {
      this.setState({
        sendStatus: 'failure',
        isLoading: false,
      });

      setTimeout(() => {
        this.setState({
          sendStatus: 'waiting',
        });
      }, 2000);
    }
  };

  confirmSend = () => {
    let {to, token, amount} = this.state;
    let list = token.split('-');
    let TokenName = list[0];
    let TokenID;
    const style = isMobile ? {} : {marginLeft: '-240px', marginTop: '-195px'};
    if (list[1] !== '_' && list[1] !== TOKENS_CONFIG.M20.type) {
      TokenID = list[1];
    }
    this.setState({
      modal: (
        <SweetAlert
          info
          showCancel
          cancelBtnText={tu('cancel')}
          confirmBtnText={tu('confirm')}
          cancelBtnBsStyle="default"
          title={tu('confirm_transaction')}
          onConfirm={this.send}
          onCancel={this.hideModal}
          style={style}
        >
          {tu('transfer_confirm_info')}<br/>
          <span className="font-weight-bold">{' '}
            <FormattedNumber
              maximumFractionDigits={7}
              minimunFractionDigits={7}
              value={amount}/>{' '}
            {TokenName}
            {TokenID && '[' + TokenID + ']'}
          </span><br/>
          {tu('to')}<br/>
          <div className="text-truncate">{to}</div>
        </SweetAlert>
      )
    });
  };

  hideModal = () => {
    this.setState({
      modal: null,
    });
  };

  setAmount = (amount) => {
    let {token, decimals} = this.state;
    if (token && amount) {
      const list = token.split('-');
      const TokenType = list[2];
      if (token === TOKENS_CONFIG.MCASH.type) {
        amount = parseFloat(amount);
        amount = round(amount, CHAIN_DECIMAL);
        if (amount <= 0) {
          amount = 0;
        }
      } else if (TokenType === TOKENS_CONFIG.M1.type) {
        amount = parseFloat(amount);
        amount = round(amount, decimals);
        if (amount <= 0) {
          amount = 0;
        }
      } else if (TokenType === TOKENS_CONFIG.M20.type) {
        amount = parseFloat(amount);
        amount = round(amount, decimals);
        if (amount <= 0) {
          amount = 0;
        }
      }
    }

    this.setState(prevState => ({
      amount,
      errors: { ...prevState.errors, amount: true }
    }))
  }

  getSelectedTokenBalance = () => {
    const {tokenBalances, tokens20, currentWallet} = this.props;
    const {token} = this.state;
    const list = token.split('-');
    const tokenType = list[list.length - 1];
    if (!token) {
      return
    }
    if (tokenType === TOKENS_CONFIG.MCASH.type) {
      this.setState({
        decimals: CHAIN_DECIMAL,
        balance: new BigNumber(currentWallet.balance).shiftedBy(-CHAIN_DECIMAL).toFixed()
      });
      return
    }
    if (tokenType === TOKENS_CONFIG.M1.type) {
      const tokenId = list[1];
      const tbsFinder = find(tokenBalances, t => `${t.token.asset_id}` === `${tokenId}`);
      let balance = tbsFinder ? new BigNumber(tbsFinder.balance).shiftedBy(-tbsFinder.token.decimals || 0).toFixed() : 0;
      let TokenDecimals = tbsFinder ? parseFloat(tbsFinder.token.decimals) : 0;
      this.setState({
        decimals: TokenDecimals,
        balance: balance
      });
      return
    }
    if (tokenType === TOKENS_CONFIG.M20.type) {
      const tokenName = list[0];
      const tokenFinder = find(tokens20, t => `${t.token.name}` === `${tokenName}`);
      let balance = tokenFinder && tokenFinder.token ? new BigNumber(tokenFinder.balance).shiftedBy(-tokenFinder.token.decimals || 0).toFixed() : 0;
      let tokenDecimals = tokenFinder && tokenFinder.token ? parseFloat(tokenFinder.token.decimals) : 0;
      this.setState({
        decimals: tokenDecimals,
        balance: balance
      })
    }
    return 0;
  };

  isAmountValid = () => {
    const {amount, balance} = this.state;
    return amount !== 0 && amount !== '' && balance >= amount;
  };

  componentDidMount () {
    this.refreshTokenBalances();
  }

  refreshTokenBalances = () => {
    let {account} = this.props;
    if (account.isLoggedIn) {
      this.props.reloadWallet();
    }
  };

  componentDidUpdate () {
    let {tokenBalances, tokens20, currentWallet} = this.props;
    let {token} = this.state;
    if (!token) {
      tokenBalances = Array.isArray(tokenBalances) ? tokenBalances.filter(tb => tb.balance > 0 && tb.token) : [];
      if (currentWallet && currentWallet.balance > 0) {
        this.setState({
          token: TOKENS_CONFIG.MCASH.type
        }, () => {
          this.getSelectedTokenBalance()
        });
      } else if (tokenBalances.length > 0) {
        this.setState({
          token: tokenBalances[0].token.name + '-' + tokenBalances[0].token.asset_id + '-' + TOKENS_CONFIG.M1.type,
        }, () => {
          this.getSelectedTokenBalance()
        })
      } else if (tokens20.length > 0 && tokenBalances.length === 0) {
        this.setState({
          token: tokens20[0].token.name + '-' + TOKENS_CONFIG.M20.type,
        }, () => {
          this.getSelectedTokenBalance()
        })
      }
    }
  }

  renderFooter () {
    let {sendStatus, isLoading} = this.state;

    if (sendStatus === 'success') {
      return (
        <Alert color="success" className="text-center">
          {tu('successful_send')}
        </Alert>
      )
    }

    if (sendStatus === 'failure') {
      return (
        <Alert color="danger" className="text-center">
          Something went wrong while submitting the transaction
        </Alert>
      )
    }

    return (
      <Fragment>
        {/*<Alert color="warning" className="text-center">*/}
        {/*{tu("address_warning")}*/}
        {/*</Alert>*/}
        <button
          type="button"
          disabled={!this.isValid() || isLoading}
          className="btn btn-primary btn-block btn-lg"
          onClick={this.confirmSend}>{tu('send')}</button>
      </Fragment>
    )
  }

  setMaxAmount = () => {
    let {balance} = this.state;
    this.setState({
      amount: balance
    });
  };

  resetForm = () => {
    this.setState({
      amount: '',
      sendStatus: 'waiting',
      isLoading: false,
      to: '',
    });
  };

  setAddress = (address) => {
    this.setState(prevState => ({
      to: address,
      errors: { ...prevState.errors, to: true }
    }));

    Client.getAddress(address).then(data => {
      this.setState({
        toAccount: data ? data : null,
      });
    })
  };

  setNote = (note) => {
    this.setState({note});
  };

  handleTokenChange = (value) => {
    this.setState({token: value}, () => {
      this.getSelectedTokenBalance();
    });
  }

  render () {
    let {intl, tokenBalances, tokens20, currentWallet} = this.props;
    let {isLoading, sendStatus, modal, to, note, toAccount, token, amount, decimals, errors} = this.state;
    const mcashBalance = currentWallet.balance > 0 ? {
      asset_id: 0,
      balance: currentWallet.balance,
      map_amount: new BigNumber(currentWallet.balance).shiftedBy(-CHAIN_DECIMAL).toFixed(),
      token_name_type: CHAIN_TOKEN,
      token: {
        name: CHAIN_TOKEN,
        decimals: CHAIN_DECIMAL,
        type: TOKENS_CONFIG.MCASH.type
      }
    } : null;
    tokenBalances = _(tokenBalances)
      .filter(tb => tb.balance > 0)
      .filter(tb => tb.token && tb.token.asset_id >= 0)
      .value();
    tokenBalances.map(item => {
      item.token_name_type = item.token.name + '-' + item.token.asset_id + '-' + TOKENS_CONFIG.M1.type;
      item.map_amount = new BigNumber(item.balance).shiftedBy(item.token.decimals ? -item.token.decimals : 0).toFixed();
      return item
    });
    tokens20.map(item => {
      item.token_name_type = item.token.name + '-' + TOKENS_CONFIG.M20.type;
      item.map_amount = new BigNumber(item.balance).shiftedBy(item.token.decimals ? -item.token.decimals : 0).toFixed();
      return item
    });
    let placeholder = '0.000000';
    let num = 0;
    if (decimals || decimals === 0) {
      placeholder = num.toFixed(decimals)
    }

    let isToValid = to.length !== 0 && isAddressValid(to);
    let isAmountValid = this.isAmountValid();

    if (sendStatus === 'success') {
      return (
        <Fragment>
          <div className="alert alert-success text-center">
            {tu('successful_send')}
          </div>
          <div className="justify-content-center">
            <button className="btn btn-primary btn-block" onClick={this.resetForm}>
              {tu('make_another_transaction')}
            </button>
          </div>
        </Fragment>
      )
    }

    return (
      <form>
        {modal}
        {isLoading && <TronLoader/>}
        <div className="form-group">
          <label>{tu('to')}</label>
          <div className="input-group mb-3">
            <input type="text"
                   onChange={(ev) => this.setAddress(ev.target.value)}
                   className={'form-control ' + (errors.to && !isToValid ? 'is-invalid' : '')}
                   value={to}/>
            {
              errors.to && !isToValid && (
                <div className="invalid-feedback">
                  {tu('fill_a_valid_address')}
                </div>
              )
            }
          </div>
        </div>
        {
          (toAccount && toAccount.name) && <Alert color="info">
            <b>{toAccount.name}</b>
          </Alert>
        }
        <div className="form-group">
          <label>{tu('token')}</label>
          <div className="input-group mb-3" style={{height: 36}}>
            <Select
              onChange={this.handleTokenChange}
              value={token}
            >
              {
                mcashBalance && (
                  <Option key={'mcash'} value={mcashBalance.token_name_type}>
                    <span>
                      {`${mcashBalance.token.name} (${mcashBalance.map_amount} ${intl.formatMessage({id: 'available'})})`}
                    </span>
                  </Option>
                )
              }

              <OptGroup label={tu('M1_token')} key="M1">
                {
                  tokenBalances.map((item, index) => (
                    <Option value={item.token_name_type} key={`m1-${index}`}>
                      <span>{item.token && item.token.name}
                        {
                          item.map_token_id !== '_' ? (
                            <span style={{
                              fontSize: 12,
                              color: '#999',
                              margin: '2px 4px 8px'
                            }}>[ID: {item.token && item.token.asset_id}]</span>
                          ) : ''
                        }
                        ({item.map_amount} {intl.formatMessage({id: 'available'})})</span>
                    </Option>
                  ))
                }
              </OptGroup>

              <OptGroup label={tu('M20_token')} key="M20">
                {
                  tokens20.map((item, index) => (
                    <Option value={item.token_name_type} key={`m20-${index}`}>
                      {item.token && item.token.name}&nbsp;({item.map_amount}&nbsp;{intl.formatMessage({id: 'available'})})
                    </Option>
                  ))
                }
              </OptGroup>
            </Select>
          </div>
        </div>
        <div className="form-group">
          <label>{tu('amount')}</label>
          <div className="input-group mb-3">
            <input type="number"
                   onChange={(ev) => this.setAmount(ev.target.value)}
                   className={'form-control ' + (errors.amount && !isAmountValid ? 'is-invalid' : '')}
                   value={amount}
                   placeholder={placeholder}
            />
            <div className="input-group-append">
              <button className="btn btn-outline-secondary"
                      type="button"
                      onClick={this.setMaxAmount}>
                MAX
              </button>
            </div>
            {
              errors.amount && !isAmountValid && (
                <div className="invalid-feedback">
                  {tu('fill_a_valid_number')}
                  {/* tu("insufficient_tokens") */}
                </div>
              )
            }
          </div>
        </div>

        <div className="form-group">
          <label>{tu('note')}</label>
          <div className="input-group mb-3">
            <textarea
              onChange={(ev) => this.setNote(ev.target.value)}
              className={'form-control'}
              value={note}
            />
            <div className="invalid-feedback">
              {tu('fill_a_valid_address')}
              {/* tu("invalid_address") */}
            </div>
          </div>
        </div>
        {this.renderFooter()}
      </form>
    )
  }
}

function mapStateToProps (state) {
  return {
    account: state.app.account,
    wallet: state.app.wallet,
    tokenBalances: state.account.tokens,
    tokens20: state.account.tokens20,
    currentWallet: state.wallet.current
  };
}

const mapDispatchToProps = {
  login,
  reloadWallet,
};

export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(SendForm))
