import { Client } from '../../services/api';
import { tu } from '../../utils/i18n';
import React, { Fragment } from 'react';
import { AddressLink } from '../common/Links';
import { FormattedNumber, injectIntl } from 'react-intl';
import { filter, trim } from 'lodash';
import Countdown from 'react-countdown-now';
import { Sticky, StickyContainer } from 'react-sticky';
import { connect } from 'react-redux';
import { Alert } from 'reactstrap';
import { BarLoader, TronLoader } from '../common/loaders';
import SweetAlert from 'react-bootstrap-sweetalert';
import { login } from '../../actions/app';
import { reloadWallet } from '../../actions/wallet';
import { Link } from 'react-router-dom';
import palette from 'google-palette';
import { Truncate } from '../common/text';
import { withTimers } from '../../utils/timing';
import { transactionResultManager } from '../../utils/mcash';
import { withTronWeb } from '../../utils/mcashWeb';
import { gets } from '../../utils/helper';

function VoteChange ({value, arrow = false}) {
  if (value > 0) {
    return (
      <span className="text-success">
        <span className="mr-1">+{value}</span>
        {arrow && <i className="fa fa-arrow-up"/>}
      </span>
    )
  }

  if (value < 0) {
    return (
      <span className="text-danger">
        <span className="mr-1">{value}</span>
        {arrow && <i className="fa fa-arrow-down"/>}
      </span>
    )
  }

  return (
    <span>-</span>
  )
}

@withTronWeb
@injectIntl
@withTimers
@connect(
  state => ({
    account: state.app.account,
    tokenBalances: state.account.tokens,
    wallet: state.wallet,
    flags: state.app.flags,
    voteList: state.voting.voteList,
    voteTimer: state.voting.voteTimer,
    walletType: state.app.wallet,
    isRightText: state.app.isRightText
  }),
  {
    login,
    reloadWallet
  }
)
export default class VoteOverview extends React.Component {
  _mounted = true

  constructor () {
    super();
    this.state = {
      privateKey: '',
      votingEnabled: false,
      votesSubmitted: false,
      submittingVotes: false,
      loading: false,
      searchCriteria: '',
      modal: null,
      viewStats: false,
      colors: palette('mpn65', 20),
      votesList: {},
      liveVotes: null,
      goSignedIn: false
    };
  }

  getVoteStatus = () => {
    let {wallet} = this.props;
    let stakeAmount = 0
    if (wallet.isOpen) {
      stakeAmount = wallet && wallet.current ? gets(wallet, 'current.stake.stake_amount') : 0
    }
    let voteState = 0
    if (stakeAmount <= 0) {
      voteState = -2
    }
    return {
      stakeAmount,
      voteState
    };
  };

  enableVoting = () => {
    this.setState({
      votingEnabled: true,
    })
  };

  // componentDidUpdate (prevProps) {
  //   if (this.props.account.isLoggedIn && (this.props.account.address !== prevProps.account.address)) {
  //     this.loadCurrentVotes(this.props.account.address);
  //   }
  // }

  async componentDidMount () {
    this._mounted = true
    let {account} = this.props;
    if (account.isLoggedIn) {
      this.props.reloadWallet();
      // this.loadCurrentVotes(account.address);
    }
    await this.loadVotes();
  }

  componentWillUnmount() {
    this._mounted = false
  }

  loadVoteList = async () => {
    let votesList = await Client.getVotesList()
    this.setState({
      votesList
    });
  }

  loadVotes = async () => {
    let {voteList} = this.props;
    if (voteList.length === 0) {
      this.setState({loading: true});
    }
    await this.loadVoteList();
    this.setState({loading: false});

  };

  loadCurrentVotes = async (address) => {
  //   let {items} = await Client.getVoter(address);
  //   if (this._mounted) {
  //     this.setState({
  //       myVotes: items
  //     });
  //   }
  };

  renderVoteStatus () {
    // let {votesAvailable, voteState} = this.getVoteStatus();
    let {voteState} = this.getVoteStatus();

    switch (voteState) {
      // case 0:
      //   return (
      //     <span className="text-success">
      //       {tu('all_votes_are_used_message')}
      //     </span>
      //   );

      // case 1:
      //   return (
      //     <span>
      //       {tu('votes_remaining_message')}:&nbsp;<b><FormattedNumber value={votesAvailable}/></b>
      //     </span>
      //   );
      //
      // case -1:
      //   return (
      //     <span className="text-danger">
      //       {tu('to_much_votes_massage')}
      //     </span>
      //   );

      case -2:
        return (
          <span className="text-danger">
            {tu('need_min_trx_to_vote_message')}
          </span>
        );
      default:
        return;
    }
  }

  onSearchChange = (searchCriteria) => {
    this.setState({
      searchCriteria: trim(searchCriteria),
    });
  };

  renderVotingBar () {
    let {votingEnabled, votesSubmitted, submittingVotes} = this.state;
    let {intl, account} = this.props;
    let {stakeAmount} = this.getVoteStatus();

    if (!account.isLoggedIn) {
      return (
        <div className="text-center">
          {tu('open_wallet_start_voting_message')}
        </div>
      );
    }

    if (votesSubmitted) {
      return (
        <Alert color="success" className="text-center m-0">
          {tu('thanks_submitting_vote_message')}
        </Alert>
      );
    }

    if (votingEnabled && stakeAmount <= 0) {
      return (
        <div className="text-center">
          {/*{tu('warning_votes')}{' '}*/}
          { 'At least 1 MCASH Stake is required to start voting. ' }
          <Link to="/account" className="text-primary">{tu('account_page')}</Link>
        </div>
      );
    }

    if (submittingVotes) {
      return (
        <div className="d-flex justify-content-center p-3" style={{lineHeight: '36px'}}>
          <BarLoader width={160}/>
        </div>
      );
    }
    if (votingEnabled) {
      return (
        <div className="d-flex flex-wrap flex-sm-nowrap" style={{lineHeight: '36px'}}>
          <div className="d-flex">
            <div style={{width: '35px', height: '35px', paddingLeft: '10px'}}>
              <i className="fa fa-search" aria-hidden="true"/>
            </div>
            <input style={{background: '#F3F3F3'}} type="text"
                   className="form-control"
                   placeholder={intl.formatMessage({id: 'search'})}
                   onChange={(ev) => this.onSearchChange(ev.target.value)}/>
          </div>
          <div className="ml-auto">
            {this.renderVoteStatus()}
          </div>
          {/*<div className="ml-2 mt-2 ml-sm-auto mt-sm-0">*/}
            {/*<button className="btn btn-danger ml-auto _cancel" onClick={this.cancelVotes}>{tu('cancel')}</button>*/}
            {/*<button className="btn btn-warning ml-1 _reset" onClick={this.resetVotes}>{tu('reset')}</button>*/}
            {/*<button className="btn btn-success ml-1 _submit" onClick={this.submitVotes}>{tu('submit_votes')}</button>*/}
          {/*</div>*/}
        </div>
      );
    }

    return (
      <div className="text-center">
        <a className="" onClick={this.enableVoting} style={{color: '#C23631'}}>
          {tu('click_to_start_voting')}
        </a>
      </div>
    );
  }

  renderVotingBarFalse () {
    let {account} = this.props;
    let {goSignedIn} = this.state;
    if (!account.isLoggedIn) {
      return (
        <div className="text-center">
          {
            goSignedIn ? <span style={{color: '#333333'}}>
                        {tu('not_signed_in')}
                    </span> : <a href="javascript:" onClick={this.notSignedIn}>
              {tu('click_to_start_voting')}
            </a>
          }
        </div>
      );
    }
  }

  notSignedIn = () => {
    this.setState({
      goSignedIn: true
    });
  }

  // resetVotes = () => {
  //   this.setState({
  //     myVotes: [],
  //   });
  // };

  cancelVotes = () => {
    this.loadCurrentVotes(this.props.account.address);
    this.setState({
      votingEnabled: false,
      searchCriteria: '',
    });
  };

  hideModal = () => {
    this.setState({
      modal: null,
    });
  };

  submitVote = async (voteAdress) => {
    let {account} = this.props;
    let res;
    this.setState({submittingVotes: true});
    const tronWebLedger = this.props.mcashWeb();
    const {mcashWeb} = this.props.account;
    if (this.props.walletType.type === 'ACCOUNT_LEDGER') {
      try {
        const unSignTransaction = await tronWebLedger.transactionBuilder.vote(voteAdress, account.address).catch(e => false);
        const {result} = await transactionResultManager(unSignTransaction, tronWebLedger);
        res = result;
      } catch (e) {
        console.error('error', e)
      }
    } else if (this.props.walletType.type === 'ACCOUNT_PRIVATE_KEY') {
      // let {success} = await Client.voteForWitnesses(account.address, witnessVotes)(account.key);
      // res = success
    } else if (this.props.walletType.type === 'ACCOUNT_MCASHLINK') {
      try {
        const unSignTransaction = await mcashWeb.transactionBuilder.vote(voteAdress, account.address).catch(e => false);
        const {result} = await transactionResultManager(unSignTransaction, mcashWeb);
        res = result;
      } catch (e) {
        console.error(e)
      }
    }
    if (res) {
      setTimeout(() => this.props.reloadWallet(), 1200);
      setTimeout(() => this.setState({votesSubmitted: false}, () => {this.loadVoteList()}), 5000);

      this.setState({
        votesSubmitted: true,
        submittingVotes: false,
        votingEnabled: false,
        modal: (
          <SweetAlert success title={tu('submissing_vote_message_title')} onConfirm={this.hideModal}>
            {tu('submissing_vote_message_0')}<br/>
            {tu('submissing_vote_message_1')}
          </SweetAlert>
        )
      });
    } else {
      this.setState({
        votesSubmitted: false,
        submittingVotes: false,
        votingEnabled: false,
        modal: (
          <SweetAlert danger title={tu('error')} onConfirm={this.hideModal}>
            {tu('submitting_vote_error_message')}
          </SweetAlert>
        )
      });
    }
    if (account.address) {
      setTimeout(() => {
        this.loadCurrentVotes(account.address)
      }, 5000)
    }
  }

  getNextCycle () {
    let {voteTimer} = this.props;
    return voteTimer;
  }

  renderVoteButton(voteAddress) {
    const {wallet} = this.props
    const vote = wallet ? gets(wallet, 'current.vote') : null
    if (vote && vote.vote_address === voteAddress) {
      return (
        <button
          style={{width: '80px'}}
          className='btn btn-sm btn-danger'
          onClick={() => {}}
        >Voted</button>
      )
    }
    return (
      <button
        style={{width: '80px'}}
        className='btn btn-sm btn-outline-success'
        onClick={() => this.submitVote(voteAddress)}
      >Vote</button>
    )
  }

  render () {
    let {votingEnabled, votesList, loading, modal, viewStats, colors, searchCriteria} = this.state;
    let {wallet, isRightText} = this.props;
    let candidates = votesList.witnesses || []

    let filteredCandidates = candidates.map((v, i) => Object.assign({
      rank: i
    }, v || {}));

    if (searchCriteria !== '') {
      filteredCandidates = filter(candidates, c => {
        if (trim(c.url.toLowerCase()).indexOf(searchCriteria.toLowerCase()) !== -1) {
          return true;
        }
        if (c.name.length > 0 && trim(c.name.toLowerCase()).indexOf(searchCriteria.toLowerCase()) !== -1) {
          return true;
        }
        return false;
      });
    }

    let totalVotes = votesList.total_votes || 0;

    let biggestGainer = votesList.fastest_rise || {}
    let {stakeAmount} = this.getVoteStatus();
    const voteTimer = votesList.next_maintenance_time ? votesList.next_maintenance_time : 0;

    return (
      <main className="container header-overlap _voteOverview">
        {modal}
        <div className="row _badge">
          <div className="col-md-4 mt-3 mt-md-0">
            <div className="card h-100 text-left widget-icon bg-line_red bg-image_nextRound">
              <div className="card-body">
                <h3 className="text-primary">
                  <Countdown date={voteTimer} daysInHours={true} onComplete={() => {
                    this.loadVotes();
                  }}/>
                </h3>
                {tu('next_round')}
              </div>
            </div>
          </div>

          <div className="col-md-4 mt-3 mt-md-0 position-relative">
            <div className="card h-100 widget-icon bg-line_green bg-image_totalVotes">
              <div className="card-body text-left">
                <h3 className="text-secondary">
                  <FormattedNumber value={totalVotes}/>
                </h3>
                {/*
                  <a href="javascript:"
                     onClick={() => this.setState(state => ({viewStats: !state.viewStats}))}>{tu("total_votes")}</a>
                */}
                {tu('total_votes')}
              </div>
            </div>
          </div>

          <div className="col-md-4 mt-3 mt-md-0">
            <div className="card h-100 widget-icon bg-line_yellow bg-image_mostRank">
              <div className="card-body text-left">
                <h3 className="text-success">
                  <VoteChange value={biggestGainer.change_cycle} arrow={true}/>
                </h3>
                <div className={(isRightText ? 'flex-row-reverse justify-content-end' : '') + ' d-flex'}>
                  <div className="_ranks mr-2" style={{whiteSpace: 'nowrap'}}>{tu('most_ranks')}</div>
                  {/* <div>--</div> */}
                  <div className="" style={isRightText ? {maxWidth: '110px'} : {}}>
                    {
                      Math.abs(biggestGainer.change_cycle) ? (
                        <AddressLink address={biggestGainer.address}>
                          {biggestGainer.name || biggestGainer.url}
                        </AddressLink>
                      ) : <span style={{color: '#999999'}}>-</span>
                    }
                  </div>
                </div>
              </div>
            </div>
          </div>
        </div>

        {
          loading ? (
              <div className="card mt-2">
                <TronLoader>
                  {tu('loading_super_representatives')}
                </TronLoader>
              </div>
            ) :
            <div className="row mt-2">
              <div className="col-md-12">
                <StickyContainer>
                  <div className="card mt-1">
                    {
                      wallet.isOpen ? (
                        <Sticky>
                          {
                            ({style}) => (
                              <div style={{borderBottom: '1px solid #D8D8D8', zIndex: 100, ...style}}
                                   className="card-body bg-white p-3">
                                {this.renderVotingBar()}
                              </div>
                            )
                          }
                        </Sticky>
                      ) : (
                        <Sticky>
                          {
                            ({style}) => (
                              <div style={{borderBottom: '1px solid #D8D8D8', zIndex: 100, ...style}}
                                   className="card-body bg-white p-3">
                                {this.renderVotingBarFalse()}
                              </div>
                            )
                          }
                        </Sticky>
                      )
                    }

                    <div className="table-responsive table-scroll">
                      <table className="table vote-table table-hover m-0">
                        <thead className="thead-light">
                        <tr>
                          <th className="text-center" style={{width: 50}}>#</th>
                          <th>{tu('name')}</th>

                          <th className="text-right" style={{width: 150}}>{tu('votes')}</th>
                          <th className="text-right" style={{width: 150}}>{tu('live')}</th>
                          <th className="text-right" style={{width: 100}}>{tu('percentage')}</th>
                          {
                            votingEnabled && stakeAmount > 0 && (
                              <th className="text-center" style={{width: 150}}>
                                {tu('your_vote')}
                              </th>
                            )
                          }
                        </tr>
                        </thead>
                        <tbody>
                        {
                          (searchCriteria.length > 0 && filteredCandidates.length === 0) &&
                          <tr>
                            <td colSpan="6" className="p-3 text-center">
                              No Super Nodes found for <b>{searchCriteria}</b>
                            </td>
                          </tr>
                        }
                        {
                          filteredCandidates.map(candidate => {
                              return (
                                <tr key={candidate.address}>
                                  {
                                    viewStats ?
                                      <th className="font-weight-bold pt-4 text-center"
                                          style={{backgroundColor: '#' + colors[candidate.rank]}}>
                                        {candidate.last_ranking}
                                      </th> :
                                      <th className="font-weight-bold pt-4 text-center">
                                        {candidate.last_ranking}
                                      </th>
                                  }
                                  <td className="d-flex flex-row ">
                                    <div className="text-center text-sm-left" style={{minWidth: '150px'}}>
                                      <Truncate>
                                        <AddressLink address={candidate.address}
                                                     className="font-weight-bold">{candidate.name || candidate.url}</AddressLink>
                                      </Truncate>
                                      <AddressLink className="small text-muted" address={candidate.address}/>
                                    </div>
                                    {
                                      candidate.hasPage && <div className="_team ml-0 ml-sm-auto">
                                        <Link className="btn btn-lg btn-block btn-default mt-1"
                                              to={`/representative/${candidate.address}`}>
                                          {tu('open_team_page')}
                                          <i className="fas fa-users ml-2"/>
                                        </Link>
                                      </div>
                                    }
                                  </td>
                                  <td className="small text-right align-middle">
                                    {
                                      totalVotes > 0 &&
                                      <Fragment>
                                        <FormattedNumber value={candidate.last_cycle_votes}/><br/>
                                      </Fragment>
                                    }
                                  </td>
                                  <td className="small text-right align-middle _liveVotes">
                                    {
                                      totalVotes > 0 &&
                                      <Fragment>
                                        <FormattedNumber value={candidate.real_time_votes}/><br/>
                                        {/* <span className={candidate.changeVotes > 0
                                                    ? 'color-green'
                                                    : 'color-red'}>
                                                  </span> */}
                                        {(candidate.change_votes > 0) ?
                                          <span className="color-green">+<FormattedNumber
                                            value={candidate.change_votes}/></span>
                                          : <span className="color-red"><FormattedNumber
                                            value={candidate.change_votes}/></span>
                                        }
                                      </Fragment>
                                    }
                                  </td>
                                  <td className="small text-right align-middle">
                                    {
                                      totalVotes > 0 &&
                                      <Fragment>
                                        <FormattedNumber value={candidate.votes_percentage}
                                                         minimumFractionDigits={2}
                                                         maximumFractionDigits={2}
                                        />%
                                      </Fragment>
                                    }
                                  </td>
                                  {
                                    votingEnabled && stakeAmount > 0 && (
                                      <td className={'text-center'}>
                                        {this.renderVoteButton(candidate.address)}
                                      </td>
                                    )
                                  }
                                </tr>
                              )
                            }
                          )
                        }
                        </tbody>
                      </table>
                    </div>
                  </div>
                </StickyContainer>
              </div>
            </div>
        }

      </main>
    );
  }
}
