import { Client as ApiClient } from "mcashscan";
import io from "socket.io-client";
import MCashWeb from "mcashweb";
import xhr from "axios/index";
import { API_URL } from "../constants.js";
// import { setLoginWithAddress } from "../actions/app.js";

const ServerNode = "https://api.trongrid.io";
const HttpProvider = MCashWeb.providers.HttpProvider; // This provider is optional, you can just use a url for the nodes instead
const fullNode = new HttpProvider(ServerNode); // Full node http endpoint
const solidityNode = new HttpProvider(ServerNode); // Solidity node http endpoint
const eventServer = ServerNode; // Contract events http endpoint
export const mcashWeb = new MCashWeb(fullNode, solidityNode, eventServer);

export const Client = new ApiClient();

export function buildClient(account) {
  return new ApiClient();
}

export function channel(path, options) {
  // return io(process.env.API_URL + path, options);
  return io("https://wlcyapi.tronscan.org/socket.io" + path, options);
  //return io('http://172.16.20.207:20110/socket.io' + path, options);
}

class ApiClient20 {
  constructor() {
    this.apiUrl = "https://api.trx.market";
    // this.apiUrl = "http://52.15.171.70:21111";
    this.ZDUrl = "https://tron274.zendesk.com";
    this.SCANUrl = "https://tronscanorg.zendesk.com";
  }

  async getexchanges20(options = {}) {
    let { data } = await xhr.get(
      `${this.apiUrl}/api/exchange/marketPair/list`,
      {
        params: options
      }
    );
    return data;
  }

  async getRegisterList(id) {
    let { data } = await xhr.get(
      `${this.apiUrl}/api/exchange/common/orderListV2/${id}`
    );

    return data;
  }

  async getCurrentList(query) {
    let { data } = await xhr.get(
      `${this.apiUrl}/api/exchange/user/order`,
      // `http://13.58.63.31:21110/api/exchange/user/order`,
      {
        params: query
      }
    );

    return data;
  }

  async getTransactionList(query) {
    let { data } = await xhr.get(
      `${this.apiUrl}/api/exchange/common/latestOrders`,
      {
        params: query
      }
    );
    return data;
  }
  async getExchangeCalc(query) {
    let { data } = await xhr.get(
      `https://wlcyapi.tronscan.org/api/exchange/calc`,
      {
        params: query
      }
    );
    return data;
  }

  async getCurrentPrice(id) {
    let { data } = await xhr.get(
      `${this.apiUrl}/api/exchange/toppriceV2/${id}`
    );
    return data;
  }

  async getKlineData20(query) {
    let { data } = await xhr.get(`${this.apiUrl}/api/exchange/kchart`, {
      params: query
    });
    return data;
  }

  async gettokenInfo20(query) {
    let { data } = await xhr.get(`${API_URL}/api/token_trc20`, {
      params: {
        sort: "issue_time",
        start: 0,
        limit: 100,
        ...query
      }
    });
    return data;
  }

  async getTokenInfoItem(contract, type) {
    // https://apilist.tronscan.org/api/token_trc20?contract=TR7NHqjeKQxGTCi8q8ZY4pL8otSzgjLj6t
    let params = {};
    let url = `${API_URL}/api/token_trc20`;
    switch (type) {
      case 1:
        params = {};
        url = `${API_URL}/api/tokens/${contract}`;
        break;
      case 2:
        params = {
          contract_address: contract
        };
        break;
      case 4:
        params = {};
        url = `${API_URL}/api/tokens/${contract}`;
        break;
      case 3:
        params = {
          contract_address: contract
        };
        break;
      default:
        break;
    }
    let { data } = await xhr.get(url, {
      params: params
    });
    return data;
  }

  async getExchanges20SearchList(options = {}) {
    let { data } = await xhr.get(
      // `http://52.15.171.70:21110/api/exchange/marketPair/searchList`,
      `${this.apiUrl}/api/exchange/marketPair/searchList`,
      {
        params: options
      }
    );
    return data;
  }

  /**
   * 添加订单渠道ID
   */
  async addChannelId(query, headers) {
    let { data } = await xhr({
      method: "post",
      url: `${this.apiUrl}/api/exchange/user/chanIdAdd`,
      // url:'http://172.16.21.24:21110/api/exchange/user/chanIdAdd',
      data: query,
      headers: headers
    });
    return data;
  }

  /**
   * 深度图
   */
  async depthChart(id) {
    let { data } = await xhr({
      method: "get",
      url: `${this.apiUrl}/api/exchange/common/deepgraph/${id}`
      // url: `http://18.222.178.103:3006/dex/exchange/common/deepgraph/${id}`
    });
    return data;
  }

  /**
   * 获取zendesk消息系统
   */
  async getNotice(lan, query) {
    let language = "en-us";
    lan === "zh" ? (language = "zh-cn") : (language = "en-us");
    let { data } = await xhr({
      method: "get",
      url: `${
        this.ZDUrl
      }/api/v2/help_center/${language}/categories/360001523732/articles.json?sort_by=created_at&sort_order=desc&per_page=${
        query.page
      }`
    });
    return data;
  }

  /**
   * 获取zendesk TRONSCAN消息系统
   */
  async getTRONNotice(lan, query) {
    let language = "en-us";
    let id = "360001621692";
    lan === "zh" ? (language = "zh-cn") : (language = "en-us");
    lan === "zh" ? (id = "360001618172") : (id = "360001621692");
    let { data } = await xhr({
      method: "get",
      url: `${
        this.SCANUrl
      }/api/v2/help_center/${language}/categories/${id}/articles.json?sort_by=created_at&sort_order=desc&per_page=${
        query.page
      }`
    });
    return data;
  }

  //币之间的换算
  async coinMarketCap(secend_token, covert) {
    let type = "tronix";
    if (secend_token === "usdt") {
      type = "tether";
    }
    let { data } = await xhr({
      method: "get",
      url: `https://api.coinmarketcap.com/v1/ticker/${type}/?convert=${covert}`
    });
    return data;
  }

  //market新接口，sortType=0 24小时交易额 sortType=1 热门 sortType=2 涨跌幅
  async getMarketNew(sortType) {
    let { data } = await xhr({
      method: "get",
      url: `${this.apiUrl}/api/exchange/marketPair/list?sortType=${sortType}`
    });
    return data;
  }
}

export const Client20 = new ApiClient20();
